

dnl
dnl XML_HOOK (script-if-xml-found, failflag)
dnl
dnl If failflag is "failure", script aborts due to lack of XML
dnl 
dnl Check for availability of the libxml library
dnl the XML parser uses libz if available too
dnl

AC_DEFUN([XML_HOOK],[
AC_ARG_WITH(xml-prefix,[  --with-xml-prefix=PFX   Prefix where libxml is installed (optional)],
            xml_prefix="$withval", xml_prefix="")
AC_ARG_WITH(xml-exec-prefix,[  --with-xml-exec-prefix=PFX Exec prefix where libxml is installed (optional)],
            xml_exec_prefix="$withval", xml_exec_prefix="")    
  if test x$xml_exec_prefix != x ; then
     xml_args="$xml_args --exec-prefix=$xml_exec_prefix"
     if test x${XML_CONFIG+set} != xset ; then
        XML_CONFIG=$xml_exec_prefix/bin/xml-config
     fi
  fi
  if test x$xml_prefix != x ; then
     xml_args="$xml_args --prefix=$xml_prefix"
     if test x${XML_CONFIG+set} != xset ; then
        XML_CONFIG=$xml_prefix/bin/xml-config
     fi
  fi
	AC_PATH_PROG(XML_CONFIG,xml-config,no)
	if test "$XML_CONFIG" = no; then
		if test x$2 = xfailure; then
			AC_MSG_ERROR(Could not find xml-config)
		fi
	fi

	AC_CHECK_LIB(xml, xmlNewDoc, [
		$1
		XML_CFLAGS=`$XML_CONFIG --cflags`
		XML_LIBS=`$XML_CONFIG --libs`
	], [
dnl		if test x$2 = xfailure; then 
dnl			AC_MSG_ERROR(Could not link sample xml program)
dnl		fi
		echo $ac_n "cross compiling; assumed OK... $ac_c"
		XML_CFLAGS=`$XML_CONFIG --cflags`
		XML_LIBS=`$XML_CONFIG --libs`		
	], `$XML_CONFIG --libs`)

	AC_SUBST(XML_CFLAGS)
	AC_SUBST(XML_LIBS)
])

AC_DEFUN([XML_CHECK], [
	XML_HOOK([],failure)
])


dnl AM_PATH_UTA([MINIMUM-VERSION, [ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND]]])
dnl Test for UTA, and define UTA_CFLAGS and UTA_LIBS
dnl
AC_DEFUN(AM_PATH_UTA,
[dnl
dnl Get the cflags and libraries from the uta-config script
dnl
AC_ARG_WITH(uta-prefix,[  --with-uta-prefix=PFX   Prefix where libuta is installed (optional)],
            uta_prefix="$withval", uta_prefix="")
AC_ARG_WITH(uta-exec-prefix,[  --with-uta-exec-prefix=PFX Exec prefix where libuta is installed (optional)],
            uta_exec_prefix="$withval", uta_exec_prefix="")    
  if test x$uta_exec_prefix != x ; then
     uta_args="$uta_args --exec-prefix=$uta_exec_prefix"
     if test x${UTA_CONFIG+set} != xset ; then
        UTA_CONFIG=$uta_exec_prefix/bin/uta-config
     fi
  fi
  if test x$uta_prefix != x ; then
     uta_args="$uta_args --prefix=$uta_prefix"
     if test x${UTA_CONFIG+set} != xset ; then
        UTA_CONFIG=$uta_prefix/bin/uta-config
     fi
  fi
 
  AC_PATH_PROG(UTA_CONFIG, uta-config, no)
  min_uta_version=ifelse([$1], ,0.3.32,$1)
  AC_MSG_CHECKING(for libuta - version >= $min_uta_version)
  no_uta=""
  if test "$UTA_CONFIG" = "no" ; then
    no_uta=yes
    AC_MSG_RESULT(no)
    ifelse([$3], , :, [$3])
  else
    UTA_CFLAGS=`$UTA_CONFIG $utaconf_args --cflags`
    UTA_LIBS=`$UTA_CONFIG $utaconf_args --libs`
 
    uta_major_version=`$UTA_CONFIG $uta_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\1/'`
    uta_minor_version=`$UTA_CONFIG $uta_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\2/'`
    uta_micro_version=`$UTA_CONFIG $uta_config_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\3/'`

    ac_save_CFLAGS="$CFLAGS"
    ac_save_LIBS="$LIBS"
    CFLAGS="$CFLAGS $UTA_CFLAGS"
    LIBS="$LIBS $UTA_LIBS"

    rm -f conf.utatest
    AC_TRY_RUN([
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

char*
my_strdup (char *str)
{
  char *new_str;
  
  if (str)
    {
      new_str = (char *)malloc ((strlen (str) + 1) * sizeof(char));
      strcpy (new_str, str);
    }
  else
    new_str = NULL;
  
  return new_str;
}

int main ()
{
  int major, minor, micro;
  char *tmp_version;

  system ("touch conf.utatest");

  /* HP/UX 9 (%@#!) writes to sscanf strings */
  tmp_version = my_strdup("$min_uta_version");
  if (sscanf(tmp_version, "%d.%d.%d", &major, &minor, &micro) != 3) {
     printf("%s, bad version string\n", "$min_uta_version");
     exit(1);
   }

   if (($uta_major_version > major) ||
      (($uta_major_version == major) && ($uta_minor_version > minor)) ||
      (($uta_major_version == major) && ($uta_minor_version == minor) && ($uta_micro_version >= micro)))
    {
      return 0;
    }
  else
    {
      printf("\n*** 'uta-config --version' returned %d.%d.%d, but the minimum version\n", $uta_major_version, $uta_minor_version, $uta_micro_version);
      printf("*** of libuta required is %d.%d.%d.\n", major, minor, micro);
      printf("*** If uta-config was wrong, set the environment variable UTA_CONFIG\n");
      printf("*** to point to the correct copy of uta-config, and remove the file\n");
      printf("*** config.cache before re-running configure\n");
      return 1;
    }
}

],,no_uta=yes,[echo $ac_n "cross compiling; assumed OK... $ac_c"])
       
    CFLAGS="$ac_save_CFLAGS"
    LIBS="$ac_save_LIBS"
  
    if test "x$no_uta" = x ; then
      AC_MSG_RESULT(yes)	
      ifelse([$2], , :, [$2])
    else
      AC_MSG_RESULT(no)
      ifelse([$3], , :, [$3])	
    fi
  fi
  AC_SUBST(UTA_CFLAGS)
  AC_SUBST(UTA_LIBS)
 
]) 

dnl Configure paths for Atlas
dnl Adapted by Al Riddoch
dnl stolen from Sam Lantinga of SDL
dnl stolen from Manish Singh
dnl stolen back from Frank Belew
dnl stolen from Manish Singh
dnl Shamelessly stolen from Owen Taylor

dnl AM_PATH_ATLAS([MINIMUM-VERSION, [ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND]]])
dnl Test for Atlas, and define ATLAS_CFLAGS and ATLAS_LIBS
dnl
AC_DEFUN(AM_PATH_ATLAS,
[dnl 
dnl Get the cflags and libraries from the atlas-config script
dnl
AC_LANG_CPLUSPLUS
AC_ARG_WITH(atlas-prefix,[  --with-atlas-prefix=PFX   Prefix where Atlas is installed (optional)],
            atlas_prefix="$withval", atlas_prefix="")
AC_ARG_WITH(atlas-exec-prefix,[  --with-atlas-exec-prefix=PFX Exec prefix where Atlas is installed (optional)],
            atlas_exec_prefix="$withval", atlas_exec_prefix="")
AC_ARG_ENABLE(atlastest, [  --disable-atlastest       Do not try to compile and run a test Atlas program],
		    , enable_atlastest=yes)

  if test x$atlas_exec_prefix != x ; then
     atlas_args="$atlas_args --exec-prefix=$atlas_exec_prefix"
     if test x${ATLAS_CONFIG+set} != xset ; then
        ATLAS_CONFIG=$atlas_exec_prefix/bin/atlas-config
     fi
  fi
  if test x$atlas_prefix != x ; then
     atlas_args="$atlas_args --prefix=$atlas_prefix"
     if test x${ATLAS_CONFIG+set} != xset ; then
        ATLAS_CONFIG=$atlas_prefix/bin/atlas-config
     fi
  fi

  AC_PATH_PROG(ATLAS_CONFIG, atlas-config, no)
  min_atlas_version=ifelse([$1], ,0.3.11,$1)
  AC_MSG_CHECKING(for Atlas - version >= $min_atlas_version)
  no_atlas=""
  if test "$ATLAS_CONFIG" = "no" ; then
    no_atlas=yes
  else
    ATLAS_CFLAGS=`$ATLAS_CONFIG $atlasconf_args --cflags`
    ATLAS_LIBS=`$ATLAS_CONFIG $atlasconf_args --libs`

    atlas_major_version=`$ATLAS_CONFIG $atlas_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\1/'`
    atlas_minor_version=`$ATLAS_CONFIG $atlas_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\2/'`
    atlas_micro_version=`$ATLAS_CONFIG $atlas_config_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\3/'`
    if test "x$enable_atlastest" = "xyes" ; then
      ac_save_CFLAGS="$CFLAGS"
      ac_save_LIBS="$LIBS"
      CFLAGS="$CFLAGS $ATLAS_CFLAGS"
      LIBS="$LIBS $ATLAS_LIBS"
dnl
dnl Now check if the installed Atlas is sufficiently new. (Also sanity
dnl checks the results of atlas-config to some extent)
dnl
      rm -f conf.atlastest
      AC_TRY_RUN([
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

char*
my_strdup (char *str)
{
  char *new_str;
  
  if (str)
    {
      new_str = (char *)malloc ((strlen (str) + 1) * sizeof(char));
      strcpy (new_str, str);
    }
  else
    new_str = NULL;
  
  return new_str;
}

int main (int argc, char *argv[])
{
  int major, minor, micro;
  char *tmp_version;

  /* This hangs on some systems (?)
  system ("touch conf.atlastest");
  */
  { FILE *fp = fopen("conf.atlastest", "a"); if ( fp ) fclose(fp); }

  /* HP/UX 9 (%@#!) writes to sscanf strings */
  tmp_version = my_strdup("$min_atlas_version");
  if (sscanf(tmp_version, "%d.%d.%d", &major, &minor, &micro) != 3) {
     printf("%s, bad version string\n", "$min_atlas_version");
     exit(1);
   }

   if (($atlas_major_version > major) ||
      (($atlas_major_version == major) && ($atlas_minor_version > minor)) ||
      (($atlas_major_version == major) && ($atlas_minor_version == minor) && ($atlas_micro_version >= micro)))
    {
      return 0;
    }
  else
    {
      printf("\n*** 'atlas-config --version' returned %d.%d.%d, but the minimum version\n", $atlas_major_version, $atlas_minor_version, $atlas_micro_version);
      printf("*** of Atlas required is %d.%d.%d. If atlas-config is correct, then it is\n", major, minor, micro);
      printf("*** best to upgrade to the required version.\n");
      printf("*** If atlas-config was wrong, set the environment variable ATLAS_CONFIG\n");
      printf("*** to point to the correct copy of atlas-config, and remove the file\n");
      printf("*** config.cache before re-running configure\n");
      return 1;
    }
}

],, no_atlas=yes,[echo $ac_n "cross compiling; assumed OK... $ac_c"])
       CFLAGS="$ac_save_CFLAGS"
       LIBS="$ac_save_LIBS"
     fi
  fi
  if test "x$no_atlas" = x ; then
     AC_MSG_RESULT(yes)
     ifelse([$2], , :, [$2])     
  else
     AC_MSG_RESULT(no)
     if test "$ATLAS_CONFIG" = "no" ; then
       echo "*** The atlas-config script installed by Atlas could not be found"
       echo "*** If Atlas was installed in PREFIX, make sure PREFIX/bin is in"
       echo "*** your path, or set the ATLAS_CONFIG environment variable to the"
       echo "*** full path to atlas-config."
     else
       if test -f conf.atlastest ; then
        :
       else
          echo "*** Could not run Atlas test program, checking why..."
          CFLAGS="$CFLAGS $ATLAS_CFLAGS"
          LIBS="$LIBS $ATLAS_LIBS"
          AC_TRY_LINK([
#include <stdio.h>
],      [ return 0; ],
        [ echo "*** The test program compiled, but did not run. This usually means"
          echo "*** that the run-time linker is not finding Atlas or finding the wrong"
          echo "*** version of Atlas. If it is not finding Atlas, you'll need to set your"
          echo "*** LD_LIBRARY_PATH environment variable, or edit /etc/ld.so.conf to point"
          echo "*** to the installed location  Also, make sure you have run ldconfig if that"
          echo "*** is required on your system"
	  echo "***"
          echo "*** If you have an old version installed, it is best to remove it, although"
          echo "*** you may also be able to get things to work by modifying LD_LIBRARY_PATH"],
        [ echo "*** The test program failed to compile or link. See the file config.log for the"
          echo "*** exact error that occured. This usually means Atlas was incorrectly installed"
          echo "*** or that you have moved Atlas since it was installed. In the latter case, you"
          echo "*** may want to edit the atlas-config script: $ATLAS_CONFIG" ])
          CFLAGS="$ac_save_CFLAGS"
          LIBS="$ac_save_LIBS"
       fi
     fi
     ATLAS_CFLAGS=""
     ATLAS_LIBS=""
     ifelse([$3], , :, [$3])
  fi
  AC_SUBST(ATLAS_CFLAGS)
  AC_SUBST(ATLAS_LIBS)
  rm -f conf.atlastest
])

dnl COAL library checks 

dnl #################### BEGIN COAL DISCOVERY MESS ########################

AC_DEFUN(AM_PATH_COAL,
[
AC_ARG_WITH(coal-prefix,[  --with-coal-prefix=PREFIX
                          Prefix where Coal is installed (optional)],
            coal_prefix="$withval", coal_prefix="")
AC_ARG_WITH(coal-exec-prefix,[  --with-coal-exec-prefix=PREFIX
                          Exec prefix where Coal is installed (optional)],
            coal_exec_prefix="$withval", coal_exec_prefix="")
AC_ARG_ENABLE(coaltest, [  --disable-coaltest     Do not try to compile and run a test Coal program],
		    , enable_coaltest=yes)

  if test x$coal_exec_prefix != x ; then
     coal_args="$coal_args --exec-prefix=$coal_exec_prefix"
     if test x${COAL_CONFIG+set} != xset ; then
        COAL_CONFIG=$coal_exec_prefix/bin/coal-config
     fi
  fi
  if test x$coal_prefix != x ; then
     coal_args="$coal_args --prefix=$coal_prefix"
     if test x${COAL_CONFIG+set} != xset ; then
        COAL_CONFIG=$coal_prefix/bin/coal-config
     fi
  fi

  AC_PATH_PROG(COAL_CONFIG, coal-config, no)
  min_coal_version=ifelse([$1], ,0.2.4,$1)
  AC_MSG_CHECKING(for Coal - version >= $min_coal_version)
  no_coal=""
  if test "$COAL_CONFIG" = "no" ; then
    no_coal=yes
  else
    COAL_CFLAGS=`$COAL_CONFIG $coalconf_args --cflags`
    COAL_LIBS=`$COAL_CONFIG $coalconf_args --libs`

    coal_major_version=`$COAL_CONFIG $coal_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\1/'`
    coal_minor_version=`$COAL_CONFIG $coal_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\2/'`
    coal_micro_version=`$COAL_CONFIG $coal_config_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\3/'`
    if test "x$enable_coaltest" = "xyes" ; then
      ac_save_CFLAGS="$CFLAGS"
      ac_save_LIBS="$LIBS"
      CFLAGS="$CFLAGS $COAL_CFLAGS"
      LIBS="$LIBS $COAL_LIBS"
dnl
dnl Now check if the installed Coal is sufficiently new. (Also sanity
dnl checks the results of coal-config to some extent)
dnl
      rm -f conf.coaltest
      AC_TRY_RUN([
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

char*
my_strdup (char *str)
{
  char *new_str;
  
  if (str)
    {
      new_str = (char *)malloc ((strlen (str) + 1) * sizeof(char));
      strcpy (new_str, str);
    }
  else
    new_str = NULL;
  
  return new_str;
}

int main (int argc, char *argv[])
{
  int major, minor, micro;
  char *tmp_version;

  /* This hangs on some systems (?)
  system ("touch conf.coaltest");
  */
  { FILE *fp = fopen("conf.coaltest", "a"); if ( fp ) fclose(fp); }

  /* HP/UX 9 (%@#!) writes to sscanf strings */
  tmp_version = my_strdup("$min_coal_version");
  if (sscanf(tmp_version, "%d.%d.%d", &major, &minor, &micro) != 3) {
     printf("%s, bad version string\n", "$min_coal_version");
     exit(1);
   }

   if (($coal_major_version > major) ||
      (($coal_major_version == major) && ($coal_minor_version > minor)) ||
      (($coal_major_version == major) && ($coal_minor_version == minor) && ($coal_micro_version >= micro)))
    {
      return 0;
    }
  else
    {
      printf("\n*** 'coal-config --version' returned %d.%d.%d, but the minimum version\n", $coal_major_version, $coal_minor_version, $coal_micro_version);
      printf("*** of Coal required is %d.%d.%d. If coal-config is correct, then it is\n", major, minor, micro);
      printf("*** best to upgrade to the required version.\n");
      printf("*** If coal-config was wrong, set the environment variable COAL_CONFIG\n");
      printf("*** to point to the correct copy of coal-config, and remove the file\n");
      printf("*** config.cache before re-running configure\n");
      return 1;
    }
}

],, no_coal=yes,[echo $ac_n "cross compiling; assumed OK... $ac_c"])
       CFLAGS="$ac_save_CFLAGS"
       LIBS="$ac_save_LIBS"
     fi
  fi
  if test "x$no_coal" = x ; then
     AC_MSG_RESULT(yes)
     ifelse([$2], , :, [$2])     
  else
     AC_MSG_RESULT(no)
     if test "$COAL_CONFIG" = "no" ; then
       echo "*** The coal-config script installed by Coal could not be found"
       echo "*** If Coal was installed in PREFIX, make sure PREFIX/bin is in"
       echo "*** your path, or set the COAL_CONFIG environment variable to the"
       echo "*** full path to coal-config."
     else
       if test -f conf.coaltest ; then
        :
       else
          echo "*** Could not run Coal test program, checking why..."
          CFLAGS="$CFLAGS $COAL_CFLAGS"
          LIBS="$LIBS $COAL_LIBS"
          AC_TRY_LINK([
#include <stdio.h>
],      [ return 0; ],
        [ echo "*** The test program compiled, but did not run. This usually means"
          echo "*** that the run-time linker is not finding Coal or finding the wrong"
          echo "*** version of Coal. If it is not finding Coal, you'll need to set your"
          echo "*** LD_LIBRARY_PATH environment variable, or edit /etc/ld.so.conf to point"
          echo "*** to the installed location  Also, make sure you have run ldconfig if that"
          echo "*** is required on your system"
	  echo "***"
          echo "*** If you have an old version installed, it is best to remove it, although"
          echo "*** you may also be able to get things to work by modifying LD_LIBRARY_PATH"],
        [ echo "*** The test program failed to compile or link. See the file config.log for the"
          echo "*** exact error that occured. This usually means Coal was incorrectly installed"
          echo "*** or that you have moved Coal since it was installed. In the latter case, you"
          echo "*** may want to edit the coal-config script: $COAL_CONFIG" ])
          CFLAGS="$ac_save_CFLAGS"
          LIBS="$ac_save_LIBS"
       fi
     fi
     COAL_CFLAGS=""
     COAL_LIBS=""
     ifelse([$3], , :, [$3])
  fi
  AC_SUBST(COAL_CFLAGS)
  AC_SUBST(COAL_LIBS)
  rm -f conf.coaltest

])

dnl #################### END COAL DISCOVERY MESS ########################

dnl AM_PATH_SKSTREAM([MINIMUM-VERSION, [ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND]]])
dnl Test for skstream, and define SKSTREAM_CFLAGS and SKSTREAM_LIBS
dnl
AC_DEFUN(AM_PATH_SKSTREAM,
[dnl 
dnl Get the cflags and libraries from the skstream-config script
dnl
AC_LANG_CPLUSPLUS
AC_ARG_WITH(skstream-prefix,[  --with-skstream-prefix=PREFIX
                          Prefix where skstream is installed (optional)],
            skstream_prefix="$withval", skstream_prefix="")
AC_ARG_WITH(skstream-exec-prefix,[  --with-skstream-exec-prefix=PREFIX
                          Exec prefix where skstream is installed (optional)],
            skstream_exec_prefix="$withval", skstream_exec_prefix="")
AC_ARG_ENABLE(skstreamtest, [  --disable-skstreamtest     Do not try to compile and run a test skstream program],
		    , enable_skstreamtest=yes)

  if test x$skstream_exec_prefix != x ; then
     skstream_args="$skstream_args --exec-prefix=$skstream_exec_prefix"
     if test x${SKSTREAM_CONFIG+set} != xset ; then
        SKSTREAM_CONFIG=$skstream_exec_prefix/bin/skstream-config
     fi
  fi
  if test x$skstream_prefix != x ; then
     skstream_args="$skstream_args --prefix=$skstream_prefix"
     if test x${SKSTREAM_CONFIG+set} != xset ; then
        SKSTREAM_CONFIG=$skstream_prefix/bin/skstream-config
     fi
  fi

  AC_PATH_PROG(SKSTREAM_CONFIG, skstream-config, no)
  min_skstream_version=ifelse([$1], ,0.3.11,$1)
  AC_MSG_CHECKING(for skstream - version >= $min_skstream_version)
  no_skstream=""
  if test "$SKSTREAM_CONFIG" = "no" ; then
    no_skstream=yes
  else
    SKSTREAM_CFLAGS=`$SKSTREAM_CONFIG $skstreamconf_args --cflags`
    SKSTREAM_LIBS=`$SKSTREAM_CONFIG $skstreamconf_args --libs`

    skstream_major_version=`$SKSTREAM_CONFIG $skstream_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\1/'`
    skstream_minor_version=`$SKSTREAM_CONFIG $skstream_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\2/'`
    skstream_micro_version=`$SKSTREAM_CONFIG $skstream_config_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\3/'`
    if test "x$enable_skstreamtest" = "xyes" ; then
      ac_save_CFLAGS="$CFLAGS"
      ac_save_LIBS="$LIBS"
      CFLAGS="$CFLAGS $SKSTREAM_CFLAGS"
      LIBS="$LIBS $SKSTREAM_LIBS"
dnl
dnl Now check if the installed skstream is sufficiently new. (Also sanity
dnl checks the results of skstream-config to some extent)
dnl
      rm -f conf.skstreamtest
      AC_TRY_RUN([
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

char*
my_strdup (char *str)
{
  char *new_str;
  
  if (str)
    {
      new_str = (char *)malloc ((strlen (str) + 1) * sizeof(char));
      strcpy (new_str, str);
    }
  else
    new_str = NULL;
  
  return new_str;
}

int main (int argc, char *argv[])
{
  int major, minor, micro;
  char *tmp_version;

  /* This hangs on some systems (?)
  system ("touch conf.skstreamtest");
  */
  { FILE *fp = fopen("conf.skstreamtest", "a"); if ( fp ) fclose(fp); }

  /* HP/UX 9 (%@#!) writes to sscanf strings */
  tmp_version = my_strdup("$min_skstream_version");
  if (sscanf(tmp_version, "%d.%d.%d", &major, &minor, &micro) != 3) {
     printf("%s, bad version string\n", "$min_skstream_version");
     exit(1);
   }

   if (($skstream_major_version > major) ||
      (($skstream_major_version == major) && ($skstream_minor_version > minor)) ||
      (($skstream_major_version == major) && ($skstream_minor_version == minor) && ($skstream_micro_version >= micro)))
    {
      return 0;
    }
  else
    {
      printf("\n*** 'skstream-config --version' returned %d.%d.%d, but the minimum version\n", $skstream_major_version, $skstream_minor_version, $skstream_micro_version);
      printf("*** of skstream required is %d.%d.%d. If skstream-config is correct, then it is\n", major, minor, micro);
      printf("*** best to upgrade to the required version.\n");
      printf("*** If skstream-config was wrong, set the environment variable SKSTREAM_CONFIG\n");
      printf("*** to point to the correct copy of skstream-config, and remove the file\n");
      printf("*** config.cache before re-running configure\n");
      return 1;
    }
}

],, no_skstream=yes,[echo $ac_n "cross compiling; assumed OK... $ac_c"])
       CFLAGS="$ac_save_CFLAGS"
       LIBS="$ac_save_LIBS"
     fi
  fi
  if test "x$no_skstream" = x ; then
     AC_MSG_RESULT(yes)
     ifelse([$2], , :, [$2])     
  else
     AC_MSG_RESULT(no)
     if test "$SKSTREAM_CONFIG" = "no" ; then
       echo "*** The skstream-config script installed by skstream could not be found"
       echo "*** If skstream was installed in PREFIX, make sure PREFIX/bin is in"
       echo "*** your path, or set the SKSTREAM_CONFIG environment variable to the"
       echo "*** full path to skstream-config."
     else
       if test -f conf.skstreamtest ; then
        :
       else
          echo "*** Could not run skstream test program, checking why..."
          CFLAGS="$CFLAGS $SKSTREAM_CFLAGS"
          LIBS="$LIBS $SKSTREAM_LIBS"
          AC_TRY_LINK([
#include <stdio.h>
],      [ return 0; ],
        [ echo "*** The test program compiled, but did not run. This usually means"
          echo "*** that the run-time linker is not finding skstream or finding the wrong"
          echo "*** version of skstream. If it is not finding skstream, you'll need to set your"
          echo "*** LD_LIBRARY_PATH environment variable, or edit /etc/ld.so.conf to point"
          echo "*** to the installed location  Also, make sure you have run ldconfig if that"
          echo "*** is required on your system"
	  echo "***"
          echo "*** If you have an old version installed, it is best to remove it, although"
          echo "*** you may also be able to get things to work by modifying LD_LIBRARY_PATH"],
        [ echo "*** The test program failed to compile or link. See the file config.log for the"
          echo "*** exact error that occured. This usually means skstream was incorrectly installed"
          echo "*** or that you have moved skstream since it was installed. In the latter case, you"
          echo "*** may want to edit the skstream-config script: $SKSTREAM_CONFIG" ])
          CFLAGS="$ac_save_CFLAGS"
          LIBS="$ac_save_LIBS"
       fi
     fi
     SKSTREAM_CFLAGS=""
     SKSTREAM_LIBS=""
     ifelse([$3], , :, [$3])
  fi
  AC_SUBST(SKSTREAM_CFLAGS)
  AC_SUBST(SKSTREAM_LIBS)
  rm -f conf.skstreamtest
])

# Configure paths for varconf
# Stolen by Michael Koch from Atlas
# Adapted by Al Riddoch
# stolen from Sam Lantinga of SDL
# stolen from Manish Singh
# stolen back from Frank Belew
# stolen from Manish Singh
# Shamelessly stolen from Owen Taylor

dnl AM_PATH_VARCONF([MINIMUM-VERSION, [ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND]]])
dnl Test for Varconf, and define VARCONF_CFLAGS and VARCONF_LIBS
dnl
AC_DEFUN(AM_PATH_VARCONF,
[dnl 
dnl Get the cflags and libraries from the varconf-config script
dnl
AC_LANG_CPLUSPLUS
AC_ARG_WITH(varconf-prefix,[  --with-varconf-prefix=PFX   Prefix where Varconf is installed (optional)],
            varconf_prefix="$withval", varconf_prefix="")
AC_ARG_WITH(varconf-exec-prefix,[  --with-varconf-exec-prefix=PFX Exec prefix where Varconf is installed (optional)],
            varconf_exec_prefix="$withval", varconf_exec_prefix="")
AC_ARG_ENABLE(varconftest, [  --disable-varconftest       Do not try to compile and run a test Varconf program],
		    , enable_varconftest=yes)

  if test x$varconf_exec_prefix != x ; then
     varconf_args="$varconf_args --exec-prefix=$varconf_exec_prefix"
     if test x${VARCONF_CONFIG+set} != xset ; then
        VARCONF_CONFIG=$varconf_exec_prefix/bin/varconf-config
     fi
  fi
  if test x$varconf_prefix != x ; then
     varconf_args="$varconf_args --prefix=$varconf_prefix"
     if test x${VARCONF_CONFIG+set} != xset ; then
        VARCONF_CONFIG=$varconf_prefix/bin/varconf-config
     fi
  fi

  AC_PATH_PROG(VARCONF_CONFIG, varconf-config, no)
  min_varconf_version=ifelse([$1], ,0.3.11,$1)
  AC_MSG_CHECKING(for Varconf - version >= $min_varconf_version)
  no_varconf=""
  if test "$VARCONF_CONFIG" = "no" ; then
    no_varconf=yes
  else
    VARCONF_CFLAGS=`$VARCONF_CONFIG $varconfconf_args --cflags`
    VARCONF_LIBS=`$VARCONF_CONFIG $varconfconf_args --libs`

    varconf_major_version=`$VARCONF_CONFIG $varconf_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\1/'`
    varconf_minor_version=`$VARCONF_CONFIG $varconf_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\2/'`
    varconf_micro_version=`$VARCONF_CONFIG $varconf_config_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\3/'`
    if test "x$enable_varconftest" = "xyes" ; then
      ac_save_CFLAGS="$CFLAGS"
      ac_save_LIBS="$LIBS"
      CFLAGS="$CFLAGS $VARCONF_CFLAGS"
      LIBS="$LIBS $VARCONF_LIBS"
dnl
dnl Now check if the installed Varconf is sufficiently new. (Also sanity
dnl checks the results of varconf-config to some extent)
dnl
      rm -f conf.varconftest
      AC_TRY_RUN([
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

char*
my_strdup (char *str)
{
  char *new_str;
  
  if (str)
    {
      new_str = (char *)malloc ((strlen (str) + 1) * sizeof(char));
      strcpy (new_str, str);
    }
  else
    new_str = NULL;
  
  return new_str;
}

int main (int argc, char *argv[])
{
  int major, minor, micro;
  char *tmp_version;

  /* This hangs on some systems (?)
  system ("touch conf.varconftest");
  */
  { FILE *fp = fopen("conf.varconftest", "a"); if ( fp ) fclose(fp); }

  /* HP/UX 9 (%@#!) writes to sscanf strings */
  tmp_version = my_strdup("$min_varconf_version");
  if (sscanf(tmp_version, "%d.%d.%d", &major, &minor, &micro) != 3) {
     printf("%s, bad version string\n", "$min_varconf_version");
     exit(1);
   }

   if (($varconf_major_version > major) ||
      (($varconf_major_version == major) && ($varconf_minor_version > minor)) ||
      (($varconf_major_version == major) && ($varconf_minor_version == minor) && ($varconf_micro_version >= micro)))
    {
      return 0;
    }
  else
    {
      printf("\n*** 'varconf-config --version' returned %d.%d.%d, but the minimum version\n", $varconf_major_version, $varconf_minor_version, $varconf_micro_version);
      printf("*** of Varconf required is %d.%d.%d. If varconf-config is correct, then it is\n", major, minor, micro);
      printf("*** best to upgrade to the required version.\n");
      printf("*** If varconf-config was wrong, set the environment variable VARCONF_CONFIG\n");
      printf("*** to point to the correct copy of varconf-config, and remove the file\n");
      printf("*** config.cache before re-running configure\n");
      return 1;
    }
}

],, no_varconf=yes,[echo $ac_n "cross compiling; assumed OK... $ac_c"])
       CFLAGS="$ac_save_CFLAGS"
       LIBS="$ac_save_LIBS"
     fi
  fi
  if test "x$no_varconf" = x ; then
     AC_MSG_RESULT(yes)
     ifelse([$2], , :, [$2])     
  else
     AC_MSG_RESULT(no)
     if test "$VARCONF_CONFIG" = "no" ; then
       echo "*** The varconf-config script installed by Varconf could not be found"
       echo "*** If Varconf was installed in PREFIX, make sure PREFIX/bin is in"
       echo "*** your path, or set the VARCONF_CONFIG environment variable to the"
       echo "*** full path to varconf-config."
     else
       if test -f conf.varconftest ; then
        :
       else
          echo "*** Could not run Varconf test program, checking why..."
          CFLAGS="$CFLAGS $VARCONF_CFLAGS"
          LIBS="$LIBS $VARCONF_LIBS"
          AC_TRY_LINK([
#include <stdio.h>
],      [ return 0; ],
        [ echo "*** The test program compiled, but did not run. This usually means"
          echo "*** that the run-time linker is not finding Varconf or finding the wrong"
          echo "*** version of Varconf. If it is not finding Varconf, you'll need to set your"
          echo "*** LD_LIBRARY_PATH environment variable, or edit /etc/ld.so.conf to point"
          echo "*** to the installed location  Also, make sure you have run ldconfig if that"
          echo "*** is required on your system"
	  echo "***"
          echo "*** If you have an old version installed, it is best to remove it, although"
          echo "*** you may also be able to get things to work by modifying LD_LIBRARY_PATH"],
        [ echo "*** The test program failed to compile or link. See the file config.log for the"
          echo "*** exact error that occured. This usually means Varconf was incorrectly installed"
          echo "*** or that you have moved Varconf since it was installed. In the latter case, you"
          echo "*** may want to edit the varconf-config script: $VARCONF_CONFIG" ])
          CFLAGS="$ac_save_CFLAGS"
          LIBS="$ac_save_LIBS"
       fi
     fi
     VARCONF_CFLAGS=""
     VARCONF_LIBS=""
     ifelse([$3], , :, [$3])
  fi
  AC_SUBST(VARCONF_CFLAGS)
  AC_SUBST(VARCONF_LIBS)
  rm -f conf.varconftest
])

# Configure paths for Janus
# stolen by Michael Koch
# Adapted by Al Riddoch
# stolen from Sam Lantinga of SDL
# stolen from Manish Singh
# stolen back from Frank Belew
# stolen from Manish Singh
# Shamelessly stolen from Owen Taylor

dnl AM_PATH_JANUS([MINIMUM-VERSION, [ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND]]])
dnl Test for Janus, and define JANUS_CFLAGS and JANUS_LIBS
dnl
AC_DEFUN(AM_PATH_JANUS,
[dnl 
dnl Get the cflags and libraries from the janus-config script
dnl
AC_LANG_CPLUSPLUS
AC_ARG_WITH(janus-prefix,[  --with-janus-prefix=PREFIX
                          Prefix where Janus is installed (optional)],
            janus_prefix="$withval", janus_prefix="")
AC_ARG_WITH(janus-exec-prefix,[  --with-janus-exec-prefix=PREFIX
                          Exec prefix where Janus is installed (optional)],
            janus_exec_prefix="$withval", janus_exec_prefix="")
AC_ARG_ENABLE(janustest, [  --disable-janustest     Do not try to compile and run a test Janus program],
		    , enable_janustest=yes)

  if test x$janus_exec_prefix != x ; then
     janus_args="$janus_args --exec-prefix=$janus_exec_prefix"
     if test x${JANUS_CONFIG+set} != xset ; then
        JANUS_CONFIG=$janus_exec_prefix/bin/janus-config
     fi
  fi
  if test x$janus_prefix != x ; then
     janus_args="$janus_args --prefix=$janus_prefix"
     if test x${JANUS_CONFIG+set} != xset ; then
        JANUS_CONFIG=$janus_prefix/bin/janus-config
     fi
  fi

  AC_PATH_PROG(JANUS_CONFIG, janus-config, no)
  min_janus_version=ifelse([$1], ,0.3.11,$1)
  AC_MSG_CHECKING(for Janus - version >= $min_janus_version)
  no_janus=""
  if test "$JANUS_CONFIG" = "no" ; then
    no_janus=yes
  else
    JANUS_CFLAGS=`$JANUS_CONFIG $janusconf_args --cflags`
    JANUS_LIBS=`$JANUS_CONFIG $janusconf_args --libs`

    janus_major_version=`$JANUS_CONFIG $janus_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\1/'`
    janus_minor_version=`$JANUS_CONFIG $janus_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\2/'`
    janus_micro_version=`$JANUS_CONFIG $janus_config_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\3/'`
    if test "x$enable_janustest" = "xyes" ; then
      ac_save_CFLAGS="$CFLAGS"
      ac_save_LIBS="$LIBS"
      CFLAGS="$CFLAGS $JANUS_CFLAGS"
      LIBS="$LIBS $JANUS_LIBS"
dnl
dnl Now check if the installed Janus is sufficiently new. (Also sanity
dnl checks the results of janus-config to some extent)
dnl
      rm -f conf.janustest
      AC_TRY_RUN([
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

char*
my_strdup (char *str)
{
  char *new_str;
  
  if (str)
    {
      new_str = (char *)malloc ((strlen (str) + 1) * sizeof(char));
      strcpy (new_str, str);
    }
  else
    new_str = NULL;
  
  return new_str;
}

int main (int argc, char *argv[])
{
  int major, minor, micro;
  char *tmp_version;

  /* This hangs on some systems (?)
  system ("touch conf.janustest");
  */
  { FILE *fp = fopen("conf.janustest", "a"); if ( fp ) fclose(fp); }

  /* HP/UX 9 (%@#!) writes to sscanf strings */
  tmp_version = my_strdup("$min_janus_version");
  if (sscanf(tmp_version, "%d.%d.%d", &major, &minor, &micro) != 3) {
     printf("%s, bad version string\n", "$min_janus_version");
     exit(1);
   }

   if (($janus_major_version > major) ||
      (($janus_major_version == major) && ($janus_minor_version > minor)) ||
      (($janus_major_version == major) && ($janus_minor_version == minor) && ($janus_micro_version >= micro)))
    {
      return 0;
    }
  else
    {
      printf("\n*** 'janus-config --version' returned %d.%d.%d, but the minimum version\n", $janus_major_version, $janus_minor_version, $janus_micro_version);
      printf("*** of Janus required is %d.%d.%d. If janus-config is correct, then it is\n", major, minor, micro);
      printf("*** best to upgrade to the required version.\n");
      printf("*** If janus-config was wrong, set the environment variable JANUS_CONFIG\n");
      printf("*** to point to the correct copy of janus-config, and remove the file\n");
      printf("*** config.cache before re-running configure\n");
      return 1;
    }
}

],, no_janus=yes,[echo $ac_n "cross compiling; assumed OK... $ac_c"])
       CFLAGS="$ac_save_CFLAGS"
       LIBS="$ac_save_LIBS"
     fi
  fi
  if test "x$no_janus" = x ; then
     AC_MSG_RESULT(yes)
     ifelse([$2], , :, [$2])     
  else
     AC_MSG_RESULT(no)
     if test "$JANUS_CONFIG" = "no" ; then
       echo "*** The janus-config script installed by Janus could not be found"
       echo "*** If Janus was installed in PREFIX, make sure PREFIX/bin is in"
       echo "*** your path, or set the JANUS_CONFIG environment variable to the"
       echo "*** full path to janus-config."
     else
       if test -f conf.janustest ; then
        :
       else
          echo "*** Could not run Janus test program, checking why..."
          CFLAGS="$CFLAGS $JANUS_CFLAGS"
          LIBS="$LIBS $JANUS_LIBS"
          AC_TRY_LINK([
#include <stdio.h>
],      [ return 0; ],
        [ echo "*** The test program compiled, but did not run. This usually means"
          echo "*** that the run-time linker is not finding Janus or finding the wrong"
          echo "*** version of Janus. If it is not finding Janus, you'll need to set your"
          echo "*** LD_LIBRARY_PATH environment variable, or edit /etc/ld.so.conf to point"
          echo "*** to the installed location  Also, make sure you have run ldconfig if that"
          echo "*** is required on your system"
	  echo "***"
          echo "*** If you have an old version installed, it is best to remove it, although"
          echo "*** you may also be able to get things to work by modifying LD_LIBRARY_PATH"],
        [ echo "*** The test program failed to compile or link. See the file config.log for the"
          echo "*** exact error that occured. This usually means Janus was incorrectly installed"
          echo "*** or that you have moved Janus since it was installed. In the latter case, you"
          echo "*** may want to edit the janus-config script: $JANUS_CONFIG" ])
          CFLAGS="$ac_save_CFLAGS"
          LIBS="$ac_save_LIBS"
       fi
     fi
     JANUS_CFLAGS=""
     JANUS_LIBS=""
     ifelse([$3], , :, [$3])
  fi
  AC_SUBST(JANUS_CFLAGS)
  AC_SUBST(JANUS_LIBS)
  rm -f conf.janustest
])
