//  Copyright (C) 1998, 1999 The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//
// created by Karsten Laux 02/2000

#include "charchooser.h"
#include <uta/application.h>
#include <uta/multilineedit.h>
#include <uta/resources.h>
#include <uta/debug.h>

#include "dii/object.h"
#include "dialog.h"
#include "buttons.h"
#include "settings.h"
#include "operation.h"

#include "../player/player.h"

using namespace uta;

#define GRID 25

CharChooser::CharChooser() :
  Screen()
{
}

Widget*
CharChooser::create(Widget* p, int x, int y, int w, int h)
{
  Widget* widget = new Widget(p,x,y,w,h);

  Label* head = new Label(widget, GRID, GRID, 400, GRID, 
			  FONT_RES("button_font"));
  head->setAlign(Label::ALIGN_LEFT);
  head->setText("CHOOSE CHARACTER TO PLAY:"); 

  characterlist_ = new ListBox(widget, GRID, GRID*3, 200, 200);
  characterlist_->setSelectedColor(Color(64,64,64,128));
  characterlist_->setBackground(SURFACE_RES("background"));

  takeBut_ = new FixButton(widget, GRID*2+200, GRID*3);
  takeBut_->setText("Select ...");

  MultiLineEdit* info = new MultiLineEdit(widget, 
					  GRID*2+200, GRID*4, 400, 80);
  info->addText("... one of your existing characters from the list. "
		"When you logout your character usually stays alive. So "
		"if you want to continue your old game press here.");
  info->setTransparency(true);

  //  charactername_ = new FixLineEdit(widget, GRID, GRID*2+200);
  createBut_ = new FixButton(widget, GRID*2+200, GRID*5+80);
  createBut_->setText("Create ...");
  
  info = new MultiLineEdit(widget, GRID*2+200, GRID*6+80, 400, 80);
  info->addText(" ... a new character. You will be offered a new dialog "
		"where you may set different attributes of your character.");
  info->setTransparency(true);


//   PushButton* okbut = new FixButton(widget, (w-75)/2, h-50);
//   okbut->setText("Cancel");



  updateDialog();

  characterlist_->selectionChanged.connect(slot(this,&CharChooser::slot_select));
  createBut_->clicked.connect(slot(this, &CharChooser::slot_create));
  takeBut_->clicked.connect(slot(this, &CharChooser::slot_take));

//   okbut->clicked.connect(slot(this, &Screen::reject));

  return widget;
}

CharChooser::~CharChooser()
{
}
 
void
CharChooser::cleanup()
{
//   //read the name from the linedit widget
//   string name = charactername_->getText();
//   DII::callMethod("Character","chooseProfile",name);
}

void
CharChooser::updateDialog()
{
  //check if there already exist some characters for this account
  DII::Object account = readState("account");
  DII::Object::MapType::const_iterator fItr = account.AsMap().find("characters");
  if(fItr != account.AsMap().end())
    {
      DII::Object::ListType characters = fItr->second.AsList();
      
//       cerr << "You have already created " << characters.size() 
// 	   << " characters !" << endl;  
  
      //and update the list
      characterlist_->clear();
      DII::Object::ListType::iterator itr = characters.begin();
      while(itr != characters.end())
	{
	  characterlist_->addElement(itr->AsString());
	  itr++;
	}
    } 
}

void
CharChooser::slot_select(const char* name)
{
  if(!name)
    return;

//   string charactername(name);

//   if(charactername.empty())
//     return;

//   DII::callMethod("Character","chooseProfile",charactername);

//   charactername_->setText(name);
}


void
CharChooser::slot_change(const char* name)
{
  if(!name)
    return;
 
  std::string charactername(name);
  if(charactername.empty())
    return;
}

void
CharChooser::slot_create()
{
//   string name = charactername_->getText();
//   if(!name.empty())
//     {
//       reject();
//     }
  reject();
}

void
CharChooser::slot_take()
{
  std::string name = characterlist_->getSelection();
  if(!name.empty())
    {
      if(Player::instance())
        Player::instance()->takeOverCharacter(name);
      accept();
    }
}
