//  Copyright (C) 1998, 1999 The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//
// created by Karsten Laux 02/2000

#include "editscreen.h"

#include "dii/object.h"
#include "buttons.h"
#include "widgets.h"
#include "settings.h"
#include <uta/resources.h>

#include "../player/player.h"
#include "../modules/char_stats.h"

#include <varconf/Config.h>
#include "toolbox/operation.h"

using namespace uta;

#define GRID 30

#ifndef HAVE_DRAND48
#define drand48() (rand()/((double)RAND_MAX))
#endif

EditScreen::EditScreen() :
  Screen(),
  female(false)
{

}

Widget*
EditScreen::create(Widget* p, int x, int y, int w, int h)
{
  
  Widget* widget = new Widget(p,x,y,w,h);

  Label* head = new Label(widget, GRID, GRID, 400, GRID, FONT_RES("button_font"));
  head->setAlign(Label::ALIGN_LEFT);
  head->setText("CREATE YOUR CHARACTER:");      

        // n is the row counter
  int n=3;

  // get a name for the character from the user
  Label *label = new Label(widget, GRID, GRID*n, 110, 25);
  label->setAlign(Label::ALIGN_LEFT);
  label->setTransparency(true);
  label->setText("Character name:");
  characterName_ = new FixLineEdit(widget, GRID+120, GRID*n);
  n++;

  // get the character type for the character from the user
	// This type must be supported by the server !
  label = new Label(widget, GRID, GRID*n, 110, 25);
  label->setAlign(Label::ALIGN_LEFT);
  label->setTransparency(true);
  label->setText("Character type:");
  type_ = new FixComboEdit(widget, GRID+120, GRID*n);

	// Fetch the list of possible character types from the registry
	// (that's Uclient registry, not system registry, for you win32 goofballs)
  std::vector<std::string> types;

  DII::Object account = readState("account");
  if(doesContain(account, "character_types")) {
    DII::Object::ListType character_types =
      account.AsMap()["character_types"].AsList();
    // There might be a simpler way to do this, 
    // but I am not yet familiar with the DII objects
    DII::Object::ListType::iterator itr = character_types.begin();
    while(itr != character_types.end()) {
      types.push_back(itr->AsString());
			itr++;
		}
  }
	else {
		// No character types, so we fill in with
		// some defaults
    types.push_back("farmer");
    types.push_back("settler");
    types.push_back("swinesherd");
    types.push_back("lych");
	}
  type_->setList(types);

//   //currently only in standalone mode we allow all character types
  bool standalone = varconf::Config::inst()->getItem("flags","standalone");
//   if(!standalone)
//     type_->disable();

  n++;
  n++;

  // get a name for the character from the user
  label = new Label(widget, GRID, GRID*n, 110, 25);
  label->setAlign(Label::ALIGN_LEFT);
  label->setTransparency(true);
  label->setText("Description:");
  
  
  //default font, not readonly
  description_ = new MultiLineEdit(widget, GRID, GRID+GRID*n, 250, 125,NULL,false);
  description_->setColor(Color(0,0,0,32));

  charImg = new Widget(widget, w-GRID-200, GRID, 200, 270);

  genderbut = new PushButton(widget, w-GRID-200, GRID+ 280, 200, 25);
  genderbut->setText("Change Gender");
  genderbut->clicked.connect(slot(this, &EditScreen::changeGender));
  genderbut->setColor(lightgrey);

  genbut = new FixButton(widget, GRID, h-50);
  genbut->setText("Create");
  genbut->clicked.connect(slot(this, &EditScreen::generate));
  if(standalone)
    genbut->disable();

  updbut = new FixButton(widget, GRID+125,h-50);
  updbut->setText("Accept");
  updbut->clicked.connect(slot(this, &EditScreen::updChar));
  if(!standalone)
    updbut->disable();

  PushButton* okbut = new FixButton(widget, w-115-GRID, h-50);
  okbut->setText("Cancel");
  okbut->clicked.connect(slot(this, &Screen::reject));

  index_ = 0;

  // lookup "portrait_*" resources

//   unsigned num_portraits = 0;
//   string id = "portrait_"+itos(num_portraits);
//   while(RES_AVAILABLE(id) && SURFACE_RES(id))
//     {
//       portraits_.push_back(id);
//       num_portraits++;
//       id = "portrait_"+itos(num_portraits);
//     }
  
//   //has user provided his own portrait ?
//   if(RES_AVAILABLE("portrait") && SURFACE_RES("portrait"))
//     portraits_.push_back("portrait");
    

  if(readState("server_have_character").AsInt())
    {
      genbut->disable();
      change->disable();
      genderbut->disable();
    }

  rddata();

  return widget;
}

EditScreen::~EditScreen()
{
}

void
EditScreen::cleanup()
{
  wbdata();
}

void 
EditScreen::wbdata()
{
  std::string name = characterName_->getText();
  DII::Object::MapType character;
  
  DII::Object tmp =  readState("character");
  if(tmp.IsMap())
    character = tmp.AsMap();

  //name
  character["name"] = name;
  //type
  DII::Object::ListType tl;
  tl.push_back(std::string(type_->getText()));
  character["parents"] = tl;

  //sex
  if(female)
    character["sex"] =  std::string("female");
  else
    character["sex"] = std::string("male");

  //description
  character["description"] = std::string(description_->getText());
    
  writeState("character",character);
}

void 
EditScreen::rddata()
{
  bool female = false;
  std::string portrait;
  std::string type = "settler";
  std::string name = "";
  std::string description = "New Character";

  try
    {
      if(readState("character").IsMap())
        {
          DII::Object::MapType character = readState("character").AsMap();
         
          if(doesContain(character,"name"))
            name = character["name"].AsString();
          if(doesContain(character,"sex"))
            female = (character["sex"].AsString() == "female");
          
          if(doesContain(character,"parents"))
            {
              DII::Object::ListType parent = character["parents"].AsList();
              type = parent.begin()->AsString();
            }
          if(doesContain(character,"description"))          
            description = character["description"].AsString();
          if(doesContain(character,"portrait"))
            portrait = readState("portrait").AsString();
        }
    }
  catch(...)
    {
    }

  characterName_->setText(name.c_str());
  description_->addText(description.c_str());
 
  const uta::Surface* img;
  if(female)
    img = SURFACE_RES("female_character");
  else
    img = SURFACE_RES("male_character");

  charImg->setBackground(img);


  type_->setText(type.c_str());

//   if(!portraits_.empty())
//     {
//       if(!portrait.empty())
//         {
//           index_ = 0;
//           while( index_ < portraits_.size() && portraits_[index_] != portrait)
//             index_++;
//           if(index_ < portraits_.size())
//             image_->setBackground(SURFACE_RES(portraits_[index_]));
//         }
//       else
//         {
//           index_ = (int)(drand48()*portraits_.size());
//           image_->setBackground(SURFACE_RES(portraits_[index_]));
//           writeState("portrait", portraits_[index_]);
//         }
//     }
}


void
EditScreen::generate()
{
  wbdata();
  
  if(Player::instance()->createCharacter())
    {
      CharStats::instance()->update();
      accept();
    }
}

void
EditScreen::updChar()
{
  generate();
  //  wbdata();
  
}

void
EditScreen::changePortrait()
{
//   if(portraits_.empty())
//     return;

//   index_++;
//   if(index_ >= portraits_.size())
//     index_ = 0;
//   image_->setBackground(SURFACE_RES(portraits_[index_]));
//   writeState("portrait", portraits_[index_]);

}

void
EditScreen::changeGender()
{
  female = !female;

  const uta::Surface* img;
  if(female)
    img = SURFACE_RES("female_character");
  else
    img = SURFACE_RES("male_character");
  charImg->setBackground(img);
}
