//  Copyright (C) 1998, 1999 The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//
// created by Karsten Laux 02/2000

#include "inventorydialog.h"
#include <uta/application.h>
#include <uta/pushbutton.h>
#include <uta/multilineedit.h>
#include <uta/resources.h>

#include "dialog.h"
#include "buttons.h"
#include "dii/dii.h"
#include "settings.h"
#include "operation.h"
#include "world.h"

#include "../player/player.h"

using namespace uta;

#define GRID 30
  
InventoryDialog::InventoryDialog() :
  Screen(),
  listbox_(NULL),
  refresh_(1000)
{
  refresh_.alarm.connect(slot(this, &InventoryDialog::readData));
}

Widget* 
InventoryDialog::create(Widget* p, int x, int y, int w, int h)
{
MARK
  Widget* widget = new Widget(p,x,y,w,h);
    
  Label* head = new Label(widget, GRID, GRID, 400, GRID, FONT_RES("button_font"));
  head->setAlign(Label::ALIGN_LEFT);
  head->setText("INVENTORY");

  listbox_ = new ListBox(widget, GRID, GRID*3, 200, h-100-4*GRID);
  listbox_->selectionChanged.connect(slot(this, &InventoryDialog::select));
  
  Label* label = new Label(widget, GRID*3+200, GRID*3, 300, GRID);
  label->setAlign(Label::ALIGN_LEFT);
  label->setText("Description:");
MARK
  desc_ = new MultiLineEdit(widget, 
			    GRID*3+200, GRID*5, w-200-3*GRID, h-100-6*GRID);

  desc_->setTransparency(true);
  desc_->addText("Description of selected item.");

  PushButton* loginbut_= new FixButton(widget, GRID, h-50);
  loginbut_->setText("Drop Item");
  loginbut_->clicked.connect(slot(this, &InventoryDialog::drop));

  PushButton* but = new FixButton(widget, (w-115-GRID), h-50);
  but->setText("Close");
  but->clicked.connect(slot(this, &Screen::reject));
MARK
  readData();
  refresh_.run();

  return widget;
}

InventoryDialog::~InventoryDialog()
{  
}

void
InventoryDialog::cleanup()
{MARK
  refresh_.halt();
  writebackData();
MARK
}

void
InventoryDialog::writebackData()
{ 
}

void
InventoryDialog::readData()
{
MARK
  if(!listbox_)
    return;
MARK
  DII::Object character = Player::instance()->currentCharacterID();

  if(!character.IsString() || character.AsString().empty())
    {
      cerr << "InventoryDialog: cannot find player character ! " << endl;
      return;
    }

  std::string id = character.AsString();
  // character = DII::callMethod("World","queryObject",id);
  character = World::World::inst()->queryObject(id);
  if(character == DII::FAILURE)
    return;
  MARK
  listbox_->clear();
 
  if(doesContain(character,"contains"))
    {
      try
	{
MARK
	  const DII::Object::ListType& contains = 
	    character.AsMap()["contains"].AsList();
	  
	  World::Entity* entity;
	  DII::Object::ListType::const_iterator itr = contains.begin(); 
	  DII::Object::MapType item;
	  while(itr != contains.end())
	    {
	      entity = World::World::inst()->lookupEntity(itr->AsString());
	      assert(entity);
	      listbox_->addElement(entity->id());
	      itr++;
	    } MARK
	}
      catch(...)
	{
	  cerr << "InventoryDialog: character object is of unexpected type ! " 
	       << endl;
	}
    }
  else
    {
      //inventory is empty !
    }

  MARK
}


void
InventoryDialog::drop()
{
  writebackData();
 
  if(Player::instance()->drop(listbox_->getSelection()))
    readData();
}

void
InventoryDialog::select(const char* data)
{
  if(data)
    {
      std::string id = std::string(data);
      DII::Object obj = World::World::inst()->queryObject(id);
      if(obj == DII::FAILURE)
	return;

      std::string desc;
      if(doesContain(obj,"description"))
	{
	  if(obj.AsMap()["description"].IsString())
	    desc_->setText(obj.AsMap()["description"].AsString().c_str());
	}
    }
}
