//  Copyright (C) 1998, 1999 The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//
// created by Karsten Laux 02/2000

#include "logindialog.h"
#include <uta/application.h>
#include <uta/pushbutton.h>
#include <uta/multilineedit.h>
#include <varconf/Config.h>

#include "dialog.h"
#include "buttons.h"
#include "widgets.h"
#include "dii/object.h"
#include "settings.h"

#include "../player/player.h"

using namespace uta;

#define GRID 30

LoginDialog::LoginDialog() :
  Screen()
{
}

Widget* 
LoginDialog::create(Widget* p, int x, int y, int w, int h)
{
  Widget* widget = new Widget(p,x,y,w,h);
    
  Label* label;

  label = new Label(widget, GRID, GRID, 300, 25);
  label->setAlign(Label::ALIGN_LEFT);
  label->setTransparency(true);
  label->setText("CONNECT TO THE GAME SERVER :");
  
  label = new Label(widget, GRID, GRID*2, 200, 25);
  label->setAlign(Label::ALIGN_LEFT);
  label->setTransparency(true);
  label->setText("Server: ");
  server_ = new FixLineEdit(widget, GRID+220, GRID*2);

  label = new Label(widget, GRID*2, GRID*3, 200, 25);
  label->setAlign(Label::ALIGN_LEFT);
  label->setTransparency(true);
  label->setText("Loginname :");
  login_ = new FixLineEdit(widget, GRID+220, GRID*3);
 
  label = new Label(widget, GRID*2, GRID*4, 200, 25);
  label->setAlign(Label::ALIGN_LEFT);
  label->setTransparency(true);
  label->setText("Password :");  
  password_ = new FixLineEdit(widget, GRID+220, GRID*4);
  
  MultiLineEdit* info = new MultiLineEdit(widget, 
					  GRID, GRID*5, 500, 300);
  info->setTransparency(true);
  info->addText("Provide your account parameters and enter the "
		"hostname or IP-address of the server.\n"
		"\n"
		"If you want to login using an existing account press LOGIN.\n"
		"\n"
		"If you however connect to this server for the first time "
		"press NEW ACCOUNT to create a new account on this server.\n"
		"\n"
		"You can control several player characters per account, so there is "
		"no need to have several accounts on the same server.");

  loginbut_= new FixButton(widget, GRID, h-50);
  loginbut_->setText("Login");
  loginbut_->clicked.connect(slot(this, &LoginDialog::login));

  createbut_= new FixButton(widget, 2*GRID+115, h-50);
  createbut_->setText("New Account");
  createbut_->clicked.connect(slot(this, &LoginDialog::createAccount));

  PushButton* but = new FixButton(widget, (w-115-GRID), h-50);
  but->setText("Cancel");
  but->clicked.connect(slot(this, &Screen::reject));

  //update the widgets
  readData();

  return widget;
}

LoginDialog::~LoginDialog()
{  
}

void
LoginDialog::cleanup()
{
  /* removed this call to writebackData() as it overwrites account object sent 
   * by the server 
   */
  //  writebackData();
}

void
LoginDialog::writebackData()
{
  DII::Object::MapType account;
  account["id"] = std::string(login_->getText());
  account["password"] = std::string(password_->getText());
  writeState("account",account);

  DII::Object::MapType server;
  server["hostname"] = std::string(server_->getText());
  writeState("server",server);

  varconf::Config::inst()->setItem("server","login",login_->getText());
  varconf::Config::inst()->setItem("server","hostname",server_->getText());
}

void
LoginDialog::readData()
{
  std::string id, hostname;
  std::string passwd;
  
  bool logged_in = false;

  try
    {
      DII::Object::MapType account = readState("account").AsMap();
      DII::Object::MapType server = readState("server").AsMap();

      id = account["id"].AsString();
      hostname = server["hostname"].AsString();
      passwd = account["password"].AsString();

      logged_in = readState("server_logged_in").AsInt();
    }
  catch(...)
    {

    }
     
  if(id.empty())
    id = varconf::Config::inst()->getItem("server","login");
  if(hostname.empty())
    hostname = varconf::Config::inst()->getItem("server","hostname");

  //and write to the widgets
  login_->setText(id.c_str());
  password_->setText(passwd.c_str());
  server_->setText(hostname.c_str());
  
  if(logged_in)
    {
      loginbut_->disable();
      createbut_->disable();
    }
  else
    {
      loginbut_->enable();
      createbut_->enable();
    }
}


void
LoginDialog::login()
{

  writebackData();

  if(Player::instance()->login(login_->getText(), password_->getText(),
	 server_->getText()))
    {
      loginbut_->disable();
      createbut_->disable();
      Screen::accept();
    };

}

void
LoginDialog::createAccount()
{

  writebackData();

  if(Player::instance()->createAccount(login_->getText(), password_->getText(),
	 server_->getText()))
    {
      loginbut_->disable();
      createbut_->disable();
      Screen::accept();
    };

}
