//  Copyright (C) 1998, 1999 The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//
// created by Karsten Laux 02/2000

#include "overview.h"

#include "dialog.h"
#include "buttons.h"
#include "settings.h"
#include "operation.h"
#include "world.h"

#include "../terrain/terrain.h"

#include <uta/resources.h>

using namespace uta;

//how large is the space between map and actual widget border ?
#define BORDER 0

Overview::Overview() :
  Screen(),
  updateTimer_(2000),
  mapWidget_(NULL)
{
//   const Surface* paper = SURFACE_RES("paper");
//   theMap_ = new Surface(*paper);  

  theMap_ = new Surface(600,400);
//   theMap_->setTransColor(black);
//   theMap_->setTransparency(true);

  updateTimer_.alarm.connect(slot(this,&Overview::updateMap));

  updateMap();
}

void
Overview::cleanup()
{
  updateTimer_.halt();
}

void
Overview::updateMap()
{
  
  theMap_->fill(transparent);

//   const Surface* paper = SURFACE_RES("paper");
//   paper->blit(theMap_);

  const uta::Surface* map;

  try 
    {
      map = Terrain::instance()->getMap();
    }
  catch(...)
    {
      map = NULL;
    };

  int max_w = theMap_->width() - 2*BORDER;
  int max_h = theMap_->height() - 2*BORDER;

  //crop map and center it:
  if(map)
    {
      int w,h;
      int dx,dy,sx,sy;
      w = max_w<map->width() ? max_w : map->width();
      h = max_h<map->height() ? max_h : map->height();
 
     if(w < map->width())
	sx = (map->width()-w)/2;
      else
	sx = 0;
      if(h < map->height())
	sy = (map->height()-h)/2;
      else
	sy = 0;
      //calculate destination offset
      dx = (max_w - w)/2 + BORDER;
      dy = (max_h - h)/2 + BORDER;
      map->blit(theMap_,Rect(dx,dy,w,h),Rect(sx,sy,w,h));  	
    }
  
  const uta::Surface* entities;

  entities = World::World::inst()->getOverview();


  //crop entities and center it:
  if(entities)
    {
      int w,h;
      int dx,dy,sx,sy;
      w = max_w<entities->width() ? max_w : entities->width();
      h = max_h<entities->height() ? max_h : entities->height();
 
     if(w < entities->width())
	sx = (entities->width()-w)/2;
      else
	sx = 0;
      if(h < entities->height())
	sy = (entities->height()-h)/2;
      else
	sy = 0;
      //calculate destination offset
      dx = (max_w - w)/2 + BORDER;
      dy = (max_h - h)/2 + BORDER;
      entities->blit(theMap_,Rect(dx,dy,w,h),Rect(sx,sy,w,h));  	
 
    }   
 
  if(mapWidget_)
    {
      mapWidget_->setBackground(theMap_);    
    }
}


Widget* 
Overview::create(Widget* p, int x, int y, int w, int h)
{
  Widget* widget = new Widget(p,x,y,w,h);
  
  int mw = theMap_->width() > w ? w : theMap_->width();
  int mh = theMap_->height() > h ? h : theMap_->height();

  mapWidget_ = new Widget(widget, 0,0,mw,mh);

  if(theMap_)
    mapWidget_->setBackground(theMap_);

  mapWidget_->setTransparency(true);

  PushButton* but = new FixButton(widget, (mw-115)/2, h-30);
  but->setText("Close");
  but->clicked.connect(slot(this, &Screen::accept));
  
  return widget;
}

Overview::~Overview()
{  
  if(theMap_)
    delete theMap_;
}


