//  The Worldforge Project
//  Copyright (C) 2002 The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//
// created by Ron Steinke 05/2002


#ifndef _CLASS_MOD_H_
#define _CLASS_MOD_H_

#include "dii/dii.h"
#include <utility>

namespace DII {

///
class ClassMod : public Module
{
  
 public:
  
  ///
  ClassMod(const char *name)
	: Module(name), m_have_init(false), m_have_deinit(false) {}
  ClassMod(const char *name, SigC::Slot1<bool,Module*> init_func)
	: Module(name), m_init(init_func), m_have_init(true), m_have_deinit(false) {}
  ClassMod(const char *name, SigC::Slot1<bool,Module*> init_func,
	SigC::Slot0<void> deinit_func) : Module(name), m_init(init_func),
	   m_deinit(deinit_func), m_have_init(true), m_have_deinit(true) {}
  ///
  virtual ~ClassMod() {}

  ///
  DII::Object init(const DII::Object &obj)
	{return m_have_init ? m_init.call(this) : SUCCESS;}
  ///
  DII::Object deinit(const DII::Object &obj)
	{if(m_have_deinit) m_deinit.call(); return SUCCESS;}

  ///
  void registerMethod(const std::pair<const char*, Method> &data, bool service = false)
    {registerMethod(data.first, data.second, service);}

  void registerMethod(const char *name, Method method, bool service = false)
    { // FIXME de-inline
      methods_[name] = method;
      if(service)
        registerService(this, name);
    }

  void needs(const char *modname) {dependencies_.push_front(modname);}

 private:
  SigC::Slot1<bool,Module*> m_init;
  SigC::Slot0<void> m_deinit;
  bool m_have_init, m_have_deinit;
};

} // namespace DII

#define WRAP_METHOD(modname, meth_name) \
  std::pair<const char*, DII::Method>(#meth_name, \
    DII::GetSlotHandle(modname::instance(), &modname::meth_name))

#define MODULE(modname) new DII::ClassMod(#modname)
#define MODULE_INIT(modname) new DII::ClassMod(#modname, \
  SigC::slot(modname::instance(), &modname::init))
#define MODULE_INIT_DEINIT(modname) new DII::ClassMod(#modname, \
  SigC::slot(modname::instance(), &modname::init), \
    SigC::slot(modname::instance(), &modname::deinit))

#endif // _CLASS_MOD_H_
