#ifndef _DII_H_
#define _DII_H_


#include <string>

#ifdef PACKAGE
#undef PACKAGE
#endif
#ifdef VERSION
#undef VERSION
#endif
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "object.h" 
#include "broadcaster.h"
#include "method.h"
#include "module.h"

namespace DII {

///
class Module;

#ifdef NEED_DII_SERVICES

/**lookup a module.
   @return: pointer to module or NULL.
*/
Module* lookupModule(const char* modname);


/**lookup Method of given modul.
 */
Method lookupMethod(Module* mod, 
		    const char* methname);

/**lookup Method of given modul.
 */ 
Method lookupMethod(const char* modname, 
		    const char* methname);
 
/**lookup and call.
 */
Object callMethod(const char* modname,
			 const char* methname,
			 const DII::Object& in = 0);

/** get list of all available services.
 */
StringSList queryServices();

/**lookup and call.
 */
Object callService(const char* servicename,
		   const DII::Object& in = 0);


/**lookup a service.
 */
Method lookupService(const char* methname);

#endif // NEED_DII_SERVICES

/**register a new service.
   The pointer to the realizing module must be passed, so that
   on unloading of this module, the service gets automatically removed
   from the service database.
   If several services of the same name have been register, lookupService
   will return the latest registered service. Thus service overloading
   is possible. 
*/
void registerService(Module* mod, const char* methname);

/**Register an already defined method as global service.
   This macro should be called from the module's constructor.
*/
#define REGISTER_AS_SERVICE(servname)\
DII::registerService(this,#servname);   

/**Register this member function as method of this module.
   This macro should be called from the module's constructor.
*/
#define REGISTER_METHOD(modname, meth_name) \
methods_[#meth_name]= new DII::FunctionHandle<modname>(this, &modname::meth_name);
 
 
/**Express dependencies.
   This macro should be called from the module's constructor.
*/
#define NEEDS_MODULE(modname) dependencies_.push_front(#modname);

}

#endif
