//  The Worldforge Project
//  Copyright (C) 1998, 1999,2000,2001 The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//
// created by Karsten Laux 03/2001

#include "dii/module.h"
#include "script/lua_script.h"

namespace DII {

///
class ScriptModule : public Module, public Lua::Script
{
 public:
  ///
  ScriptModule(const std::string& name, const std::string& filename, const std::string& mode);
  ///
  virtual ~ScriptModule();
  ///
  DII::Object init(const DII::Object&) {return callMethod("init");}
  ///
  DII::Object deinit(const DII::Object&) {return callMethod("deinit");}
  ///
  virtual bool message(const DII::Object& msg);
  ///
  DII::Object toDIIObject(const Lua::Value&);
  ///
  Lua::Value toLuaValue(const DII::Object&);
  ///
  void setExtensionPath(const std::string& path) { scriptPath_ = path; };

 protected:
  ///
  std::string msgHandler_;
  /** Load another script file into this VM.
     File IO is disabled for our VM, so this method provides
     a "safe" way to load other LUA chunks into this VM's 
     context.
  */
  static int loadExtension(Lua::Script::Handle);
  ///
  static int registerMethod(Lua::Script::Handle);
  ///
  static int registerAsService(Lua::Script::Handle);
  ///
  static int needsModule(Lua::Script::Handle);
  /// Connect scriptfunction (arg[2]) to the signal of another module (arg[1]).
  static int DII_registerMsgHandler(Lua::Script::Handle); 
  /** Emit signal carrying a message.
      Returns 1 (to LUA) if someone has handled this message,
      otherwise 0.
  */
  static int DII_emitMessage(Lua::Script::Handle);
  /// call <arg[1]>::<arg[2]>(<arg[3]>)
  static int DII_callMethod(Lua::Script::Handle); 
  /// call <arg[1]>(<arg[2]>)
  static int DII_callService(Lua::Script::Handle);
  ///
  static int _ALERT(Lua::Script::Handle);
  ///
  static int Module_setOption(Lua::Script::Handle);
  ///
  static int Module_getOption(Lua::Script::Handle);
  ///
  static int Module_registerOption(Lua::Script::Handle);

  /** Configure a module option.
      Overriding this method from baseclass Module in order to forward 
      settings to the script function "configure".
  */
  void configure();
  ///
  std::string scriptPath_;

};

} // namespace DII
