//  The Worldforge Project
//  Copyright (C) 1998, 1999 The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//
// created by Karsten Laux 06/2000


#ifndef _ENVIRONMENT_H_
#define _ENVIRONMENT_H_

#include <uta/timer.h>
#include <uta/sample.h>
#include <uta/surface.h>
#include <uta/music.h>

namespace DII {class Module;}

///
class Environment : virtual public SigC::Object
{
 public:
  
  ///
  Environment();
  ///
  virtual ~Environment() {assert(instance_); instance_ = 0;}

  static Environment* instance() {return instance_ ? instance_ : new Environment();}

  ///
  bool init(DII::Module*);
  ///
  void deinit();

  ///
  bool eventHandler(bool, uta::DirectSurface*);
  ///
  void setTarget(uta::Surface*);
  ///
  bool render();
  ///
  bool weather(const std::string&);

  ///
  void tick();  
  ///
  void fasttick();

 protected:
  ///
  void configure();     
  ///
  enum { DAY, NIGHT, SUNNY, FOG, RAIN, SNOW, THUNDER };
  ///
  void prepare();
  ///
  void pause();
  ///
  void resume();

  ///
  int weather_;
  ///
  int daytime_;
  ///
  uta::Timer timer_;
  ///
  uta::Timer fasttimer_;
  ///
  std::vector< const uta::Sample* > birds_;
  ///
  const uta::Sample*  rain_;
  ///
  uta::Surface* target_;
  ///
  uta::Surface* cache_;
  ///
  uta::Music* thunder_;
  ///
  std::vector <uta::Point> raindrops_;
  ///
  std::vector<int> flakeState_;  
  ///
  struct {
    int start;
    int max;
    int raise;
    int fall;
    double probability;
  } lightning_;
  ///
  int lightlevel_;
  ///
  int lightdelta_;

  static Environment* instance_;
};

#endif
