//  The Worldforge Project
//  Copyright (C) 2002 The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//
// created by Ron Steinke 5/2002

#define NEED_DII_SERVICES

#include "loader.h"
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "class_mod.h"
#include "method_convert.h"

#include "bind_module.h"

#include "music/music.h"
#include "modules/appmod.h"
#include "modules/char_stats.h"
#include "modules/console.h"
#include "dialogs/dialogs.h"
#include "environment/environment.h"
#include "modules/inventory.h"
#include "registry/registry.h"
#include "renderer/renderer.h"
#include "modules/viewport.h"
#include "modules/observer.h"

// ------------------- conversion functions -------------------

static DII::Object
get_message(SigC::Callback1<void,const std::string&> *s, const DII::Object& _msg)
{
  std::string output;

  if(_msg.IsString())
    output = _msg.AsString();
  else if(_msg.IsMap()) 
    {
      const DII::Object::MapType & msg = _msg.AsMap();
      DII::Object::MapType::const_iterator itr = msg.find("text");
      if(itr != msg.end())
        output = itr->second.AsString();
      else if((itr = msg.find("info")) != msg.end())
        output = itr->second.AsString();
      else if((itr = msg.find("message")) != msg.end())
        output = itr->second.AsString();
    }
  else
    return false;

  s->call(output);

  return true;
}

static DII::Object
wrap_inv_update(SigC::Callback1<bool,const std::list<std::string>&> *s,
	const DII::Object& contains)
{
  if(!contains.IsList())
    return false;

  try
    {
      std::list<std::string> ids;
      DII::Object::ListType::const_iterator itr;
      for(itr = contains.AsList().begin(); itr != contains.AsList().end(); itr++)
	ids.push_back(itr->AsString());
      
      return s->call(ids);
    }
  catch(...)
    {
      cerr << "Inventory::update() caught exception !" << endl;
      return false;
    }
}

static DII::Object
wrap_weather(SigC::Callback1<bool, const std::string&> *s, const DII::Object& obj)
{
  std::string whatNow;

  if(obj.IsString())
    whatNow = obj.AsString();
  else if(obj.IsList() && obj.AsList().size() > 0)
    whatNow = obj.AsList().begin()->AsString();

  return s->call(whatNow);
}

static DII::Object
wrap_highlight(SigC::Callback3<void,const std::string&,int,RLE::Color> *s,
	const DII::Object& arg)
{
   if(!arg.IsMap())
      return false;
   
   try
   {
   
     std::string id="";
     int timeout = -1;
     RLE::Color color = 0x00FF;
     uta::Pixelformat format(uta::Pixelformat::RGB565);
     DII::Object::MapType::const_iterator itr;
     const DII::Object::MapType& _arg = arg.AsMap();
     
     if((itr = _arg.find("id")) != _arg.end())
       id = itr->second.AsString();
     if((itr = _arg.find("timeout")) != _arg.end())
       timeout = static_cast<int>(itr->second.AsNum());
     if((itr = _arg.find("color")) != _arg.end())
       if(itr->second.IsList() && itr->second.AsList().size() == 3)
         color = static_cast<RLE::Color>(format.mapToPixel(
               uta::Color((unsigned char)(itr->second.AsList()[0].AsNum()),
                                                                          (unsigned char)(itr->second.AsList()[1].AsNum()),
                          (unsigned char)(itr->second.AsList()[2].AsNum())
                         )));
     
//      cerr << "Renderer::highlight("<<id<<","<<timeout<<")"<<endl;
     
     s->call(id, timeout, color);
   }
   catch(...)
     {
       return false;
     }
   return true;
}

static DII::Object 
wrap_music_vol(SigC::Callback1<void,int> *s, const DII::Object& obj)
{
  int volume_;

  if(obj.IsInt())
    {
      volume_ = obj.AsInt();
    }
  else if(obj.IsString())
    {
      volume_ = atoi(obj.AsString().c_str());
    }
  else if(obj.IsList() && obj.AsList().size() > 0)
    {
      std::string str = obj.AsList().front().AsString();
      volume_ = atoi(str.c_str());
    }

  s->call(volume_);

  return true;
}

// ------------------- module functions ------------------

static DII::Module* Registry_module()
{
  DII::ClassMod *module = MODULE(Registry);

  module->registerMethod(WRAP_METHOD(Registry, open));
  module->registerMethod(WRAP_METHOD(Registry, close));
  //  module->registerMethod(WRAP_METHOD(Registry, set), true);
  module->registerMethod(WRAP_METHOD(Registry, readRegistryEntry), true);
  module->registerMethod(WRAP_METHOD(Registry, writeRegistryEntry), true);
  module->registerMethod(WRAP_METHOD(Registry, flushRegistry), true);

  return module;
}

static DII::Module* App_module()
{
  DII::ClassMod *module = MODULE_INIT(App);

  module->registerMethod(WRAP_METHOD(App, slot_message));
  module->registerMethod(WRAP_METHOD(App, quit), true);
  module->registerMethod(WRAP_METHOD(App, version), true);

  return module;
}

static DII::Module* Console_module()
{
  DII::ClassMod *module = MODULE_INIT_DEINIT(Console);

  module->registerMethod(WRAP_CONVERT_METHOD(Console, std_out, get_message), true);
  module->registerMethod(WRAP_CONVERT_METHOD(Console, std_err, get_message), true);
  module->registerMethod(WRAP_METHOD(Console, showLog), true);
  module->registerMethod(WRAP_METHOD(Console, hideLog), true);

  module->needs("Gui");

  return module;
}

static DII::Module* Inventory_module()
{
  DII::ClassMod *module = MODULE_INIT_DEINIT(Inventory);

  module->registerMethod(WRAP_CONVERT_METHOD(Inventory, update, wrap_inv_update));

  module->needs("Gui");

  return module;
}

static DII::Module* CharStats_module()
{
  DII::ClassMod *module = MODULE_INIT_DEINIT(CharStats);

  module->registerMethod(WRAP_METHOD(CharStats, update));

  module->needs("Gui");

  return module;
}

static DII::Module* Dialogs_module()
{
  DII::ClassMod *module = MODULE_INIT_DEINIT(Dialogs);

  module->registerMethod(WRAP_METHOD(Dialogs, editCharacter), true);
  module->registerMethod(WRAP_METHOD(Dialogs, chooseCharacter), true);
  module->registerMethod(WRAP_METHOD(Dialogs, login), true);
  module->registerMethod(WRAP_METHOD(Dialogs, inventory));
  module->registerMethod(WRAP_METHOD(Dialogs, showMap), true);

  module->needs("Gui");

  return module;
}

static DII::Module* Viewport_module()
{
  DII::ClassMod *module = MODULE_INIT_DEINIT(Viewport);

  module->registerMethod(WRAP_METHOD(Viewport, showViewport), true);
  module->registerMethod(WRAP_METHOD(Viewport, hideViewport), true);
  module->registerMethod(WRAP_METHOD(Viewport, getScreen));

  module->needs("Gui");

  return module;
}

static DII::Module* Environment_module()
{
  DII::ClassMod *module = MODULE_INIT_DEINIT(Environment);

  module->registerMethod(WRAP_METHOD(Environment, render));
  module->registerMethod(WRAP_METHOD(Environment, setTarget));
  module->registerMethod(WRAP_CONVERT_METHOD(Environment, weather, wrap_weather), true);

  module->needs("Viewport");

  return module;
}

static DII::Module* Renderer_module()
{
  DII::ClassMod *module = MODULE_INIT_DEINIT(Renderer);

  module->registerMethod(WRAP_METHOD(Renderer, update));
  module->registerMethod(WRAP_METHOD(Renderer, move));
  module->registerMethod(WRAP_METHOD(Renderer, remove));

  module->registerMethod(WRAP_METHOD(Renderer, observ));
  
  module->registerMethod(WRAP_METHOD(Renderer, setMap));
  module->registerMethod(WRAP_METHOD(Renderer, redraw));
  module->registerMethod(WRAP_METHOD(Renderer, setGamma));
  module->registerMethod(WRAP_METHOD(Renderer, setClouds));
  module->registerMethod(WRAP_CONVERT_METHOD(Renderer, highlight,
    wrap_highlight));

  module->needs("Viewport");
  module->needs("Terrain");
  module->needs("Environment");

  return module;
}

static DII::Module* MusicModule_module()
{
  DII::ClassMod *module = MODULE_INIT_DEINIT(MusicModule);

  module->registerMethod(WRAP_METHOD(MusicModule, play));
  module->registerMethod(WRAP_METHOD(MusicModule, stop));
  module->registerMethod(WRAP_METHOD(MusicModule, pause));
  module->registerMethod(WRAP_METHOD(MusicModule, resume));

  module->registerMethod(WRAP_CONVERT_METHOD(MusicModule, music_vol,
    wrap_music_vol), true);

  return module;
}

static DII::Module* Observer_module()
{
  DII::ClassMod *module = MODULE_INIT(Observer);

  module->registerMethod(WRAP_METHOD(Observer, addID));
  module->registerMethod(WRAP_METHOD(Observer, removeID));

  return module;
}

// ------------------- get_bind_module -------------------

DII::Module* DII::Loader::
get_bind_module(const std::string& filename)
{
  Module* module_ = NULL;

  if(filename == "Registry") // dii internal
    module_ = Registry_module();
  else if(filename == "App") // called on init, shutdown in uclient.cc, some in lua
    module_ = App_module();
  else if(filename == "Gui") // broken out in patch, still called in lua
    module_ = Gui_module();
  else if(filename == "Console") // called only in lua scripts
    module_ = Console_module();
  else if(filename == "Inventory") // broken out in patch, one call in lua
    module_ = Inventory_module();
  else if(filename == "CharStats") // broken out in patch
    module_ = CharStats_module();
  else if(filename == "Dialogs") // never used
    module_ = Dialogs_module();
  else if(filename == "Viewport") // mostly gone, weird signal handler crap left
    module_ = Viewport_module();
  else if(filename == "Player") // broken out in patch, still called in lua
    module_ = Player_module();
  else if(filename == "Environment") // broken out in patch
    module_ = Environment_module();
  else if(filename == "Renderer") // broken out in patch, still called in lua
    module_ = Renderer_module();
  else if(filename == "Music") // unused, there doesn't even appear to be code for this
    module_ = MusicModule_module();
  else if(filename == "Terrain") // broken out in patch
    module_ = Terrain_module();
  else if(filename == "Settings") // used only in lua
    module_ = Settings_module();
  else if(filename == "Observer") // used only in lua
    module_ = Observer_module();

  return module_;
}
