//  The Worldforge Project
//  Copyright (C) 2002 The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//
// created by Ron Steinke 5/2002

#define NEED_DII_SERVICES

#include "loader.h"
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "class_mod.h"
#include "method_convert.h"

#include "bind_module.h"

#include "modules/gui.h"

// ------------------- conversion functions -------------------

static void
MapToDesc(const DII::Object::MapType& arg, Janus::Node& parent)
{
  DII::Object::MapType::const_iterator itr = arg.begin();
  while(itr != arg.end())
    {
      if(itr->second.IsMap())
	{
	  Janus::Node child;
	  // set ID (may get overridden by explicit attribute)
	  child.attributes["id"] = itr->first;

	  // extract type information ...
	  DII::Object::MapType::const_iterator itr2 = 
	    itr->second.AsMap().find("type");
	  if(itr2 !=  itr->second.AsMap().end())
	    if(itr2->second.IsString())
	      child.name = itr2->second.AsString();

	  MapToDesc(itr->second.AsMap(), child);

	  parent.childs.push_back(child);
	}
      else if(itr->second.IsString())
	{
	  parent.attributes[itr->first] = itr->second.AsString();
	}
      else if(itr->second.IsNum())
	{
	  parent.attributes[itr->first] = ftos(itr->second.AsNum());
	}
      else
	{
	  cerr << "Gui::MapToDesc() DII::Object::ListType not supported."
	       << endl;
	}
      itr++;
    }
}

static DII::Object
obj_to_janus_node(SigC::Callback1<std::string,Janus::Node&> *s, const DII::Object &obj)
{
  if(!obj.IsMap())
    return DII::FAILURE;
  
  Janus::Node description;
  description.name = "janus";
  
  // recursively convert DII::Object::MapType to Janus::Node tree
  MapToDesc(obj.AsMap(), description);

  std::string id = s->call(description);

  if(id.empty())
    {
#ifdef DEBUG
      std::cerr << "Gui::create_dialog() failed to create dialog" << std::endl;
#endif
      return DII::FAILURE;
    }

  return id;
}

static DII::Object
wrap_run_dialog(SigC::Callback1<Janus::Values,const std::string&> *s,
		const DII::Object &obj)
{
  if(!obj.IsString())
    return DII::FAILURE;

  Janus::Values vals = s->call(obj.AsString());

  DII::Object::MapType result;
  Janus::Values::iterator itr = vals.begin();
  while(itr != vals.end())
    {
      result[itr->first] = itr->second;
      itr++;
    }

  return result;
}

static DII::Object
wrap_release_menubutton(SigC::Callback1<bool,unsigned int> *s, const DII::Object& obj)
{
  if(!obj.IsMap())
    return false;
  const DII::Object::MapType & param = obj.AsMap();
  DII::Object::MapType::const_iterator itr = param.find("id");
  if (itr == param.end())
    return false;

  return s->call((int)itr->second.AsNum());
}

static DII::Object
wrap_open_progressbar(SigC::Callback1<std::string,const std::string&> *s,
	const DII::Object& obj)
{
  if(!obj.IsString())
    return DII::FAILURE;

  std::string result = s->call(obj.AsString());

  return result.empty() ? DII::FAILURE : result;
}

static DII::Object
wrap_popup_questionbox(SigC::Callback2<bool,const std::string&, const std::string&> *s,
		       const DII::Object& obj)
{
  if(!obj.IsMap())
    return DII::FAILURE;

  const DII::Object::MapType & param = obj.AsMap();

  std::string title; 
  DII::Object::MapType::const_iterator itr = param.find("title");
  if(itr != param.end())
    title = itr->second.AsString();
  std::string question; 
  if((itr = param.find("question")) != param.end())
    question = itr->second.AsString();

  return s->call(title, question) ? "yes" : "no";
}

static DII::Object
wrap_popup_messagebox(SigC::Callback2<void,const std::string&,const std::string&> *s,
	const DII::Object& obj)
{
  if(!obj.IsMap())
    return false;

  const DII::Object::MapType & param = obj.AsMap();

  std::string title; 
  DII::Object::MapType::const_iterator itr = param.find("title");
  if(itr != param.end())
    title = itr->second.AsString();
  std::string message; 
  if((itr = param.find("message")) != param.end())
    message = itr->second.AsString();

  s->call(title, message);

  return true;
}

static DII::Object
wrap_bind_menubutton(SigC::Callback0<unsigned int> *s, const DII::Object&)
{
  unsigned int i = s->call();
  if(i == 0)
    return DII::FAILURE;

  DII::Object::MapType result;
  result["id"] = (int) i;
  return result;
}

static DII::Object
wrap_change_dialog(SigC::Callback4<void,const std::string&, const std::string&,
	const Janus::Values*, const std::string*> *s, const DII::Object& obj)
{
  if(!obj.IsMap())
    return false;
  
  std::string id;
  std::string sub_id;
 
  const DII::Object::MapType& arg = obj.AsMap();
  DII::Object::MapType::const_iterator itr;
  
  try
    {
      Janus::Values attributes;
      std::string value;
      bool have_attrib = false, have_value = false;

      //      cerr << "Gui::change_dialog :" << endl;

      if((itr = arg.find("id")) != arg.end())
	id = itr->second.AsString();
      if((itr = arg.find("sub_id")) != arg.end())
	sub_id = itr->second.AsString();
      if((itr = arg.find("attributes")) != arg.end())
	{
	  have_attrib = true;
	  DII::Object::MapType::const_iterator itr2 = itr->second.AsMap().begin();
	  while(itr2 != itr->second.AsMap().end())
	    {
	      if(itr2->second.IsString())
		attributes[itr2->first] = itr2->second.AsString();
	      else if(itr2->second.IsNum())
		attributes[itr2->first] = ftos(itr2->second.AsNum());
	      
	      //      cerr << itr2->first<<" := " << attributes[itr2->first] << endl;
	      itr2++;
	    }
	}
      if((itr = arg.find("val")) != arg.end())
	{
	  value = itr->second.AsString();
	  have_value = true;
	}
      s->call(id, sub_id, have_attrib ? &attributes : 0, have_value ? &value : 0);
    }
  catch(...)
    {
      cerr << "Gui::change_dialog() rose exception ! " << endl;
      return false;
    }
  return true;
}

static DII::Object
wrap_call_dialog(SigC::Callback4<void,const std::string&,const std::string&,
	const std::string&,const std::string&> *s, const DII::Object& obj)
{
  if(!obj.IsMap())
    return false;
  
  std::string id;
  std::string sub_id;
  std::string cmd;
  std::string param;

  const DII::Object::MapType& arg = obj.AsMap();
  DII::Object::MapType::const_iterator itr;
  
  try
    {
      if((itr = arg.find("id")) != arg.end())
	id = itr->second.AsString();
      if((itr = arg.find("sub_id")) != arg.end())
	sub_id = itr->second.AsString();
      if((itr = arg.find("cmd")) != arg.end())
	cmd = itr->second.AsString();
      if((itr = arg.find("arg")) != arg.end())
	param = itr->second.AsString();

      //      cerr << "Gui::call_dialog("<<id<<","<<sub_id<<","<<cmd<<","<<param<<")"<<endl;

	s->call(id, sub_id, cmd, param);

    }
  catch(...)
    {
      cerr << "Gui::call_dialog() rose exception ! " << endl;
      return false;
    }
  return true;
}

///arg is list: id, value
static DII::Object
wrap_update_progressbar(SigC::Callback2<bool,const std::string&,double> *s,
			const DII::Object& arg)
{
  if(!arg.IsList() && arg.AsList().size() != 2)
    return false;
  DII::Object::ListType::const_iterator itr = arg.AsList().begin();
  std::string id="";
  double val = 0;
  if(itr->IsString())
    id = itr->AsString();
  itr++;
  if(itr->IsNum())
    val = itr->AsNum();

  return s->call(id, val);
}

// ------------------- module functions ------------------

DII::Module* Gui_module()
{
  DII::ClassMod *module = MODULE_INIT_DEINIT(Gui);

  module->registerMethod(WRAP_METHOD(Gui, request_screenarea), true);
  module->registerMethod(WRAP_METHOD(Gui, bind_screenarea), true);
  module->registerMethod(WRAP_METHOD(Gui, release_screenarea), true);
  module->registerMethod(WRAP_CONVERT_METHOD(Gui, popup_messagebox,
     wrap_popup_messagebox), true);
  module->registerMethod(WRAP_CONVERT_METHOD(Gui, popup_questionbox,
     wrap_popup_questionbox), true);

  module->registerMethod(WRAP_CONVERT_METHOD(Gui, bind_menubutton,
    wrap_bind_menubutton));
  module->registerMethod(WRAP_CONVERT_METHOD(Gui, release_menubutton,
    wrap_release_menubutton));
  module->registerMethod(WRAP_METHOD(Gui, change_menubutton));

  module->registerMethod(WRAP_CONVERT_METHOD(Gui, create_dialog,
    obj_to_janus_node));
  module->registerMethod(WRAP_CONVERT_METHOD(Gui, run_dialog,
    wrap_run_dialog));
  module->registerMethod(WRAP_METHOD(Gui, destroy_dialog));
  module->registerMethod(WRAP_METHOD(Gui, show_dialog));
  module->registerMethod(WRAP_METHOD(Gui, hide_dialog));
  module->registerMethod(WRAP_CONVERT_METHOD(Gui, change_dialog,
    wrap_change_dialog));
  module->registerMethod(WRAP_CONVERT_METHOD(Gui, call_dialog,
    wrap_call_dialog));
  module->registerMethod(WRAP_CONVERT_METHOD(Gui, open_progressbar,
    wrap_open_progressbar));
  module->registerMethod(WRAP_CONVERT_METHOD(Gui, update_progressbar,
    wrap_update_progressbar));
  module->registerMethod(WRAP_METHOD(Gui, close_progressbar));

  module->needs("App");

  return module;
}
