//  The Worldforge Project
//  Copyright (C) 1998, 1999 The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//
// created by Karsten Laux 12/1999

#ifndef _LOADER_H_
#define _LOADER_H_

#include "dii/dii.h"
#include <iostream>
#include <set>
#include <list>
#if __GNUC__ < 3
#include <slist>
#else
#include <ext/slist>
#endif

namespace DII {

///
class Loader
{
 public:
  ///
  Loader();
  ///
  virtual ~Loader();
  
  ///
  static void setSearchPath(const std::string& path) { path_ = std::string(path); };
  ///
  static void setScriptPath(const std::string& path) { scriptPath_ = std::string(path); };
  ///
  static bool loadScript(const std::string& name, const std::string& mode = "default");
  ///
  static bool loadModule(const std::string& name);
  ///
  static bool releaseModule(const std::string& modname);
  ///
  static void releaseAll();
  ///
  static void dump(std::ostream &);
  ///
  static void getModuleList(std::list<std::string>&);

 protected:
  /// resolve dependencies, register its services, init the module 
  static bool init_module(DII::Module* module);
  /// bind DLL
  static DII::Module* bind_module(const std::string& filename);
  /// create the instance to bind
  static DII::Module* get_bind_module(const std::string& filename);
  /// 
  static void removeDeps(const std::string&);

 private:
  ///Module ControlBlock
  typedef struct
  {
    ///
    StringSList usedBy;
    ///
    Module* mod;
  } ModuleCB;

  ///
  typedef std::map<std::string, ModuleCB> modulDB;
  ///modulename, dlhandle
  static modulDB modules_;
  ///
  static std::set<std::string> uninitialized_;
  ///
  static std::string path_;
  ///
  static std::string scriptPath_;

};

} // namespace

#endif
