//  The Worldforge Project
//  Copyright (C) 2002 The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//
// created by Ron Steinke 5/2002

#define NEED_DII_SERVICES

#include "loader.h"
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "class_mod.h"
#include "method_convert.h"

#include "bind_module.h"

#include "player/player.h"

// ------------------- conversion functions -------------------

static DII::Object wrap_say(SigC::Callback1<void,const std::string&> *s,
	const DII::Object & obj)
{
    std::string txt;

    if (!obj.IsString()) {
        if (!obj.IsList() || obj.AsList().size() < 1)
            return false;
        txt = obj.AsList().begin()->AsString();
    } else
        txt = obj.AsString();

    s->call(txt);

    return true;
}

static DII::Object return_string_list(SigC::Callback0<std::list<std::string> > *s,
			       const DII::Object&)
{
    std::list < std::string > servers = s->call();

    std::list < std::string >::iterator itr = servers.begin();
    DII::Object::ListType result;
    while (itr != servers.end()) {
        result.push_back(*itr);
        itr++;
    }

    return result;
}

static DII::Object wrap_queryServer(SigC::Callback2<DII::Object,
	const std::string&,int> *s, const DII::Object & arg)
{
    std::string hostname;
    int
     port =
        WFNetwork::GAMESERVER_PORT;

    // parse arg list
    if (arg.IsList() && arg.AsList().size() > 0) {
        DII::Object::ListType::const_iterator itr = arg.AsList().begin();
        if (!itr->IsString())
            return DII::FAILURE;
        hostname = itr->AsString();

        if (itr++ != arg.AsList().end())
            if (itr->IsNum())
                port = (int)itr->AsNum();
    } else
        return DII::FAILURE;

    return s->call(hostname, port);
}

static DII::Object wrap_login(SigC::Callback3<bool,const std::string&,
	const std::string&,const std::string&> *s, const DII::Object & arg)
{
    if (!arg.IsMap())
        return false;

    DII::Object::MapType::const_iterator _itr;
    std::string _id;
    std::string _password;
    std::string _hostname;

    try {
        if ((_itr = arg.AsMap().find("id")) != arg.AsMap().end())
            _id = _itr->second.AsString();
        if ((_itr = arg.AsMap().find("password")) != arg.AsMap().end())
            _password = _itr->second.AsString();
        if ((_itr = arg.AsMap().find("hostname")) != arg.AsMap().end())
            _hostname = _itr->second.AsString();
    }
    catch(...) {
        cerr << "Player::login() invalid account data " << endl;
        return false;
    }

    return s->call(_id, _password, _hostname);
}

static DII::Object wrap_playerLook(SigC::Callback1<void,const std::string&> *s,
	const DII::Object & obj)
{
    std::string id;
    if (obj.IsString())
        id = obj.AsString();
    else if (obj.IsList() && obj.AsList().size() > 0)
        id = obj.AsList().front().AsString();

    s->call(id);

    return true;
}

// ------------------- module functions ------------------

DII::Module* Player_module()
{
  DII::ClassMod *module = MODULE_INIT_DEINIT(Player);

    module->registerMethod(WRAP_CONVERT_METHOD(Player, login, wrap_login));
    module->registerMethod(WRAP_METHOD(Player, logout), true);
    module->registerMethod(WRAP_CONVERT_METHOD(Player, createAccount, wrap_login));
    module->registerMethod(WRAP_METHOD(Player, createCharacter));
    module->registerMethod(WRAP_METHOD(Player, takeOverCharacter));
    module->registerMethod(WRAP_METHOD(Player, currentCharacterID));
    module->registerMethod(WRAP_CONVERT_METHOD(Player, say, wrap_say), true);
    module->registerMethod(WRAP_CONVERT_METHOD(Player, look,
        wrap_playerLook), true);
    module->registerMethod(WRAP_CONVERT_METHOD(Player, action,
        wrap_playerLook), true);
    module->registerMethod(WRAP_CONVERT_METHOD(Player, playerLook,
        wrap_playerLook), true);
    module->registerMethod(WRAP_METHOD(Player, sendOperation));

    module->registerMethod(WRAP_METHOD(Player, queryMemory));
    module->registerMethod(WRAP_CONVERT_METHOD(Player, queryServer,
        wrap_queryServer));
    module->registerMethod(WRAP_CONVERT_METHOD(Player, queryServerList,
        return_string_list));

    module->registerMethod(WRAP_METHOD(Player, getTime));

    module->registerMethod(WRAP_METHOD(Player, selectEntity));

    module->registerMethod(WRAP_METHOD(Player, moveTo));
    module->registerMethod(WRAP_METHOD(Player, drop), true);
    module->registerMethod(WRAP_METHOD(Player, drag), true);
    module->registerMethod(WRAP_METHOD(Player, transfer), true);
    module->registerMethod(WRAP_METHOD(Player, inventory) /*, true*/);
    module->registerMethod(WRAP_METHOD(Player, leftclick));
    module->registerMethod(WRAP_METHOD(Player, rightclick));

    // module->registerMethod(WRAP_METHOD(Player, event));

    module->needs("Dialogs");

  return module;
}
