//
//  The Worldforge Project
//  Copyright (C) 1998,1999  The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//
// created by Karsten Laux 02/2001

#include "bind_module.h"
using namespace DII;

#include <varconf/Config.h>
#include <uta/debug.h>
#include <iostream>

#include "dii/class_mod.h"
#include "dii/method_convert.h"

using std::cerr;
using std::endl;

static DII::Object
wrap_getItem(const DII::Object& _arg)
{
  DII::Object value;
  try
    {
      DII::Object::MapType arg = _arg.AsMap();
      std::string section = arg["section"].AsString();
      std::string key = arg["key"].AsString();

      debug(cerr << "Settings: getting "<<section<<"::"<<key<<" ." << endl;);

      varconf::Variable var = 
	varconf::Config::inst()->getItem(section, key);

      if(var.is_int())
	value = (int)var;
      else if(var.is_bool())
        value = (bool)var;
      else if(var.is_double())
	value = (double)var;
      else 
	value = (std::string)var;
    }
  catch(...)
    {
      debug(cerr<<"Settings::getItem() called with malformed argument."<<endl;);
      value = DII::FAILURE;
    }
  
  return value;
}

static DII::Object
wrap_setItem(const DII::Object& _arg)
{
  try
    {
      DII::Object::MapType arg = _arg.AsMap();

      varconf::Variable var;
      DII::Object value = arg["value"];
      if(value.IsInt())
	var = (int)value.AsInt();
      else if(value.IsFloat())
	var = value.AsFloat();
      else 
	var = value.AsString();
      
      std::string section = arg["section"].AsString();
      std::string key = arg["key"].AsString();
      
      debug(cerr << "Settings: setting "<<section<<"::"<<key<<" ." << endl;);

      varconf::Config::inst()->setItem(section, key, var);
      return true;
    }
  catch(...)
    {
      debug(cerr<<"Settings::setItem() called with malformed argument."<<endl;);
      return false;
    }
}

static void
wrap_varconf_sigv(SigC::Callback1<bool,const DII::Object&> *s,
	const std::string& section, const std::string& key)
{
  DII::Object::MapType msg;
  msg["section"] = section;
  msg["key"] = key;
  
  varconf::Variable var = varconf::Config::inst()->getItem(section, key);
  
  if(var.is_int() || var.is_bool())
    msg["value"] = (int)var;
  else if(var.is_double())
    msg["value"] = (double)var;
  else 
    msg["value"] = (std::string)var;  
  
  s->call(msg);
}

DII::Module* Settings_module()
{
  DII::ClassMod *module = new DII::ClassMod("Settings");

  // Have to use static functions instead of SigC::convert(), since
  // varconf::Config is not a SigC::Object
  module->registerMethod("getItem", new DII::SlotHandle<DII::Object,const DII::Object&>(
	SigC::slot(wrap_getItem)));
  module->registerMethod("setItem", new DII::SlotHandle<DII::Object,const DII::Object&>(
	SigC::slot(wrap_setItem)));

  varconf::Config::inst()->
    sigv.connect(SigC::convert(module->signal.slot(), &wrap_varconf_sigv));

  return module;
}
