//  The Worldforge Project
//  Copyright (C) 2002 The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//
// created by Ron Steinke 5/2002

#define NEED_DII_SERVICES

#include "loader.h"
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "class_mod.h"
#include "method_convert.h"

#include "bind_module.h"

#include "terrain/terrain.h"
#include "world/world.h"
#include "modules/console.h"

#include <uta/debug.h>

// ------------------- conversion functions -------------------

static DII::Object 
wrap_setGroundLevel(const DII::Object& arg)
{
  if(!arg.IsList() && arg.AsList().size() != 3)
    return false;
  try
    {
      double x = arg.AsList()[0].AsNum();
      double y = arg.AsList()[1].AsNum();
      double level = arg.AsList()[2].AsNum();
      
      World::World::inst()->setGroundLevel(x,y,static_cast<World::HeightMap::level_t>(level));            
    }
  catch(...)
    {
      return false;
    }
   
  return true;
}

DII::Object 
wrap_getGroundLevel(const DII::Object& arg)
{
  if(!arg.IsList() && arg.AsList().size() != 2)
    return DII::FAILURE;
  
  double level = 0.0;
  
  try
    {
      double x = arg.AsList()[0].AsNum();
      double y = arg.AsList()[1].AsNum();
      
      level = World::World::inst()->getGroundLevel(x,y);   
    }
  catch(...)
    {
      return DII::FAILURE;
    }

  return level;  
}

static DII::Object
wrap_view(SigC::Callback1<void,const Pixel&> *s, const DII::Object& arg)
{
  if(!arg.IsList() && arg.AsList().size() != 2)
    return false;
  
//  assert(cache_);

  try
    {
      float x = atof(arg.AsList()[0].AsString().c_str());
      float y = atof(arg.AsList()[1].AsString().c_str());
      
      debugN(12, std::cerr << "Terrain::view() switching viewport to ("<<x<<", "<<y<<") " << std::endl;);

      Pixel pix = w2p(Coord(x,y,0));

      s->call(pix);
    }
  catch(...)
    {
      cerr << "exception in Terrain::view()" << endl;
      return false;
    }
  return true;
}

static DII::Object
wrap_setVisibleRange(SigC::Callback1<void,const ViewRect&> *s,
	const DII::Object& obj)
{
  DII::Object::MapType param = obj.AsMap();

  DII::Object::ListType ul = param["upperleft"].AsList();
  DII::Object::ListType lr = param["lowerright"].AsList();
  DII::Object::ListType::const_iterator itr;

  ViewRect range_;

  itr = ul.begin();
  range_.imin = itr->AsInt();
  itr++;
  range_.jmin = itr->AsInt();

  itr = lr.begin();
  range_.imax = itr->AsInt();
  itr++;
  range_.jmax = itr->AsInt();

  s->call(range_);

  return true;
}

static DII::Object
wrap_readMapFile(SigC::Callback2<bool,const std::string&,const Coord&> *s,
	const DII::Object& obj)
{  
  std::string name;
  Coord offset = Coord(0.0,0.0,0.0);

  try
    {
      if(obj.IsString())
        {
          name = obj.AsString();
        }
      else if(obj.IsList() && obj.AsList().size()>0)
        {
          DII::Object::ListType::const_iterator itr = obj.AsList().begin();
          name = (itr++)->AsString();
          if(obj.AsList().size() == 3)
            {
              //2d offset given !
              offset.x = atof(itr->AsString().c_str());
              itr++;
              offset.y = atof(itr->AsString().c_str());
              
                      
            }
          else if(obj.AsList().size() == 4)
            {
              //3d offset given !
              offset.x = atof(itr->AsString().c_str());
              itr++;
              offset.y = atof(itr->AsString().c_str());
              itr++;              
              offset.z = atof(itr->AsString().c_str());
            
            }
        }
      else
        {
          std::cerr << "wrap_readMapFile: args are not a list" << obj.GetType() << std::endl << std::flush;
          return false;
        }
    }
  catch(...)
    {
      if(Console::instance())
        Console::instance()->std_out("readMapFile: wrong parameter !\n");
      std::cerr << "Caught exception in Terrain::readMapFile! " << std::endl <<
			std::flush;
      return false;
    }

  return s->call(name, offset);
}

static DII::Object
wrap_readXmlMapFile(SigC::Callback1<bool,const std::string&> *s,
	const DII::Object& obj)
{  
  std::string name;

  if(obj.IsString())
    name = obj.AsString();
  else if(obj.IsList() && obj.AsList().size() > 0)
    {
      obj.AsList().begin();
      name = obj.AsList().front().AsString();
    }
  else
    return false;

  return s->call(name);
}

// ------------------- module functions ------------------

DII::Module* Terrain_module()
{
  DII::ClassMod *module = MODULE_INIT_DEINIT(Terrain);


  module->registerMethod(WRAP_METHOD(Terrain, setTarget));
  module->registerMethod(WRAP_METHOD(Terrain, prepare));
  module->registerMethod(WRAP_METHOD(Terrain, render));
  module->registerMethod(WRAP_CONVERT_METHOD(Terrain, setVisibleRange,
    wrap_setVisibleRange));
  module->registerMethod(WRAP_CONVERT_METHOD(Terrain, readMapFile,
    wrap_readMapFile), true);
  module->registerMethod(WRAP_CONVERT_METHOD(Terrain, readXmlMapFile,
    wrap_readXmlMapFile), true);
  module->registerMethod(WRAP_METHOD(Terrain, getMap));
  module->registerMethod(WRAP_METHOD(Terrain, getTerrainMap));
  module->registerMethod(WRAP_CONVERT_METHOD(Terrain, view, wrap_view), true);

  // SigC::slot() has trouble with overloaded functions

  module->registerMethod("setGroundLevel",
    new DII::SlotHandle<DII::Object,const DII::Object&>(
      SigC::slot(wrap_setGroundLevel)));
  module->registerMethod("getGroundLevel",
    new DII::SlotHandle<DII::Object,const DII::Object&>(
      SigC::slot(wrap_getGroundLevel)));
/*
  module->registerMethod(WRAP_CONVERT_METHOD(World::World, setGroundLevel,
    wrap_setGroundLevel));
  module->registerMethod(WRAP_CONVERT_METHOD(World::World, getGroundLevel,
    wrap_getGroundLevel));
*/
  return module;
}
