//
//  The Worldforge Project
//  Copyright (C) 1998-2002 The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//

//  written by Michael Koch <oknqueror@gmx.de> in July 2002

#ifndef _CONTENT_REQUEST_H
#define _CONTENT_REQUEST_H

#include <string>

#include <ghttp.h>

#include <sigc++/basic_signal.h>

class ContentRequest : virtual public SigC::Object
{
  public:

    ///
    static void setBaseURL(const std::string& url);
    ///
    static void setCacheDir(const std::string& dir);
    ///
    static const std::string & getCacheDir() {
        return s_cachedir;
    }
    ///
    static void setProxy(const std::string& host, unsigned int port);

    ///
    enum Mode { SYNC = 0, ASYNC = 1 };
    ///
    enum State { DONE = 0, PROCESSING, ERROR };
    ///
    ContentRequest(Mode mode = SYNC);
    ~ContentRequest();
    ///
    State state() const {
        return m_state;
    }
    ///
    void create(const std::string& filename);
    ///
    void process();
    ///
    bool isReady() const {
        return m_state != PROCESSING;
    }

    SigC::Signal1<void, const std::string&> signalMediaAvailable;

  private:

    ///
    void saveFile();
    ///
    std::string fullFilename(std::string relFilename);
    ///
    std::string fullURL(std::string relURL);

    ghttp_request* m_request;
    State m_state;
    std::string m_filename;
    
    static std::string s_baseurl;
    static std::string s_cachedir;
    static std::string s_proxy_host;
    static unsigned int s_proxy_port;
};

#endif // _CONTENT_REQUEST_H
