//
//  The Worldforge Project
//  Copyright (C) 1998, 1999 The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//

//  written by Karsten <klaux@rhrk.uni-kl.de> in August 2000

#include "media_index.h"

#include <fstream>
#include <uta/debug.h>

using std::cerr;
using std::endl;

std::string MediaIndex::emptyString_ = "";
MediaIndex::Index MediaIndex::index_;

const std::string& 
MediaIndex::lookupFilename(const std::string& name)
{
  Index::iterator itr = index_.find(name);
  if(itr != index_.end())
    return itr->second;

  return emptyString_;
}

bool
MediaIndex::initialize(const std::string& install_dir)
{
  std::string indexfile = install_dir+"/forge/media.ind";
 
  std::ifstream mediaInd;
  mediaInd.open(indexfile.c_str());

  if(!mediaInd)
    {
      // cerr << "MediaIndex: Cannot access media index file !" << endl;
      // cerr << "      Run build_media_index.sh to recreate it." 
	   // << endl<< endl;
      return false;
    }

  index_.clear();

  debugN(8,cerr << "MediaIndex: parsing media index file ... ";);
  
  char one_line_c[256];
  std::string one_line;
  int start, end;
  std::string name, filename;
  char c;
  Index::iterator itr;
  //bool new_inmedia2d = false;
  //bool old_inmedia2d = false;
  
  while(!mediaInd.eof())
    {
      mediaInd.get(one_line_c,256,'\n');
      //skip '\n'
      mediaInd.get(c);
      
      if(strlen(one_line_c) > 0)
	{
	  one_line=std::string(one_line_c);
	  end = one_line.find(".png");
	  start = one_line.rfind(" ");
	  
	  name = one_line.substr(0,end);
	  filename = 
	    install_dir+"/forge/"+one_line.substr(start+1,one_line.size()-start-1);

	  itr = index_.find(name);

// 	  debugN(8,
// 		 if(itr != index_.end())
// 	  {
// 	    cerr << "WARNING; media.ind entry "<<name<<" occured more than once !"
// 		 <<endl 
// 		   << "FILE A"<<index_.find(name)->second<<endl
// 		 << "FILE B"<<filename<< endl;
// 	  }
// 		 );
	  
	  if(itr == index_.end())
	    index_[name]=filename;
	  else 
	    {
	      // in case of double indexed file, prefer those from "media-2d"
	      if ( (filename.find("media-2d") != std::string::npos) ||
		   !(itr->second.find("media-2d") != std::string::npos))
		index_[name] = filename;
	      
	    }
	}
    }
  debugN(8,cerr << index_.size() <<" entries. "<<endl;);      
  mediaInd.close();

  return true;
}

