//
//  The Worldforge Project
//  Copyright (C) 1998, 1999 The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//

//  written by Al Riddoch <alriddoch@zepler.org> in July 2002

#ifndef _MEDIA_SERVER_H
#define _MEDIA_SERVER_H

#include "sprite_cache.h"

#include <network/connection.h>
#include <network/msgdispatch.h>

#include <uta/timer.h>

#include <sigc++/object.h>

#include <queue>
#include <set>

class SpriteDesc;
class SpriteFactory;
class ContentRequest;

class SpriteFileSet
{
  public:
    int count;
    SpriteID sprite;

    SpriteFileSet() : count(0) { }
};

class MediaServer : virtual public SigC::Object
{
  public:

    static MediaServer* instance() { return instance_ ? instance_ : new MediaServer(); }
    ///
    void setServer(const std::string& host, unsigned int port, SpriteFactory *);
    ///
    void setContentBaseURL(const std::string& baseurl);
    ///
    void request(const SpriteDesc & sd);
    ///
    bool requestMediaFile(const std::string& filename);
    ///
    bool registerCacheReplacement(SpriteID sprite);
    ///
    const std::queue<std::string> & getContentQueue() const {
        return contentQueue;
    }
    ///
    void addFileSet(const std::set<std::string> & filenames) {
        contentSet = filenames;
    }

  private:

    ///uplink to media server
    SpriteFactory* factory_;
    ///
    uta::Timer timer_;
    ///
    MsgDispatch dispatcher_;
    ///
    Connection connection_;
    ///
    enum { INIT, CONNECTED };
    ///
    int state_;
    ///
    std::string hostname_;
    ///
    int port_;
    ///
    std::string contentBaseURL_;
    ///
    ContentRequest* mediaRequest_;
    ///
    std::queue<std::string> contentQueue;
    ///
    std::set<std::string> contentSet;
    ///
    std::map<std::string, SpriteFileSet *> fileSetMap;

    ///
    MediaServer();
    ///
    ~MediaServer();
    ///
    void tick();
    ///
    void disconnected();
    ///
    bool connect();
    ///
    void OpInfo(const Atlas::Objects::Operation::Info&);
    ///
    void OpError(const Atlas::Objects::Operation::Error&);

    static MediaServer* instance_;
    ///
    void mediaAvailable(const std::string& filename);
};

#endif // _MEDIA_SERVER_H
