#ifndef _RAW_H_
#define _RAW_H_

/* magic string for old (but currently used raw file format) */
#define RAW_MAGIC "SPRITERAW"

/* magic string for new rle compressed raw file format */
#define RAWRLE_MAGIC "SPRRLERAW"

/* Fileformat for storing already decoded sprites.

   "SPRITERAW - Format"
   ------------------------------
   (see below for the new format "SPRRLERAW" )

   Format of file:
   The first 9 bytes contain "SPRITERAW" followed by a '\0' character.
   At offset 10 the struct RawHeader is written, followed by the pixeldata.
   The frames of the sprite are stored directly following each other to
   the file.
   All frames of an animation in a row, one row for each direction.
   If the sprite is layered, this block is repeated for every layer.
   
   A sprite with 2 layers and four directions, each containing an animation
   of for frames would look like this:

   L0D0F0 L0D0F1 L0D0F2 L0D0F3 L0D0F4
   L0D1F0 L0D1F1 L0D1F2 L0D1F3 L0D1F4
   L0D2F0 L0D2F1 L0D2F2 L0D2F3 L0D2F4
   L0D3F0 L0D3F1 L0D3F2 L0D3F3 L0D3F4
   L1D0F0 L1D0F1 L1D0F2 L1D0F3 L1D0F4
   L1D1F0 L1D1F1 L1D1F2 L1D1F3 L1D1F4
   L1D2F0 L1D2F1 L1D2F2 L1D2F3 L1D2F4
   L1D3F0 L1D3F1 L1D3F2 L1D3F3 L1D3F4
     
   The byte offset to access the pixeldata of a sprite for a given
   layer, direction and frame calculates as follows:
   
   BPP = pixelformat.BytesPerPixel;
   orientations is either 1,4, or 8 depending on header.spriteType

   spriteWidth = header.width / header.frames;
   spriteHeight = header.height / (header.layers * orientations)
   spritePitch = header.width * BPP;
   
   offset = 
      <layer> * orientations * spriteWidth * header.width*BPP
      + <direction> * spriteWidth * header.width * BPP
      + <frame> * spriteWidth * BPP;
   
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

   "SPRRLERAW - Format"
   ------------------------------
   (see below for the new format "SPRRLERAW" )

   Format of file:
   The first 9 bytes contain "SPRRLERAW" followed by a '\0' character.
   At offset 10 the struct RawHeader is written.

   IMPORTANT: 
   The width and height entries have a different meaning !!!
   Now width and height are the geometries of a single frame and not of the
   compound surface.

   In contrast to the old format now a table of follows. The table contains
   entries of the type RLE::Surface

   typedef struct
   {
     size_t length;  // run length of encoded image
     void* data;     // in memory: absolute address of compressed pixels
                     // in file:   offset inside file
     int width;      // width of uncompressed image
     int height;     // height of uncompressed image
   } Surface;
   
   IMPORTANT: "data" has to get converted when saving and when loading !
   
   The table itself is organized just as the frames in the SPRITERAW format:
   
   L0D0F0 L0D0F1 L0D0F2 L0D0F3 L0D0F4
   L0D1F0 L0D1F1 L0D1F2 L0D1F3 L0D1F4
   L0D2F0 L0D2F1 L0D2F2 L0D2F3 L0D2F4
   L0D3F0 L0D3F1 L0D3F2 L0D3F3 L0D3F4
   L1D0F0 L1D0F1 L1D0F2 L1D0F3 L1D0F4
   L1D1F0 L1D1F1 L1D1F2 L1D1F3 L1D1F4
   L1D2F0 L1D2F1 L1D2F2 L1D2F3 L1D2F4
   L1D3F0 L1D3F1 L1D3F2 L1D3F3 L1D3F4
    
   So the entry calculates as follows:
   entry_offset = ((<layer> * orientations + <direction>) * frames + <frame>) 
                  * sizeof(RLE::Surface);


*/   
class RawFormat
{
 public:

  /// 
  enum  
  {
    FLAG_LOOPABLE = 1 << 0,
    FLAG_CONTAINER = 1 << 1
  };

  ///
  typedef struct 
  {
    /// when was the file last updated
    long int timestamp;
    /// with of surface
    int width;
    /// height of surface
    int height;
    /// libuta id for pixelformat
    int pixelformat;
    ///
    int spriteFormat;
    ///
    int spriteType;
    ///
    int layers;
    ///
    int frames;
    
  } Header;

};

#endif // !_RAW_H_


