//
//  The Worldforge Project
//  Copyright (C) 1998, 1999 The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//

//  written by Karsten <klaux@rhrk.uni-kl.de> in May 2000

#ifndef _SPRITE_FACTORY_H
#define _SPRITE_FACTORY_H

#include "sprite.h"
#include "toolbox/local_hash_map.h"
#include <dii/object.h>

#include <map>
#include <list>

///
class SpriteDesc
{
 public:
  std::string type;
  std::string mode;
  std::string sex;
  std::string status;
  std::string size;
  
  ///used for passing floating values
  std::map<std::string, double> values;

  std::string asString() const 
    { return type+"."+mode+"."+sex+"."+status+"."+size; };
};

///
struct compDef
{
  bool operator()(SpriteDesc d1, SpriteDesc d2) const
  {
    return strcmp(d1.asString().c_str(),d2.asString().c_str()) < 0;
  }
};

///
class SpriteFactory
{
 public:
  
  ///
  SpriteFactory();
  ///
  ~SpriteFactory();
  
  ///
  SpriteDesc findMatch(const SpriteDesc&);
  ///
  Sprite* create(const SpriteDesc&);
  ///
  void destroy(Sprite*, const SpriteDesc&);
  ///
  void installSpriteDefinition(const std::string &, const DII::Object &);

 protected:
  ///
  unsigned classify(const SpriteDesc& desc);
  ///recursively find best matching sprite
  bool walker(SpriteDesc desc, SpriteDesc& result);  
  ///
  std::map<unsigned, SpriteDesc> matches_;
  ///filename prefix == install_dir + forge
  std::string spriteFileRoot_;
  ///cache filename prefix
  std::string spriteCacheRoot_;
  ///
  typedef struct 
  {
    ///filename + path, but without extension
    std::string filenameBase;
    ///e.g. ".png"
    std::string filenameSuffix;
    ///if > 0, <skip> frames will be skipped 
    int skip;
    /// > 1 means layered sprite
    unsigned layers;   
    /// in which layer shall the content of a sprite be blitted ?
    unsigned contentLayer; 
    ///which layer is the first roof layer ?
    unsigned roofLayer; 
    ///number of first animation frame
    unsigned first_frame;
    ///last frame of animation animated = (last_frame != first_frame)
    unsigned last_frame;
    ///valid values or 8,5,4, or 1
    unsigned directions; 
    ///how many ms between two frames of an animation
    unsigned delay;
    ///is this a loopable animation ?
    bool loopable;
    ///may this sprite display contained items ? (e.g an open box)
    bool container;
    ///shall this animation restart at random times ?
    bool isIdleAnimation;
    ///shall the animation get played forwards,backwards,forwards,... ?
    bool reversable;
    ///at which frames shall the sound be played
    std::list<unsigned int> soundSyncFrames;
    ///which sound shall be played
    std::string soundFile;

  } SpriteDefinition;

  ///
  typedef STRING_HASH_MAP(SpriteDefinition) DefinitionDB;
  ///
  DefinitionDB definitions_;
  ///
  Sprite* buildSprite(DefinitionDB::iterator def);
  ///
  void convertFormat(Image*);
  ///
  bool loadSpriteDefinitions();
  ///
  enum { NATIVE, INDEXED, DISPLAY };
  ///
  int spriteFormat_;

};

#endif
