//
//  The Worldforge Project
//  Copyright (C) 1998,1999  The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//
// created by Karsten Laux 01/1999

#include "appmod.h"

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <sigc++/convert.h>

#include <uta/debug.h>
#include <uta/resources.h>
#include <uta/version.h>

#include <varconf/Config.h>
#include "operation.h"

#include <uta/application.h>

#include "dii/module.h"

#include "../player/player.h"

#include "gui.h"

using namespace uta;

#include "ftos.h"

class App* App::instance_ = 0;

void
App::slot_message(const std::string &str)
{
  cerr << "INFO: " << str << std::endl;
}

void
App::quit()
{
 
  const std::string msg = "Disconnect from the server.\n"
                          "Your character will stay in game.\n\n"
			  "Are you absolutely sure ?";

  
  if(Gui::instance()->popup_questionbox(std::string(), msg))
    {
      //be nice to the server ...
      Player::instance()->logout();
      
      uapp->quit();
    }
}

std::string
App::version()
{
  SDL_version compiled;
  SDL_VERSION(&compiled);

  std::string info = 
    (std::string)"\nUClient Version "+VERSION+" "
    "\n- statically linked - \n"
    "    libuta Version "+
    itos(getVersion()->major)+"."+
    itos(getVersion()->minor)+"."+
    itos(getVersion()->patch)+" loaded\n"+
    "    libuta Version "+
    itos(UTA_MAJOR_VERSION)+"."+
    itos(UTA_MINOR_VERSION)+"."+
    itos(UTA_PATCHLEVEL)+" compiled\n"+
//     "    SDL Version "+    
//     itos(::SDL_Linked_Version()->major)+"."+
//     itos(::SDL_Linked_Version()->minor)+"."+
//     itos(::SDL_Linked_Version()->patch)+" loaded.\n"+
    "    SDL Version "+    
    itos(compiled.major)+"."+
    itos(compiled.minor)+"."+
    itos(compiled.patch)+" compiled \n"+
#ifdef ATLAS_OBJECT_HAS_LONG 
    "    Atlas-C++ Version 0.4\n";
#else
    "    Atlas-C++ Version 0.3\n";
#endif
    
  return info;
}

// See sigc++ "powerusers" documentation
static void convert_uint_to_int(SigC::Callback1<bool,const DII::Object&> *s,
	unsigned int u)
{
  s->call((int) u);
}

bool
App::init(DII::Module *module)
{
  tick_.alarm1.connect(SigC::convert(module->signal.slot(), convert_uint_to_int));
  
  return true;
}
