//
//  The Worldforge Project
//  Copyright (C) 1998,1999  The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//
// created by Karsten Laux 07/2000

#include "char_stats.h"
using namespace DII;

#include <uta/debug.h>
#include <uta/resources.h>
#include "settings.h"
#include "operation.h"

#include "gui.h"

CharStats* CharStats::instance_;

void
CharStats::deinit()
{
  if(button_)
    {
      debugN(3,cerr << "CharStats: successfully looked up Module Gui."<<endl;);

      uta::Rect r = button_->getRect();

      DII::Object::MapType screenarea;
      
      screenarea["NAME"] = std::string("screenarea");
      screenarea["section"] = 3;
      screenarea["x"] = r.upperLeft().x;
      screenarea["y"] = r.upperLeft().y;
      screenarea["width"] = r.width();
      screenarea["height"] = r.height(); 
      
      debugN(3,cerr << "CharStats: releasing screenarea."<<endl;);

      Gui::instance()->release_screenarea(screenarea);
   
      delete button_;
      button_ = 0;
    }
}

bool
CharStats::init(DII::Module*)
{
  DII::Object out;
  
  DII::Object::MapType in;
  in["NAME"] = std::string("screenarea");
  in["section"] = 3; 

  out = Gui::instance()->request_screenarea(in);

  if(!doesContain(out,"section"))
    {
      cerr << "CharStats: failed to obtain screenarea."<<endl;
      return false;
    }

  //discard old input and use old output as new input
  in = out.AsMap();
  
  //actually bind the screenarea
  out = Gui::instance()->bind_screenarea(in);
  //assert(success);
  
  if(!doesContain(out,"section"))
    {
      cerr << "CharStats: failed to bind screenarea."<<endl;
      return false;
    }

  int x = out.AsMap()["x"].AsInt();
  int y = out.AsMap()["y"].AsInt();
  int w = out.AsMap()["width"].AsInt();
  int h = out.AsMap()["height"].AsInt();

  uta::Widget* rootwin = objToPtr<uta::Widget*>(out.AsMap()["parent_widget"]);

  
  debugN(3, cerr << "CharStats: initializing Widget...";);
 
  button_ = new uta::Button(rootwin,x,y,w,h);

  DII::Object tmp = readState("portrait");
  if(tmp.IsString())
    button_->setBackground(SURFACE_RES(tmp.AsString()));
  else
    button_->setBackground(SURFACE_RES("portrait_0"));

  button_->setAutoDelete(false);

  return true;
}



void
CharStats::update()
{
  if(!button_)
    return;

  DII::Object tmp = readState("portrait");
  if(tmp.IsString())
    button_->setBackground(SURFACE_RES(tmp.AsString()));
  else
    button_->setBackground(SURFACE_RES("portrait_0"));
}
