//  The Worldforge Project
//  Copyright (C) 1998, 1999 The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//
// created by Karsten Laux 01/2000


#ifndef _CONSOLE_H_
#define _CONSOLE_H_

#include <Atlas/Objects/Operation/RootOperation.h>
#include <sigc++/object.h>

#include "panel.h"
#include "dialog.h"

#include <uta/multilineedit.h>
#include <uta/event.h>

namespace DII {class Module;}

///
class LogWindow : public Screen
{
 protected:
  ///
   std::string txt_;

 public:
  ///
  LogWindow(const std::string& txt) : Screen(), txt_(txt) {};
  ///
  virtual ~LogWindow() {};
  ///
  void addText(const char* txt)
    {
      if(widget_) ((uta::MultiLineEdit*)(widget_))->addText(txt);
    } 
 protected:
  ///
  uta::Widget* create(uta::Widget* p, int x, int y, int w, int h)
  {
    uta::MultiLineEdit* widget = new uta::MultiLineEdit(p,x,y,w,h);
    widget->setText(txt_.c_str());
    return widget;
  };
};

///
class Console : virtual public SigC::Object
{
  
 public:
  
  ///
  Console() : panel_(0), logWindow_(0) {assert(!instance_); instance_ = this;}
  ///
  virtual ~Console() {assert(instance_); instance_ = 0;}
  
  ///
  void callCommand(const std::string& cmd, const std::string& param="");

  ///
  bool init(DII::Module*);
  ///
  void deinit();
  ///
  static Console* instance() {return instance_ ? instance_ : new Console();}

  /**standard output.
     accepts Atlas::String
  */
  void std_out(const std::string&);
  /**standard error output.
     accepts Atlas::String
  */
  void std_err(const std::string&);
  
  ///
  bool userinput(const Atlas::Objects::Operation::RootOperation&);
  ///
  bool showLog();
  ///
  bool hideLog();
 
  ///
  std::string slot_userinput(const std::string&);

 protected:

  ///
  Panel* panel_;
  ///
  LogWindow* logWindow_;

  static Console* instance_;
};


#endif
