//
//  The Worldforge Project
//  Copyright (C) 1998,1999  The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//  
// created by Karsten Laux 01/1999

#define NEED_DII_SERVICES

// these sizes depend on the used images
#define PANEL_HEIGHT 90
#define SIDEPANEL_WIDTH 72   
          // changed from 70 to 72 because of bug in uta/blitters.cc
#define BORDER_HEIGHT 16
#define BORDER_WIDTH 32
#define BORDER_SIZE 16
#define EDGE_SIZE 16
#define DIALOG_BORDER 5
#define BUTTON_HEIGHT 25
#define BUTTON_WIDTH 25

#define IND_WIDTH 43
//#define SEP_WIDTH 8
#define SEP_WIDTH 0

#define MENU_BUTTONSIZE 30
 
// this is the dynamic part of the layout
#define SCREEN_HEIGHT rootwin->height()
#define SCREEN_WIDTH rootwin->width()
 
// Viewport Size
// 682
#define VIEWPORT_WIDTH (SCREEN_WIDTH-SIDEPANEL_WIDTH-3*BORDER_SIZE)
// 462
#define VIEWPORT_HEIGHT (SCREEN_HEIGHT-PANEL_HEIGHT-3*BORDER_HEIGHT)
#define VIEWPORT_WIDTH_HALF VIEWPORT_WIDTH/2-8
 
 
// 333
#define VIEW_WIDTH VIEWPORT_WIDTH_HALF+16
// 462
#define VIEW_HEIGHT VIEWPORT_HEIGHT
 
#define VIEW_X EDGE_SIZE
#define VIEW_Y EDGE_SIZE    

#define PANEL_Y rootwin->height()-PANEL_HEIGHT-BORDER_HEIGHT


#include <sigc++/convert.h>

#include <uta/resources.h>
#include <uta/label.h>
#include <uta/pushbutton.h>
#include <uta/rootwindow.h>
#include <uta/questionbox.h>
#include <uta/messagebox.h>
#include <uta/application.h>

using namespace uta;

#include "gui.h"
#include "operation.h"
#include "viewport.h"

#include "dii/dii.h"

using namespace DII;
  
void GuiCallback::call(const std::string& dialog_id, 
		    const std::string& cmd, 
                    const Janus::Value& val)
  {
    // build argument .. simply passing all values:
    Janus::Values vals = engine_->getValues(dialog_id);
    DII::Object::MapType arg;
    Janus::Values::iterator itr = vals.begin();
    while(itr != vals.end())
      {
	arg[itr->first] = itr->second;
	itr++;
      }

    arg["value"] = val;

    // determine which method to call
    size_t p = cmd.find(':');
    if(p == std::string::npos)
      {
	// treat cmd as service identifier...
	//	cerr << "GuiCallback::call() calling "<<cmd<<endl;
	DII::callService(cmd.c_str(), arg);
      }
    else
      {
	std::string mod = cmd.substr(0,p);
	std::string method = cmd.substr(p+1,cmd.size()-p-1);
	//cerr << "GuiCallback::call() calling "<<mod<<"::"<<method<<endl;
	DII::callMethod(mod.c_str(), method.c_str(), arg);
      }
    
  }

class Gui* Gui::instance_ = 0;

Gui::Gui() :
  rootwin(NULL),
  callback_(NULL)
{
  assert(!instance_);
  instance_ = this;

  available_.resize(6);
  bound_.resize(6);

  Janus::init_libuta();
}

static bool
button_convert(SigC::Callback1<bool, const DII::Object&> *s, const Operation &o)
{
  return s->call(o.AsObject());
}

bool
Gui::init (DII::Module *module)
{
  debugN(13, cerr << "Gui: initializing ... " << endl;);

  create();
  
  callback_ = new GuiCallback(&engine_);

  debugN(13, cerr << "Gui: done initializing." << endl;);

  buttonClicked.connect(SigC::convert(module->signal.slot(), button_convert));
 
 return true;
}

void
Gui::deinit ()
{
  debugN(13, cerr << "Gui: shutting down ... ";);
  
  if(callback_)
    delete callback_;

  debugN(13, cerr << " [ok]" << endl;);
}

PushButton*
Gui::createMenuButton(int ind)
{
  
  int max = (PANEL_HEIGHT / BUTTON_HEIGHT) ;
  int n;
  PushButton* but;

  if(ind >= max*2)
    {
      n = ind - max*2;
      if(n >= max)
	{
	  n -= max;
	  but= new PushButton(rootwin,
			      rootwin->width()-SIDEPANEL_WIDTH-2*BORDER_HEIGHT-MENU_BUTTONSIZE,
			      PANEL_Y+n*MENU_BUTTONSIZE,
			      MENU_BUTTONSIZE, MENU_BUTTONSIZE, MENU_FIRSTBUTTON+ind);
	}
      else
	{
	  but= new PushButton(rootwin,
			      rootwin->width()-SIDEPANEL_WIDTH-2*BORDER_HEIGHT-2*MENU_BUTTONSIZE,
			      PANEL_Y+n*MENU_BUTTONSIZE,
			      MENU_BUTTONSIZE, MENU_BUTTONSIZE, MENU_FIRSTBUTTON+ind);
	}
      but->setUpPic(SURFACE_RES("menu_button"));
      but->setDownPic(SURFACE_RES("menu_button_pressed"));
    }
  else
    {
      n = ind;
      if(n>= max)
	{
	  n-=max;
	  but= new PushButton(rootwin,
			      BORDER_HEIGHT+MENU_BUTTONSIZE,
			      PANEL_Y+n*MENU_BUTTONSIZE,
			      MENU_BUTTONSIZE, MENU_BUTTONSIZE, MENU_FIRSTBUTTON+ind);
	}
      else
	{
	  but= new PushButton(rootwin,
			      BORDER_HEIGHT,
			      PANEL_Y+n*MENU_BUTTONSIZE,
			      MENU_BUTTONSIZE, MENU_BUTTONSIZE, MENU_FIRSTBUTTON+ind);
	}

      but->setUpPic(SURFACE_RES("menu_button"));
      but->setDownPic(SURFACE_RES("menu_button_pressed"));
    }

  but->setColor(lightgrey);
  but->clicked1.connect(SigC::slot(this, &Gui::buttonClick));
  
  return but;
}

void
Gui::create()
{

  PushButton* scroll_up;
  PushButton* scroll_down;
 
  DII::Object in, out;
  
  rootwin = uta::Application::Instance->rootWindow();
  //Resources* res = uta::Resources::instance;
  
  assert(rootwin);
  //assert(res);

  int menusize = (PANEL_HEIGHT / BUTTON_HEIGHT) * 4;
  menu_.resize(menusize);

  for(int n=0; n < menusize; n++)
    {
      menu_[n].used = false;
      menu_[n].button = createMenuButton(n);
    }

  Rect panelslotRect = Rect(BORDER_SIZE+2*MENU_BUTTONSIZE + SEP_WIDTH+ IND_WIDTH,
			    rootwin->height()-PANEL_HEIGHT-BORDER_HEIGHT,
			    rootwin->width()-3*BORDER_SIZE-SIDEPANEL_WIDTH-MENU_BUTTONSIZE*4-SEP_WIDTH-IND_WIDTH,
			    PANEL_HEIGHT);
/*
  Rect panelslotRect = Rect(BORDER_SIZE+2*MENU_BUTTONSIZE + SEP_WIDTH+ IND_WIDTH,
			    rootwin->height()-PANEL_HEIGHT-BORDER_HEIGHT,
			    rootwin->width()-3*BORDER_SIZE-SIDEPANEL_WIDTH-MENU_BUTTONSIZE*4-2*SEP_WIDTH-2*IND_WIDTH,
			    PANEL_HEIGHT);
*/ 
  Rect indicatorLeftRect = Rect(BORDER_SIZE+2*MENU_BUTTONSIZE,
			    rootwin->height()-PANEL_HEIGHT-BORDER_HEIGHT,
			    SEP_WIDTH+ IND_WIDTH,
			    PANEL_HEIGHT);
/*
  Rect indicatorRightRect = Rect(2*MENU_BUTTONSIZE+ rootwin->width()-2*BORDER_SIZE-SIDEPANEL_WIDTH-MENU_BUTTONSIZE*4-SEP_WIDTH-IND_WIDTH,
			    rootwin->height()-PANEL_HEIGHT-BORDER_HEIGHT,
			    SEP_WIDTH+ IND_WIDTH,
			    PANEL_HEIGHT);
*/

  Rect sideslotRect = Rect(rootwin->width()-SIDEPANEL_WIDTH-BORDER_SIZE,
			   BORDER_HEIGHT,
			   SIDEPANEL_WIDTH,
			   rootwin->height()-3*BORDER_HEIGHT-PANEL_HEIGHT);
 
  
  Rect viewportRect = Rect(BORDER_SIZE,BORDER_HEIGHT,
			   VIEWPORT_WIDTH, VIEWPORT_HEIGHT);


  Rect portraitRect = Rect(rootwin->width()-SIDEPANEL_WIDTH-BORDER_SIZE,
			   rootwin->height()-PANEL_HEIGHT-BORDER_HEIGHT,
			   SIDEPANEL_WIDTH,
			   PANEL_HEIGHT);


  available_[0].push_front(viewportRect);
  available_[1].push_front(sideslotRect);
  available_[2].push_front(panelslotRect);
  available_[3].push_front(portraitRect);
  available_[4].push_front(indicatorLeftRect);
//  available_[5].push_front(indicatorRightRect); 
  available_[5].push_front(Rect(0, 0, 0, 0)); 

  Widget* edge;

  //the four edges of the screen
  edge = new Widget(rootwin,
		     0,0,
		     EDGE_SIZE,EDGE_SIZE);
  edge->setBackground(SURFACE_RES("border_edge"));
  edge = new Widget(rootwin,
		     0,rootwin->height()-EDGE_SIZE,
		     EDGE_SIZE,EDGE_SIZE);
  edge->setBackground(SURFACE_RES("border_edge"));
  edge = new Widget(rootwin,
		     rootwin->width()-EDGE_SIZE, rootwin->height()-EDGE_SIZE,
		     EDGE_SIZE,EDGE_SIZE);
  edge->setBackground(SURFACE_RES("border_edge"));
  edge = new Widget(rootwin,
		     rootwin->width()-EDGE_SIZE,0,
		     EDGE_SIZE,EDGE_SIZE);
  edge->setBackground(SURFACE_RES("border_edge"));
  
  //the four aditional edges of the viewport and character panel
  edge = new Widget(rootwin,
		     VIEWPORT_WIDTH+BORDER_SIZE,VIEWPORT_HEIGHT+BORDER_HEIGHT,
		     EDGE_SIZE,EDGE_SIZE);
  edge->setBackground(SURFACE_RES("border_edge"));

  //fill up the rest of the border
  Widget* border;
  
  //below viewport
  border = new Widget(rootwin,
		       EDGE_SIZE,VIEWPORT_HEIGHT+BORDER_HEIGHT,
		       VIEWPORT_WIDTH,BORDER_HEIGHT);
  border->setBackground(SURFACE_RES("border1_h"));
  scroll_up=new PushButton(border,
			VIEWPORT_WIDTH/2-8,0,
			EDGE_SIZE, EDGE_SIZE, SCROLLUP_BUTTON);
  scroll_up->setUpPic(SURFACE_RES("triangle_up"));
  scroll_up->setDownPic(SURFACE_RES("triangle_up_pressed"));

  scroll_up->clicked1.connect(SigC::slot(this, &Gui::buttonClick));  
  
  //right side of viewport
  border = new Widget(rootwin,
		       VIEWPORT_WIDTH+EDGE_SIZE,EDGE_SIZE,
		       BORDER_HEIGHT,VIEWPORT_HEIGHT);
  border->setBackground(SURFACE_RES("border1_v"));

  //between sidepanel and charspanel
  border = new Widget(rootwin,
		       VIEWPORT_WIDTH+2*EDGE_SIZE,VIEWPORT_HEIGHT+EDGE_SIZE,
		       SIDEPANEL_WIDTH,BORDER_HEIGHT);
  border->setBackground(SURFACE_RES("border1_h"));
 
  // left of chars panel
  border = new Widget(rootwin,
		      rootwin->width()-2*BORDER_HEIGHT-SIDEPANEL_WIDTH, 
		      rootwin->height()-PANEL_HEIGHT-BORDER_HEIGHT,
		      BORDER_HEIGHT, PANEL_HEIGHT);
  border->setBackground(SURFACE_RES("border1_v"));

  // frame around the screen
  border = new Widget(rootwin,
		      BORDER_HEIGHT,0,rootwin->width()-2*BORDER_HEIGHT, BORDER_HEIGHT);
  border->setBackground(SURFACE_RES("screen_frame_h"));

  border = new Widget(rootwin,
		      BORDER_HEIGHT,rootwin->height()-BORDER_HEIGHT,rootwin->width()-2*BORDER_HEIGHT, BORDER_HEIGHT);
  border->setBackground(SURFACE_RES("screen_frame_h"));
  scroll_down=new PushButton(border,
			VIEWPORT_WIDTH/2-8,0,
			EDGE_SIZE, EDGE_SIZE, SCROLLDOWN_BUTTON);
  scroll_down->setUpPic(SURFACE_RES("triangle_down"));
  scroll_down->setDownPic(SURFACE_RES("triangle_down_pressed"));

  scroll_down->clicked1.connect(SigC::slot(this, &Gui::buttonClick));  

  border = new Widget(rootwin,
		      0,BORDER_HEIGHT,BORDER_HEIGHT,rootwin->height()-2*BORDER_HEIGHT);
  border->setBackground(SURFACE_RES("screen_frame_v"));

  border = new Widget(rootwin,
		      rootwin->width()-BORDER_HEIGHT,BORDER_HEIGHT,BORDER_HEIGHT,rootwin->height()-2*BORDER_HEIGHT);
  border->setBackground(SURFACE_RES("screen_frame_v"));

}

void
Gui::buttonClick(int id)
{
  DII::Object::MapType msg;  
  msg["what"] = std::string("clicked");

  switch(id)
    {
    case SCROLLDOWN_BUTTON:
      msg["button"] = std::string("scrolldown");
      break;
    case SCROLLUP_BUTTON:
      msg["button"] = std::string("scrollup");
      break;
    case SIDE_BUTTON1:
      msg["button"] = std::string("side_1");
      break;
    case SIDE_BUTTON2:
      msg["button"] = std::string("side_2");
      break;
    case SIDE_BUTTON3:
      msg["button"] = std::string("side_3");
      break;
    case SIDE_BUTTON4:
      msg["button"] = std::string("side_4");
      break;
    default:
      if(id >= MENU_FIRSTBUTTON)
	{
	  msg["button"] = std::string("menu");
	  msg["id"] =  id - MENU_FIRSTBUTTON + 1;
	}
      
      break;
    }

  Operation op = Operation::Instantiate();
  op.SetParents(DII::Object::ListType(1,std::string("userinput")));
  op.SetFrom("Gui");
  op.GetArgs().push_back(msg);

  buttonClicked.emit(op);
}

DII::Object
Gui::request_screenarea(const DII::Object& obj)
{
  if(!obj.IsMap())
    return DII::FAILURE;

  DII::Object::MapType result;
  const DII::Object::MapType& param = obj.AsMap();
  DII::Object::MapType::const_iterator ptr;
  
  // Can this ever be negative?
  int sect;

  ptr = param.find("section");
  if(ptr != param.end())
    sect = (int)ptr->second.AsNum();
  else
    return DII::FAILURE;

  Rect r;


  // these are optional
  int x=0,y=0,w=0,h=0;
      
  if((ptr = param.find("x")) != param.end())	  
    x = (int)ptr->second.AsNum();
  if((ptr = param.find("y")) != param.end())	  
    y = (int)ptr->second.AsNum();
  if((ptr = param.find("width")) != param.end())	  
    w = (int)ptr->second.AsNum();
  if((ptr = param.find("height")) != param.end())	  
    h = (int)ptr->second.AsNum();
  
  if(w & h)
    r = Rect(x,y,w,h);
  else
    r = Rect(0,0,0,0);
  
  
  debugN(13,cerr << "requested area from section "<<sect << endl;);

  if((unsigned int)sect < available_.size() && available_[sect].size() > 0)
    {
      r = available_[sect].front();
      //need to add check whether the requested rect
      //is actually available
      
      result["NAME"] = std::string("screenarea");
      result["section"] = sect;
      result["x"] = r.upperLeft().x;
      result["y"] = r.upperLeft().y;
      result["width"] = r.width();
      result["height"] = r.height(); 
      
      return result;
    }

  return DII::FAILURE;
}


DII::Object
Gui::bind_screenarea(const DII::Object& obj)
{
  if(!obj.IsMap())
    return DII::FAILURE; 
  
  DII::Object::MapType result;
  
  const DII::Object::MapType& param = obj.AsMap();
  
  int sect;
  DII::Object::MapType::const_iterator ptr;
      
  ptr = param.find("section");
  if(ptr != param.end())
    sect = (int)ptr->second.AsNum();
  else
    return DII::FAILURE;

  Rect r;
  
  // these are optional
  int x=0,y=0,w=0,h=0;
  
  if((ptr = param.find("x")) != param.end())	  
    x = (int)ptr->second.AsNum();
  if((ptr = param.find("y")) != param.end())	  
    y = (int)ptr->second.AsNum();
  if((ptr = param.find("width")) != param.end())	  
    w = (int)ptr->second.AsNum();
  if((ptr = param.find("height")) != param.end())	  
    h = (int)ptr->second.AsNum();
  
  if(w & h)
    r = Rect(x,y,w,h);
  else
    r = Rect(0,0,0,0);
  
  if((unsigned int)sect < available_.size() && available_[sect].size() > 0)
    {
      
      //need to add check whether the requested rect
      //is actually available
      r = available_[sect].front();
      available_[sect].remove(r);
      bound_[sect].push_front(r);
      
      
      result["NAME"] = std::string("screenarea");
      result["section"] = sect;
      result["x"] = r.upperLeft().x;
      result["y"] = r.upperLeft().y;
      result["width"] = r.width();
      result["height"] = r.height(); 
      result["parent_widget"] = ptrToObj<Widget*>(rootwin);
      
      return result;
    }

  return DII::FAILURE;
}  

bool
Gui::release_screenarea(const DII::Object& obj)
{  
  if(!obj.IsMap())
    return false;

  const DII::Object::MapType& param = obj.AsMap();

  DII::Object::MapType::const_iterator ptr;      
  int sect;
  
  ptr = param.find("section");
  if(ptr != param.end())
    sect = (int)ptr->second.AsNum();
  else
    return false;
  
  // these are optional
  int x=0,y=0,w=0,h=0;
  
  if((ptr = param.find("x")) != param.end())	  
    x = (int)ptr->second.AsNum();
  if((ptr = param.find("y")) != param.end())	  
    y = (int)ptr->second.AsNum();
  if((ptr = param.find("width")) != param.end())	  
    w = (int)ptr->second.AsNum();
  if((ptr = param.find("height")) != param.end())	  
    h = (int)ptr->second.AsNum();
  
  Rect r = Rect(x,y,w,h);
  
  if((unsigned int)sect < bound_.size() && bound_[sect].size() > 0)
    {
      //need to add check whether the requested rect
      //is actually available
      
      r = bound_[sect].front();
      bound_[sect].remove(r);
      available_[sect].push_front(r);
      
    }

  return true;
}  

void
Gui::popup_messagebox(const std::string &title, const std::string &message)
{ 
  bool visible = Viewport::instance()->hideViewport();

  uta::MessageBox MB(title.empty() ? "Message" : title, message);
  MB.setColor(lightgrey);
  MB.setAutoDelete(false);
  MB.exec();

  if(visible)
    Viewport::instance()->showViewport();
}

std::string
Gui::create_dialog(Janus::Node &description)
{
  std::string id = engine_.create(&description);
  if(!id.empty()) // set this new dialog to use our callback object
    engine_.setCallback(id, callback_);

  return id;
}

void
Gui::destroy_dialog(const std::string &str)
{
  engine_.remove(str);
}

Janus::Values
Gui::run_dialog(const std::string& str)
{
  bool visible = Viewport::instance()->hideViewport();

  Janus::Values vals = engine_.execute(str);
  
  if(visible)
    Viewport::instance()->showViewport();

  return vals;
}

void
Gui::show_dialog(const std::string& str)
{
  engine_.show(str);
}

void
Gui::hide_dialog(const std::string& str)
{
  engine_.hide(str);
}

void
Gui::change_dialog(const std::string &id, const std::string &sub_id,
	const Janus::Values *attributes, const std::string *value)
{
  if(attributes)
    engine_.setAttributes(id, sub_id, *attributes);
  if(value)
    engine_.setValue(id, sub_id, *value);
}

void
Gui::call_dialog(const std::string &id, const std::string &sub_id,
	const std::string &cmd, const std::string &param)
{ 
      if(!id.empty() && !sub_id.empty() && !cmd.empty())
	{
		// Forward this dialog to the Janus engine
	  engine_.call(id, sub_id, cmd, param);
	}
      else
	{
	  cerr << "Gui::call_dialog(..) missing parameter " << endl;
	}
}

bool
Gui::popup_questionbox(const std::string &title, const std::string &question)
{ 
  bool visible = Viewport::instance()->hideViewport();

  uta::QuestionBox QB(title.empty() ? "Question" : title, question);
  QB.setColor(lightgrey);
  QB.setAutoDelete(false);

  bool result = QB.exec();
  
  if(visible)
    Viewport::instance()->showViewport();

  return result;
}

unsigned int
Gui::bind_menubutton()
{
  for(unsigned int i=0; i<menu_.size(); i++)
    {
      if(!menu_[i].used)
	{
	  menu_[i].used = true;
	  return i+1;
	}      
    }

  return 0;
}

bool
Gui::release_menubutton(unsigned int i)
{
  if(i > 0 && (i < menu_.size()) && menu_[i].used)
    {
      menu_[i-1].used = false;
      return true;
    }

  return false;
}

bool
Gui::change_menubutton(const DII::Object& obj)
{
  if(!obj.IsMap()|| !doesContain(obj,"id"))
    return false;

  try
    {
      DII::Object::MapType param = obj.AsMap();
      
      unsigned int i = (unsigned int)param["id"].AsNum() - 1;
      
      if((i < menu_.size()) && menu_[i].used)
	{
	  if(doesContain(obj,"text"))
	    menu_[i].button->setText(param["text"].AsString().c_str());
	  if(doesContain(obj,"background"))
	    menu_[i].button->setBackground(SURFACE_RES(param["background"].AsString()));
	  if(doesContain(obj,"upPic"))
	    menu_[i].button->setUpPic(SURFACE_RES(param["upPic"].AsString())); 
	  if(doesContain(obj,"downPic"))
	    menu_[i].button->setDownPic(SURFACE_RES(param["downPic"].AsString()));  
	  
	  return true;
	}
    }
  catch(...)
    {
    }

  return false;
}

///returns string id, arg is title
std::string
Gui::open_progressbar(const std::string& id)
{
  std::map<std::string, uta::ProgressDialog*>::iterator pitr = progressBars_.find(id);
  if(pitr != progressBars_.end())
    return std::string();

  progressBars_[id] = new ProgressDialog();
  progressBars_[id]->info(id.c_str());
  progressBars_[id]->exec();
  return id;
}

bool
Gui::update_progressbar(const std::string &id, double val)
{
  std::map<std::string, uta::ProgressDialog*>::iterator pitr = progressBars_.find(id);
  if(pitr == progressBars_.end())
    return false;

  pitr->second->setValue((int)val);
  
  return true;
}
///arg is string: id
bool 
Gui::close_progressbar(const std::string& id)
{
  std::map<std::string, uta::ProgressDialog*>::iterator pitr = progressBars_.find(id);
  if(pitr == progressBars_.end())
    return false;

  delete pitr->second;
  
  progressBars_.erase(pitr);
  
  return true;
}
