//
//  The Worldforge Project
//  Copyright (C) 1998,1999  The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//
// created by Karsten Laux 07/2000

#include "inventory.h"

#include "dii/object.h"
#include "gui.h"

using namespace DII;

#include "operation.h"
#include <uta/debug.h>
using namespace uta;

Inventory* Inventory::instance_ = 0;

bool
Inventory::update(const std::list<std::string> &ids)
{
  if(!panel_)
    return false;

  panel_->update(ids);

  return true;
}

void
Inventory::deinit()
{
  if(panel_)
    {
      debugN(3,cerr << "Inventory: successfully looked up Module Gui."<<endl;);

      DII::Object::MapType screenarea;
      uta::Rect r = panel_->getRect();

      screenarea["NAME"] = std::string("screenarea");
      screenarea["section"] = 1;
      screenarea["x"] = r.upperLeft().x;
      screenarea["y"] = r.upperLeft().y;
      screenarea["width"] = r.width();
      screenarea["height"] = r.height(); 
      
      debugN(3,cerr << "Inventory: releasing screenarea."<<endl;);

      Gui::instance()->release_screenarea(screenarea);
   
      delete panel_;
      panel_ = 0;
    }
}

bool
Inventory::init(DII::Module*)
{
  DII::Object out;
  
  DII::Object::MapType in;
  in["NAME"] = std::string("screenarea");
  in["section"] = 1; 

  out = Gui::instance()->request_screenarea(in);

  if(!doesContain(out,"section"))
    {
      cerr << "Inventory: failed to obtain screenarea."<<endl;
      return false;
    }

  //discard old input and use old output as new input
  in = out.AsMap();
  
  //actually bind the screenarea
  out = Gui::instance()->bind_screenarea(in);
  //assert(success);
  
  if(!doesContain(out,"section"))
    {
      cerr << "Inventory: failed to bind screenarea."<<endl;
      return false;
    }
  
  int x = out.AsMap()["x"].AsInt();
  int y = out.AsMap()["y"].AsInt();
  int w = out.AsMap()["width"].AsInt();
  int h = out.AsMap()["height"].AsInt();

  Widget* rootwin = objToPtr<Widget*>(out.AsMap()["parent_widget"]);

  debugN(3, cerr << "Inventory: initializing Widget...";);
 
  panel_ = new InventoryPanel(rootwin,x,y,w,h);
  panel_->setAutoDelete(false);

  debugN(3, cerr << "Inventory: done initializing." << endl;);
  return true;
}
