//
//  The Worldforge Project
//  Copyright (C) 1998,1999  The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//
// created by Karsten Laux 07/2000

#include "inventory_panel.h"

#include <uta/resources.h>
#include <uta/togglebutton.h>
#include <uta/debug.h>

#include "action_chooser.h"

#include "../player/player.h"
#include "viewport.h"
#include "console.h"

using namespace uta;

#include "dii/dii.h"
#include "media/sprite.h"
#include "media/sprite_cache.h"
#include "operation.h"
#include "misc.h"

#include "world.h"

#define FEW 5
#define SOME 10


InventoryPanel::InventoryPanel(Widget *parent, int x, int y, int w, int h) :
  Widget(parent, x, y, w, h),
  offset_(0),
  up(NULL),
  down(NULL)
{
  setBackground(SURFACE_RES("sidepanel"));

  const Surface* back = SURFACE_RES("inv_button");
  if(!back)
    return;
 
  //preload some stuff
  addItemEntry("acorn", *back);
  addItemEntry("coin", *back);

  int bw = back->width();
  int bh = back->height();

  int xoff = (w - bw)/2;
  int ystep = bh + 3; // bh pixels for image, 3 gap

  int n = h / ystep; 
  int head_gap = (h + 3 - n * ystep) / 2; // equalize gap top and bottom
  slots_.resize(n);

  debugN(3,cerr << "Inventory using "<< n << " slots." << endl;);

  for(int i=0; i < n; i++)
    {
      slots_[i].button = new Button(this,xoff,i*ystep + head_gap,bw,bh,i);
      slots_[i].button->setBackground(back);
      slots_[i].button->clicked1.connect(slot(this,&InventoryPanel::selectItem));

      slots_[i].label = new Label(slots_[i].button,bw-12,bh-12,12,12);
    }
  
//   down = new PushButton(this, 0, h-25, 25, 25);
//   down->setUpPic(SURFACE_RES("arrow_down"));
//   down->setDownPic(SURFACE_RES("arrow_down_pressed"));
//   down->setRepeat(false);
//   down->clicked.connect(slot(this, &InventoryPanel::slot_scrollDown));

//   up = new PushButton(this, w-25, h-25, 25, 25);
//   up->setUpPic(SURFACE_RES("arrow_up"));
//   up->setDownPic(SURFACE_RES("arrow_up_pressed"));
//   up->setRepeat(false);
//   up->clicked.connect(slot(this, &InventoryPanel::slot_scrollUp));
 
//   up->disable();
}

 
InventoryPanel::~InventoryPanel()
{

  //the widgets will be deleted automatically, so nothing to do here
}

void
InventoryPanel::addItemEntry(const std::string& type, const Surface& back)
{
  
  ItemEntry item;
  SpriteID gid = 0;
  DII::Object::MapType req;
  DII::Object result;

  const Surface* icon;
  const Surface* basic_icon;
  Rect src, dst;

  basic_icon = SURFACE_RES(type);
  
  if(!basic_icon || basic_icon->empty())
    {
      SpriteDesc desc;
      desc.type = type;
      gid = SpriteCache::inst()->request(desc);
      SpriteCache::inst()->lock(gid);
      Sprite* sprite = SpriteCache::inst()->lookup(gid);
      basic_icon = sprite->getLayer(0,0,0);
    }

  icon = SURFACE_RES(type+"_few");

  item.icon_few = new Surface(back);
  item.icon_few->convert(Pixelformat::DISPLAY);

  if(icon && !icon->empty())
    center(icon, item.icon_few);
  else
    center(basic_icon,item.icon_few);

  icon = SURFACE_RES(type+"_some");
  item.icon_some = new Surface(back);
  item.icon_some->convert(Pixelformat::DISPLAY);

  if(icon && !icon->empty())
    center(icon, item.icon_some);
  else
    center(basic_icon, item.icon_some);

  icon = SURFACE_RES(type+"_many");
  item.icon_many = new Surface(back);
  item.icon_many->convert(Pixelformat::DISPLAY);

  if(icon && !icon->empty())
    center(icon, item.icon_many);
  else
    center(basic_icon, item.icon_many);

  //if at all, we have just copied the sprite surface,
  //so we may release it again
  if(gid != 0)
    SpriteCache::inst()->release(gid);

  Content::iterator itr = content_.find(type);
  if(itr != content_.end())
    {
      delete itr->second.icon_few;
      delete itr->second.icon_some;
      delete itr->second.icon_many;
    }

  content_[type] = item;

}

void
InventoryPanel::update(const std::list<std::string>& contains)
{
  // update local data structures
 
  Content::iterator itr = content_.begin();
  while(itr != content_.end())
    {
      itr->second.ids.clear();
      itr++;
    }

  World::Entity* entity;

  std::list<std::string>::const_iterator IDitr;
  for(IDitr = contains.begin(); IDitr != contains.end(); IDitr++)
    {
      entity = World::World::inst()->lookupEntity(*IDitr);
      assert(entity);

      std::string type = entity->parent();

      itr = content_.find(type);
      if(itr != content_.end())
	{
	  itr->second.ids.push_back(entity->id());
	}
      else
	{
	  debugN(3,cerr << "InventoryPanel: creating new slot for item type " << type << endl;);
          const Surface* back = SURFACE_RES("inv_button");
          if(!back)
            return;
	  addItemEntry(type, *back);
	  itr = content_.find(type);
	  itr->second.ids.push_back(entity->id());
	}
    }

  updateWidget();
}

void
InventoryPanel::updateWidget()
{
  Content::iterator itr = content_.begin();
  unsigned n = 0;
  while(itr != content_.end() && n < offset_)
    {
      if(itr->second.ids.size())
	n++;
      itr++;
    }

  unsigned int i=0;
  const Surface* icon;
  std::string txt;

  unsigned int count;
  while(itr != content_.end() && i != slots_.size())
    {
      count = itr->second.ids.size();
      if(count > 0)
	{
	  if(count <= FEW)
	    icon = itr->second.icon_few;
	  else if(count <= SOME)
	    icon = itr->second.icon_some;
	  else
	    icon = itr->second.icon_many;
	  
	  txt = itos(count);
       
	  slots_[i].button->setBackground(icon);
	  slots_[i].label->setText(txt.c_str());
	  i++;
	}
      itr++;
    }

  //clear remaining slots
  while(i < slots_.size())
    {
      slots_[i].button->setBackground(SURFACE_RES("inv_button"));
      slots_[i].label->setText("");
      i++;
    }
}



void
InventoryPanel::selectItem(int id)
{
  Content::iterator itr = content_.begin();
  unsigned n = 0;
  while(itr != content_.end() && n < offset_)
    {
      if(itr->second.ids.size())
	n++;
      itr++;
    }

  int i=0;
  while(itr != content_.end())
    {
      if(itr->second.ids.size() > 0)
	{
	  if(i == id)
	    {
	      if(itr->second.ids.empty())
		return;
   
	      bool visible = Viewport::instance()->hideViewport();
	      
	      std::vector<std::string> actions;
	      actions.push_back("Give or Put");
	      actions.push_back("Drop to Ground");
	      actions.push_back("Trade");
	      
	      ActionChooser chooser(actions, 0 , itr->second.ids.size());
	      chooser.setAutoDelete(false);
	      
	      if( chooser.exec() != uta::Dialog::OK)
		{
		  if(visible)
		    Viewport::instance()->showViewport();
		  return;
		}
	      
	      if(visible)
		Viewport::instance()->showViewport();

	      std::string action = chooser.selectedAction();
	      unsigned amount = chooser.selectedAmount();
	      
	      DII::Object::ListType idList;
	      IDList::iterator IDitr = itr->second.ids.begin();
	      //copy amount ids from itr->second.ids to idList
	      while(amount)
		{
		  idList.push_back(*IDitr);
		  IDitr++;
		  amount--;
		}

	      if(Player::instance())
		{
	          if(action == "Drop to Ground")
		    {
		      if(Console::instance())
		        Console::instance()->std_out(std::string("dropping ")
			  +itos(idList.size())+std::string(" items .\n"));
		      Player::instance()->drop(idList);
		    }
	          else if(action == "Trade")
		    {
		      DII::Object::ListType::iterator itr;
		      for(itr = idList.begin(); itr != idList.end(); itr++)
		        Player::instance()->say(std::string("I would like to sell ")+itr->AsString());
		    
		    }
	          else if(action == "Give or Put")
		    {	
		      if(Console::instance())
		        Console::instance()->std_out(std::string("giving ")
			  +itos(idList.size())+std::string(" items away.\n"));
		      Player::instance()->drag(idList);
		    }
		}  
	      
	    }	      
	   
	  i++;
	}
      itr++;
    }
}


bool
InventoryPanel::center(const Surface* src, Surface* dst)
{
  return centerBlit(src,dst);
}

void
InventoryPanel::slot_scrollUp()
{
  if(offset_ > 0)
    {
      down->enable();
      offset_--;
      updateWidget();
    }
  else
    {
      up->disable();
    }
  up->grabFocus();
}

void
InventoryPanel::slot_scrollDown()
{
  if(offset_ < slots_.size())
    {
      up->enable();
      offset_++;
      updateWidget();
    }
  else
    {
      down->disable();
    }

  down->grabFocus();
}
