//
//  The Worldforge Project
//  Copyright (C) 1998,1999  The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//
// created by Karsten Laux 02/2001

#include "observer.h"
using namespace DII;
using namespace World;

#include "dii/module.h"
#include "world/world.h"

#define EMIT_MESSAGE(ent,event)\
{\
DII::Object::MapType msg;\
msg["from"]="Observer";\
msg["event"]=event;\
msg["args"]=DII::Object::ListType(1,ent->asObject());\
signal.emit(msg);\
}

Observer* Observer::instance_ = 0;

bool Observer::init(DII::Module *module)
{
  signal.connect(module->signal.slot());
  return true;
}

void
Observer::addID(const std::string& str)
{
   toObserv_.push_back(str); 
}
  
void
Observer::removeID(const std::string& str)
{
   toObserv_.remove(str);
}


void 
Observer::disappeared(const ::World::Entity* ent)
{
  std::list<std::string>::iterator itr = toObserv_.begin();
  while(itr != toObserv_.end())
  {
     if(*itr == ent->id())
        EMIT_MESSAGE(ent,"disappeared")
     itr++;
  }
}
  
void 
Observer::moved(const ::World::Entity* ent)
{
  std::list<std::string>::iterator itr = toObserv_.begin();
  while(itr != toObserv_.end())
  {
     if(*itr == ent->id())
        EMIT_MESSAGE(ent,"moved")
     itr++;
  }
}
      
void 
Observer::changed(const ::World::Entity* ent)
{
  std::list<std::string>::iterator itr = toObserv_.begin();
  while(itr != toObserv_.end())
  {
     if(*itr == ent->id())
        EMIT_MESSAGE(ent,"changed")
     itr++;
  }
}
  
void 
Observer::deleted(const ::World::Entity* ent)
{
  std::list<std::string>::iterator itr = toObserv_.begin();
  while(itr != toObserv_.end())
  {
     if(*itr == ent->id())
        EMIT_MESSAGE(ent,"deleted")
     itr++;
  }
}
  
void 
Observer::created(const ::World::Entity* ent)
{
  std::list<std::string>::iterator itr = toObserv_.begin();
  while(itr != toObserv_.end())
  {
     if(*itr == ent->id())
        EMIT_MESSAGE(ent,"created")
     itr++;
  }
}
