//
//  The Worldforge Project
//  Copyright (C) 1998,1999  The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//
// created by Karsten Laux 01/1999

#include "panel.h"

#include <uta/debug.h>
#include <uta/resources.h>
#include <uta/application.h>

using namespace uta;


#define CHAR_WIDTH 70
//#define IND_WIDTH 43
#define IND_WIDTH 0
#define MENU_WIDTH  30
//#define SEP_WIDTH 8
#define SEP_WIDTH 0
#define MENU_POS (width()-CHAR_WIDTH-MENU_WIDTH-2*SEP_WIDTH)
#define CHAT_POS (IND_WIDTH+SEP_WIDTH)
#define CHAT_HEIGHT  height()
#define CHAT_WIDTH  (width()-2*IND_WIDTH-2*SEP_WIDTH)


Panel::Panel(Widget *parent, int x, int y, int w, int h) :
  Widget(parent, x, y, w, h),
  index_(0)
{
  debugN(13,std::cerr << "Panel: starting up ..." << std::endl;);
  
  // Seperator widget - no longer used
  //Widget* sep;

  // setBackground(SURFACE_RES("panel_background"));

 
//   sep = new Widget(this,0,0,SEP_WIDTH,height());
//   sep->setBackground(SURFACE_RES("seperator_v"));

//   Widget* health = new Widget(this,
// 			      SEP_WIDTH,0,
// 			      IND_WIDTH,height());
//   health->setBackground(SURFACE_RES("health_background"),false);

  
//   sep = new Widget(this,SEP_WIDTH+IND_WIDTH,0,SEP_WIDTH,height());
//   sep->setBackground(SURFACE_RES("seperator_v"));

  Widget* frame = new Widget(this, CHAT_POS,0,CHAT_WIDTH, CHAT_HEIGHT);
  frame->setBackground(SURFACE_RES("text_panel"));

  chatWin_ = new Terminal(frame,16,0,CHAT_WIDTH-32,CHAT_HEIGHT-22,
			  FONT_RES("small_text"));
  chatWin_->setMaxLinesToStore(1024);

  chatWin_->scrolled.connect(slot(this, &Panel::slot_terminal));
  
  scrollbar_ = new Slider(frame, 
			  CHAT_WIDTH-16, 0,
			  16, CHAT_HEIGHT, Slider::VERTICAL);
  
  scrollbar_->setBackground(NULL);
  scrollbar_->setButtonColor(lightgrey);
  scrollbar_->setRange(0,chatWin_->linesCount());
  scrollbar_->setValue(chatWin_->currentLine());
  scrollbar_->valueChanged.connect(slot(this, &Panel::slot_scrollbar));
  

  //input line
  console_ = new LineEdit(frame, 
			  16,CHAT_HEIGHT-22,
			  CHAT_WIDTH-32,18,
			  FONT_RES("small_text"));

  console_->setTextColor(darkgreen, lightgrey);



//   sep = new Widget(this,CHAT_POS+CHAT_WIDTH,0,SEP_WIDTH,height());
//   sep->setBackground(SURFACE_RES("seperator_v"));

    
//   Widget* mana = new Widget(this, 
// 			    CHAT_POS+CHAT_WIDTH,0,
// 			    IND_WIDTH,height());
//   mana->setBackground(SURFACE_RES("mana_background"), false);

//   sep = new Widget(this,CHAT_POS+CHAT_WIDTH+IND_WIDTH,
// 		   0,SEP_WIDTH,height());
//   sep->setBackground(SURFACE_RES("seperator_v"));

 
  
  console_->enterPressed.connect(slot(this, &Panel::command));

  //install hotkey filter
  uta::Application::keystroke.connect(slot(this, &Panel::keyfilter));
 

  debugN(13,std::cerr << "Panel: ready. "<<std::endl;);

}

 
Panel::~Panel()
{
  //the widgets will be deleted automatically, so nothing to do here
}

 
bool
Panel::keyfilter(const uta::KeyEvent* key)
{
  //only accept key press events
  if(key->state() != uta::KeyEvent::PRESSED)
    return false;

  bool handled = false;

  switch(key->key())
    {
    case uta::KEY_KP_ENTER:
    case uta::KEY_RETURN:
    case uta::KEY_QUOTEDBL:
      if(console_)
	console_->grabFocus();
      handled = false;
      break;
    case uta::KEY_F1:
      history_completize();
      handled = true;
      break;
    case uta::KEY_F2:
      history_prev();
      handled = true;
      break;
    case uta::KEY_F3:
      history_next();
      handled = true;
      break;
    default:
      break;
    }
  return handled;
}
 
void 
Panel::slot_scrollbar(int n)
{ 
  chatWin_->scrollTo(n);
}

void 
Panel::slot_terminal()
{
  scrollbar_->setDelta(chatWin_->visibleLines()/3+1);
  scrollbar_->setRange(0,chatWin_->linesCount());
  scrollbar_->setValue(chatWin_->currentLine());   
}


void Panel::addMessage (const char* data)
{
  chatWin_->addText(data);
}

void Panel::command (const char* data)
{
  // grab the incoming argument and save it; otherwise
  // libsigc will clobber it when we clear the input line
  std::string temp(data);
  //clear the input line
  console_->setText("");
  if(temp.size() > 0)
    history_add(temp);

  userinput.emit(temp);
}


void Panel::history_up ()
{
  chatWin_->scrollUp(10);
}

void Panel::history_down ()
{
  chatWin_->scrollDown(10);
}

void
Panel::history_completize()
{
  if(history_.empty())
    return;

  std::string in = console_->getText();
  size_t n = 0;
  while(n < history_.size())
    {
      if(history_[n].find(in) != std::string::npos)
	break;
      n++;
    }  
  if(n < history_.size())
    console_->setText(history_[index_].c_str());
}

void
Panel::history_prev()
{
  if(history_.empty())
    return;

  if(index_ > 1)
    {
      index_--;
      console_->setText(history_[index_].c_str());
    }
}

void
Panel::history_next()
{
  if(history_.empty())
    return;

  if(index_ < history_.size()-1)
    {
      index_++;
      console_->setText(history_[index_].c_str());
    }
}

void
Panel::history_add(const std::string& txt)
{
  size_t n=0;
  while(n < history_.size())
    {
      if(txt == history_[n])
	break;
      n++;
    }

  if(n == history_.size())
    history_.push_back(txt);

}
