//  The Worldforge Project
//  Copyright (C) 1998, 1999 The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//
// created by Karsten Laux 02/2000


#ifndef _VIEWPORT_H_
#define _VIEWPORT_H_

#include <sigc++/object.h>
#include <uta/widget.h>

namespace DII {class Module;}

/**Viewport.
   This implementation uses a "DirectSurface", which cannot be overlayed
   by other widgets and especially dialogs. So in order to show a dialog
   hideViewport must be called, which turns off the viewport.

   Viewport emits:

   * info operations: 
   from = Viewport
   arg0 = "hidden" | "exposed"
   arg1 = (long)pointer to viewport surface

   * userinput operations
   from = Viewport
   arg0 = "clicked" | "selected"
   arg1 = List(x,y) | List(x1,y1,x2,y2)

*/
class Viewport : virtual public SigC::Object
{
 public:
  
  ///
  Viewport() : widget_(0), background_(0) {assert(!instance_); instance_ = this;}
  ///
  virtual ~Viewport() {assert(instance_); instance_ = 0; hideViewport();}

  static Viewport* instance() {return instance_ ? instance_ : new Viewport();}

  ///
  bool init(DII::Module*);
  ///
  void deinit();
  ///
  bool hideViewport();
  ///
  bool showViewport();
  ///
  uta::DirectSurface* getScreen();

  ///
  void slot_clicked(const uta::Point&);
  ///
  void slot_rightclicked(const uta::Point&);
  ///
  void slot_moved(const uta::Point&);
  ///
  void slot_selected(const uta::Rect&);

  /// arg1 is true for expose, false for hide
  SigC::Signal2<bool, bool, uta::DirectSurface*,
	SigC::FixedMarshal<bool,false> > signalExposeHide;
  /// arg1 is 0 for left button, 1 for right button
  SigC::Signal2<bool, int, const uta::Point&,
	SigC::FixedMarshal<bool,false> > signalClicked;
  ///
  SigC::Signal1<bool, const uta::Point&,
	SigC::FixedMarshal<bool,false> > signalMoved;
  ///
  SigC::Signal1<bool, const uta::Rect&,
	SigC::FixedMarshal<bool,false> > signalSelected;

 protected:
  ///
  uta::Widget* widget_;
  ///
  uta::Widget* background_;

  static Viewport* instance_;
};

#endif
