//  The Worldforge Project
//  Copyright (C) 1998, 1999 The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//
// created by Karsten Laux 01/2000

#include "music.h"
#include "operation.h"
#include <uta/application.h>
#include <uta/resources.h>

#include "../modules/viewport.h"
#include "../modules/console.h"

#ifndef HAVE_DRAND48
#define drand48() rand()/((double)RAND_MAX)
#endif

using namespace DII;
using namespace uta;

/* for some (unknown) reason libuta, SDL_mixer or smpeg refuses to play
   an already played mp3 stream (rewind does not work).
   So we discard the music object after having played it and create 
   a new one from the original file ...

   this is actually more a workaround than a nice implementation
   Karstem, 2000/07/15
*/

MusicModule* MusicModule::instance_ = 0;

MusicModule::MusicModule():
  timer_(5000),  // check every 5 seconds
  currentSong_(NULL),
  volume_(70)
{
  assert(!instance_);
  instance_ = this;

  timer_.halt();
}

void
MusicModule::deinit()
{  
  timer_.halt();

  if(currentSong_)
    {
      currentSong_->stop();
      //  delete currentSong_;
    }
}


bool
MusicModule::init(DII::Module*)
{  
  songs_= uta::Resources::instance->songList();
  
  if(songs_.size() == 0)
    {
      if(Console::instance())
	Console::instance()->std_err(std::string("MusicModule: couldn't find any piece of music ! \n"));
      return false;
    }

  currentSong_ = NULL;

  //install event handler:
  Viewport* viewport_ = Viewport::instance();
  if(!viewport_)
    {
      cerr << "MusicModule: Error, viewport not found !";
      return false;
    }

  /* with this connection eventHandler is called whenever the
     viewport is turned on or off
     viewport_->signal.connect(SigC::slot(this, &MusicModule::eventHandler));
  */
 
  timer_.alarm.connect(slot(this, &MusicModule::tick));
  timer_.run();

  uapp->mixer()->setMusicVolume(volume_);

  return true;
}

void
MusicModule::tick()
{
  //if song is still playing or paused ... return
  if(uapp->mixer()->currentSong() &&
     (uapp->mixer()->currentSong()->playing() || uapp->mixer()->currentSong()->paused()))
    return;

  //otherwise choose new one
  unsigned index = (unsigned)(drand48() * (double)songs_.size());
  if(index >= songs_.size())
    index = songs_.size()-1;

//   DII::callService("std_out",string("now playing song \"")+songs_[index]+string("\"\n"));

  uapp->mixer()->setMusic(songs_[index].c_str());
  uapp->mixer()->rewindMusic();
  uapp->mixer()->startMusic();  
}

bool
MusicModule::eventHandler(const DII::Object& obj)
{
  const DII::Object::ListType& args = 
    obj.AsMap().find("args")->second.AsList();

  std::string opType = typeOfOperation(obj);
  
  /*check for info-operation from Viewport
    argv(1) contains pointer to it's screen, so if the pointer

    is valid the viewport is visible ->  we may play sounds :)
  */
  if((opType == "info") && 
     (obj.AsMap().find("from")->second.AsString() == "Viewport") &&
     (args.size() >= 2))
    {
      DII::Object::ListType::const_iterator itr = args.begin();
      // const DII::Object& arg0 = *(itr);
      itr++;
      const DII::Object& arg1 = *(itr);

      if(objToPtr<uta::DirectSurface*>(arg1) != NULL)
	{
// 	  timer_.run();
// 	  if(currentSong_)
// 	    if(currentSong_->paused())
// 	      currentSong_->resume();
	  
// 	  if(currentSong_)
// 	    currentSong_->setVolume(volume_);
	  uapp->mixer()->setMusicVolume(volume_);
	}
      else
	{ 
// 	  timer_.halt();
// 	  if(currentSong_)
// 	    if(currentSong_->playing())
// 	      currentSong_->pause(); 
// 	  if(currentSong_)
// 	    currentSong_->setVolume(volume_/2);	  
	  uapp->mixer()->setMusicVolume(volume_/2);
	}
    }
  return false;
}


  
void 
MusicModule::play()
{
  timer_.run();
}
  
  
void 
MusicModule::stop()
{
  //   if(currentSong_)
//     currentSong_->stop();
  uapp->mixer()->stopMusic();

  timer_.halt();
}
  
void 
MusicModule::pause()
{
//   if(currentSong_)
//     if(currentSong_->playing())
//       currentSong_->pause();
  uapp->mixer()->pauseMusic();
}

void 
MusicModule::resume()
{
 //  if(currentSong_)
//     if(currentSong_->paused())
//       currentSong_->resume();
  uapp->mixer()->resumeMusic();
}

void 
MusicModule::music_vol(int volume)
{
  volume_ = volume;

#ifdef DEBUG
  cerr << "MusicModule: setting volume to "<<volume_ << endl;
#endif

  uapp->mixer()->setMusicVolume(volume_);
}
