//  The Worldforge Project
//  Copyright (C) 1998, 1999 The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//
// created by Karsten Laux 10/2000


#ifndef _CONNECTION_H_
#define _CONNECTION_H_

//Atlas includes (order is partially important)
#include <Atlas/Message/Object.h>
#include <Atlas/Codec.h>
#include <Atlas/Net/Stream.h>
#include <Atlas/Objects/Root.h>
#include <Atlas/Objects/Encoder.h>


#include <sigc++/signal_system.h>

class tcp_socket_stream;

/**Connection.
   This is the uplink to the gameserver. It hides the Atlas network stuff.
    
   Upon construction a bridge
   instance (e.g. MsgDispatch) has to be provided; all incoming data is
   piped into this bridge for further computing.


   usage:
   MsgDispatch dispatcher;
   ...
   Connection c(&dispatcher);

   dispatcher->EntityLoginArrived.connect(...);
   ...

   c.connect("gameserv10.worldforge.org",6767);
   if(c.state() != Connection::CONNECTED)
     {
         cerr << "Connect failed !" << endl;
         exit(1);
     }

   while(c.connected())
     {
  
        c.poll();   
        if(something to send)
           c.send(...);
     }

 */
class Connection : virtual public SigC::Object
{
 public:
  ///
  enum { NOT_CONNECTED, CONNECTING, NEGOTIATING, CONNECTED };
  
  ///
  Connection(Atlas::Bridge*);
  ///
  virtual ~Connection();
  
  ///
  virtual void send(const Atlas::Objects::Root* o);
  ///process data in input and output buffers
  virtual void poll();

  ///
  void connect(const std::string& inetName, unsigned int portnum);
  ///
  void disconnect();
  ///
  bool connected() const { return state_ == CONNECTED; };
  ///
  int state() const { return state_; };
  ///
  std::string getPeerName() const;
  ///
  float getLag() const { return lag_; };
  ///
  SigC::Signal0<void> disconnected;
  ///
  void setTimeout(unsigned sec, unsigned usec=0);
  
 protected:
  ///
  tcp_socket_stream *netStream_;
  ///
  Atlas::Codec<std::iostream>* codec_;
  ///
  Atlas::Objects::Encoder* encoder_;
  ///
  int state_;
  ///
  Atlas::Bridge* decoder_;
  ///
  float lag_;
  ///
  std::string peerName_;
  ///
  unsigned timeoutSec_;
  ///
  unsigned timeoutUsec_;

};

#endif
