//  The Worldforge Project
//  Copyright (C) 1998, 1999 The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//
// created by Karsten Laux 01/2000

#include "lobby.h"
#include <uta/debug.h>
#include <iostream>

#include "../modules/console.h"
#include "player.h"

using std::cerr;
using std::endl;
  
Lobby::Lobby()
{
  sightDecoder_.rawObj.connect(slot(this, &Lobby::processSight));
  sightDecoder_.account.connect(slot(this, &Lobby::processSightAccount));
  sightDecoder_.gameEntity.connect(slot(this, &Lobby::processGameEntity));
  soundDecoder_.talkOp.connect(slot(this, &Lobby::processSoundTalk));     
}

void
Lobby::processSightOp(const Atlas::Objects::Operation::Sight& sight)
{
  debugN(11, cerr << "Lobby: A sight operation arrived." << endl;);
  const Atlas::Message::Object::ListType & args = sight.GetArgs();
  const Atlas::Message::Object & objseen = args.front();
  sightDecoder_.process(objseen);
}
 

void
Lobby::processSoundOp(const Atlas::Objects::Operation::Sound& sound)
{
  debugN(11, cerr << "Lobby: A sound operation arrived." << endl;);
  const Atlas::Message::Object::ListType & args = sound.GetArgs();
  const Atlas::Message::Object & objheart = args.front();
  soundDecoder_.process(objheart);   
}

 
void
Lobby::processSoundTalk(const Atlas::Objects::Operation::Talk& talkOp)
{
  DII::Object obj;
  std::string say;
  std::string from;
  try
    {
      obj = talkOp.GetArgs().front();
      say = obj.AsMap()["say"].AsString();
      from = talkOp.GetFrom();

      std::map<std::string, std::string>::iterator itr = players_.find(from);
      if(itr != players_.end())
	from = itr->second;
      else
	from = "<unknown>";

      if(Console::instance())
	Console::instance()->std_out(from+std::string(" : ")+say+std::string("\n"));
    }
  catch(...)
    {
      cerr << "Lobby: error in talk operation" << endl;
    }
}  

void
Lobby::processSight(const Atlas::Message::Object& obj)
{  
  debugN(11, cerr << "Lobby: A sight of raw object." << endl;);
  try
    {
      const Message::Object::MapType & objmap = obj.AsMap();
      
      if (objmap.find("parents")->second.AsList().front().AsString() == "players") 
			{
        
        const Message::Object::ListType & playerlist = objmap.find("contains")->second.AsList();
        Message::Object::ListType::const_iterator I;
        
        gettingList_ = true;
        if(Console::instance())
	  Console::instance()->std_out(std::string("Connected players : "));

        for (I = playerlist.begin(); I != playerlist.end(); I++) 
        {
          if(Player::instance())
	    Player::instance()->playerLook(I->AsString());
        }
	
        gettingList_ = false;
      }
			else if (objmap.find("parents")->second.AsList().front().AsString() == "room")
 			{
        
        const Message::Object::ListType & playerlist = objmap.find("people")->second.AsList();
        Message::Object::ListType::const_iterator I;
        
        gettingList_ = true;
        if(Console::instance())
	  Console::instance()->std_out(std::string("Connected players : "));

        for (I = playerlist.begin(); I != playerlist.end(); I++) 
        {
					// This doesn't seem to be the right thing to do - it's
					// returning all the info for a player, I only want the info
					// on the connected one.
          if(Player::instance())
	    Player::instance()->playerLook(I->AsString());
        }
	
        gettingList_ = false;
      }
   }
  catch(...)
    {
      cerr << "Lobby: error while receiving player list " << endl;
    }

}

void
Lobby::processSightAccount(const Atlas::Objects::Entity::Account& o)  
{
  debugN(11, cerr << "Lobby: A sight of account." << endl;);
  try
    {
      std::string id = o.GetId();
      std::string name = o.GetName();
      
      if(gettingList_)
	{
          if(Console::instance())
	    Console::instance()->std_out( name+std::string("  "));
	}
      else
	{
          if(Console::instance())
	    Console::instance()->std_out( std::string("---- ")+name
	      + std::string(" has arrived ----\n"));
	}
      players_[id] = name;
    }
  catch(...)
    {
      cerr << "Lobby: error in account sight operation" << endl;
      return;
    } 
}   

void
Lobby::processGameEntity(const Atlas::Objects::Entity::GameEntity& ge)
{
  debugN(11, cerr << "Lobby: A sight game entity." << endl;);
  try
  {
    std::string id = ge.GetId();
    std::string name = ge.GetName();
		
    if(gettingList_)
      {
        if(Console::instance())
	  Console::instance()->std_out( name+std::string("  "));
      }
    else
      {
        if(Console::instance())
	  Console::instance()->std_out( std::string("---- ")+name
	    + std::string(" has arrived ----\n"));
      }
    players_[id] = name;
  }
  catch(...)
  {
    cerr << "Lobby: error in game entity sight operation" << endl;
    return;
  } 
}

void 
Lobby::processAppearanceOp(const Atlas::Objects::Operation::Appearance& o)
{
  debugN(11, cerr << "Lobby: An appearance ." << endl;);
  const Message::Object::MapType & arg1 = o.GetArgs().front().AsMap();
  std::string id = arg1.find("id")->second.AsString();
  
  if(Player::instance())
    Player::instance()->playerLook(id);
}
 
void 
Lobby::processDisappearanceOp(const Atlas::Objects::Operation::Disappearance& o)
{
  debugN(11, cerr << "Lobby: A disappearance ." << endl;);
  const Message::Object::MapType & arg1= o.GetArgs().front().AsMap();
  Message::Object::MapType::const_iterator I = arg1.find("id");
  if (I == arg1.end()) {
    cerr << "Lobby: Disappearance op with no id" << endl;
  }
  const std::string & id = I->second.AsString();
 
  std::string name = "<unknown>";
  std::map<std::string, std::string>::iterator itr = players_.find(id);
  if(itr != players_.end())
    name = itr->second;

  players_.erase(id);

  if(Console::instance())
    Console::instance()->std_out( std::string("---- ")+name
      + std::string(" has quit ----\n"));
}
  
