//  The Worldforge Project
//  Copyright (C) 1998, 1999 The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//
// created by Karsten Laux 01/2000


#ifndef _PLAYER_H_
#define _PLAYER_H_

#include <sigc++/object.h>
#include <Atlas/Objects/Operation/RootOperation.h>

#include "../network/connection.h"
#include "../network/wf_net.h"

#include <uta/event.h>
#include <uta/surface.h>
#include "../network/msgdispatch.h"
#include "coord.h"
#include "world.h"
#include "lobby.h"

#include "dii/object.h"
namespace DII {class Module;}

///
class Player : virtual public SigC::Object
{
  
 public:
  
  ///
  Player();
  ///
  virtual ~Player();

  static Player* instance() {return instance_ ? instance_ : new Player();}

  ///
  bool init(DII::Module*);
  ///
  void deinit();

  ///
  bool logout();
  ///
  bool login(const std::string &_id, const std::string &_password,
	     const std::string &_hostname = std::string());
  ///
  bool createAccount(const std::string &_id, const std::string &_password,
	     	     const std::string &_hostname = std::string());
  ///
  bool createCharacter();
  ///
  bool takeOverCharacter(const std::string&);
  ///
  std::string currentCharacterID();

  
  /**user has selected given entity.
     now some action might need to performed.
     (this is called from Viewport/Renderer)
     @param: id (std::string) of entity
  */
  bool selectEntity(const std::string&);
  ///
  bool moveTo(const DII::Object&);
  ///
  bool rightclick(const DII::Object&);
  ///
  bool leftclick(const DII::Object&);
  ///
  void look(const std::string&);
  ///
  void action(const std::string&);
  ///
  void serverGet();
  ///
  void playerLook(const std::string &id = std::string());
  /** Query a describing object for the given id.
      If id is an empty string, a list of all known ids is returned.
      (this calls infact World::queryObject())
  */
  DII::Object queryMemory(const DII::Object& id);
  ///proxy for World::getTime()
  DII::Object getTime();
  ///
  void say(const std::string&);
  ///
  bool drag(const DII::Object&);
  ///
  bool drop(const DII::Object&);
  ///
  bool transfer(const DII::Object&);
  ///
  bool inventory();
  /** Query server for status informations.
      Arg is a list with first element beeing the hostname and the 
      second (optional arg) the port to connect to.
      This establishes a TCP connection to the server machine at
      the given port and sends a Get Operation.
      The method returns the received Info operation or DII::FAILURE
      if the request timed out.
  */
  DII::Object queryServer(const std::string& hostname,
			  int port = WFNetwork::GAMESERVER_PORT);
  /** Returns list of running game servers.
      This queries metaserver.worldforge.org for a list
      of currently running servers.
      The a list of strings is returned. 
  */
  std::list<std::string> queryServerList();

  /** Send operation to game server.
      @param is a list containing two elements:
      The first element is to be a string identifying the operation to send,
      the second element is a map of attributes.
      FIXME ... does this _really_ make sense ?
   */
  bool sendOperation(const DII::Object&);

  ///hotkey filter
  bool keyfilter(const uta::KeyEvent*);

  /// maybe we want to control more than one later ...
  World::Entity* currentCharacter(void); 

  SigC::Signal1<bool, const Atlas::Message::Object&,
	SigC::FixedMarshal<bool,false> > msgSignal;
  SigC::Signal1<bool, const Atlas::Objects::Operation::RootOperation&,
	SigC::FixedMarshal<bool,false> > opSignal;

 protected:

  ///uplink to gameserver
  Connection* connection_;
  ///
  uta::Timer timer_;
  ///
  uta::Timer slowTimer_;
  ///
  MsgDispatch dispatcher_;
  ///
  std::string characterID_; 
  /// 
  std::list<std::string> selectedIDs_;    
  ///
  uta::Surface draggedIcon_;
  /** Game states.
      INIT : just started
      CONNECTED: connected to game server
      QUERYING: querying server for infos about running game
      LOBBY : chatting in the LOBBY
      INGAME : actually playing the game ;)
  */
  enum { INIT, CONNECTED, LOBBY, INGAME };
  ///
  enum { NO_ERR, UNKNOWN_ERR, NOACCOUNT_ERR };
  ///
  int state_;
  ///
  int error_;
  ///
  std::string accountID_;
  ///
  Lobby lobby_;
  ///
  DII::Object::MapType serverInfo_;
  ///
  void configure ();

  /**Slots
   */
  //@{
  ///
  void tick();
  ///
  void slowTick();
  ///
  void disconnected();
  ///
  bool connect(const std::string& host, unsigned int port);
  ///
  void OpInfo(const Atlas::Objects::Operation::Info&);
  ///
  void OpError(const Atlas::Objects::Operation::Error&);
  ///
  void OpSight(const Atlas::Objects::Operation::Sight&);
  ///
  void OpSound(const Atlas::Objects::Operation::Sound&);
  ///
  void OpDisappearance(const Atlas::Objects::Operation::Disappearance&);
  ///
  void OpAppearance(const Atlas::Objects::Operation::Appearance&);
  //@}

  static Player* instance_;
};

#endif
