//  The Worldforge Project
//  Copyright (C) 1998, 1999 The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//
// created by Karsten Laux 01/2000

#include "registry.h"
#include "dii/serialize.h"

#include <queue>
#include <fstream>
#include <stdlib.h>
 
using std::cerr;
using std::endl;

#undef DEBUG
#ifdef DEBUG
#define INFO(prg) prg
#else
#define INFO(prg)
#endif

/*split the pathname devided by either "/" or "|"
  and return the number of pathcomponents
*/
unsigned int split_path(const std::string &s,std::queue<std::string> &argc)
{
  std::string::const_iterator iter;
  std::string this_string = "";
  
  for(iter = s.begin(); iter != s.end(); iter++)
    {
      if((*iter) == '/' || (*iter) == '|')
	{
	  if(!this_string.empty())
	    argc.push(this_string);
	  this_string = "";
	}
      else
	{
	  this_string += (*iter);
	}
    }

  if(!this_string.empty())
    argc.push(this_string);

  return argc.size();	
}

using namespace DII;

class Registry* Registry::instance_ = 0;

bool
Registry::open(const std::string& filename)
{
  if(!filename.empty())
    {
      filename_ = filename;
      DII::Object obj(tree_);
      if(DII::Serialize::read(filename.c_str(), obj))
	if(obj.IsMap())
	  {
	    tree_ = obj.AsMap();
	    return true;
	  }
    }
  return false;
}

bool
Registry::flushRegistry()
{
  return !filename_.empty() &&
    DII::Serialize::write(tree_, filename_.c_str());
}

void
Registry::close()
{
  flushRegistry();
  filename_ = "";
}

// DII::Object
// Registry::set(const DII::Object& obj)
// {
//   DII::Object::MapType req;

//   if(!obj.IsList() || obj.AsList().size() != 3)
//     return DII::FAILURE;
  
//   DII::Object::ListType::const_iterator itr = obj.AsList().begin();

//   string path;
//   string key;
//   string type;

//   queue<string> tmp;
//   split_path(itr->AsString(), tmp);
//   itr++;

//   if(tmp.empty())
//     return DII::FAILURE;
    
//   while(tmp.size() > 1)
//     {
//       //      subdirs.push(tmp.front());
//       path = path+"/"+tmp.front();
//       tmp.pop();
//     }

//   key = tmp.front();
  
//   req["path"] = path;
//   req["key"] = key;

//   type = itr->AsString();
//   itr++;

//   if (type == "int")
//     req["value"]=atoi(itr->AsString().c_str());
//   else if (type == "float")
//     req["value"]=atof(itr->AsString().c_str()); 
//   else
//     req["value"] = *itr; 
  
//   return writeRegistryEntry(req);
// }

DII::Object
Registry::readRegistryEntry(const DII::Object& obj)
{
  DII::Object result;

  if(!obj.IsString() && !obj.IsMap())
    return false;

  std::string key;
  std::string path;

  const DII::Object::MapType* target = &tree_;
  DII::Object::MapType::const_iterator itr;

  std::queue<std::string> subdirs;
  std::string subdir;
  
  if(obj.IsString())
    {
      INFO(cerr << "Registry: converting String to path ..." << endl;);
      std::queue<std::string> tmp;
      split_path(obj.AsString(), tmp);
      
      if(tmp.empty())
	return false;
    
      while(tmp.size() > 1)
	{
	  subdirs.push(tmp.front());
	  path = path+"/"+tmp.front();
	  tmp.pop();
	}

      key = tmp.front();
    }
  else
    {

      const DII::Object::MapType& param = obj.AsMap();

      itr = param.find("key");
      if(itr != param.end())
	key = itr->second.AsString();

      itr = param.find("path");
      if(itr != param.end())
	{
	  path = itr->second.AsString();
	  //split the path into its components
	  split_path(path, subdirs);
	}
    }

  INFO(cerr << "Registry read from ";);

  while(!subdirs.empty())
    {
      //take first level subdir
      subdir = subdirs.front();
      //remove it from the queue
      subdirs.pop();

      INFO(cerr << "/" << subdir ;);

      itr = target->find(subdir);
      if(itr != target->end())
	{
	  target = &(itr->second.AsMap());
	}
      else
	{
	  break;
	}
      
    }

  INFO(cerr << "/" << key;);

  itr = target->find(key);

  if(itr != target->end())
    {
      INFO( cerr << "  [OK]" << endl;);
      return itr->second;
    }
 
  INFO(cerr << "  [FAILED]" << endl;);
    
  return false;
}

void
Registry::writeRegistryEntry(const DII::Object& obj)
{
  DII::Object value;

  if(obj.IsMap())
    {
      DII::Object::MapType::const_iterator itr;
      DII::Object::MapType::iterator itr2;
      const DII::Object::MapType& param = obj.AsMap();

      std::string key;
      std::string path;
      
      DII::Object::MapType* target = &tree_;


      itr = param.find("key");
      if(itr != param.end())
	key = itr->second.AsString();
      itr = param.find("value");
      if(itr != param.end())
	value = itr->second;

      INFO( cerr << "Registry write to ";);

      itr = param.find("path");
      if(itr != param.end())
	{
	  path = itr->second.AsString();

	   std::queue<std::string> subdirs;
	   std::string subdir;

	   //split the path into its components
	   split_path(path, subdirs);
	   
	   while(!subdirs.empty())
	     {
	       //take first level subdir
	       subdir = subdirs.front();
	       //remove it from the queue
	       subdirs.pop();

	       INFO( cerr << "/" << subdir; );

	       itr2 = target->find(subdir);
	       if(itr2 == target->end())
		 {
		   (*target)[subdir] = DII::Object::MapType();
		   itr2 = target->find(subdir);
		 }
	       
	       target = &(itr2->second.AsMap());
	     }
	}

      INFO(cerr << "/" << key << "  [OK]" << endl; );

      (*target)[key] = value;
      
    }
  else
    {
      INFO( cerr << "Registry wrong type passed to writeRegistryEntry()" 
	     << endl;);
    }
}

std::string
Registry::readInstallDir()
{
  DII::Object entry = Registry::instance()->readRegistryEntry(
    std::string("/install_dir"));

  return entry.IsString() ? entry.AsString() : "";
}
  


