//  The Worldforge Project
//  Copyright (C) 1998, 1999 The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//
// created by Karsten Laux 01/2000

#ifndef _REGISTRY_H_
#define _REGISTRY_H_

#include <sigc++/object.h>

#include "dii/object.h"

///
class Registry : virtual public SigC::Object
{
  
 public:
  
  ///
  Registry() {assert(!instance_); instance_ = this;}
  ///
  virtual ~Registry() {assert(instance_); instance_ = 0; flushRegistry();}
  ///
  static Registry* instance() {return instance_ ? instance_ : new Registry();}

  /**read xml formatted file into internal DB.
     @param: DII::Object::String
  */
  bool open(const std::string&);
  /**write content of internal DB into given file.
     @param: DII::Object::String
  */
  void close();
  ///flushes registry to file
  bool flushRegistry();

  /**read entry (from current path if no path is specified).
     @param: DII::Object::Map
             <map>
	         <key>STRING: keyname, required </key>
		 <path>STRING: pathname, optional</path>
	     </map>
     @return: DII::Object
  */
  DII::Object readRegistryEntry(const DII::Object&);
  /**write entry.
     @param: DII::Object::Map
             <map>
	         <key>STRING: keyname, required </key>
		 <value>ATLAS::OBJECT: value</value>
                 <path>STRING: pathname, optional</path>
	     </map>
  */
  void writeRegistryEntry(const DII::Object&);
  /// Wrapper around a very common case of readRegistryEntry()
  std::string readInstallDir();
  

 protected:
  ///
  DII::Object::MapType tree_;
  ///
  std::string filename_;
 
  static Registry *instance_;
};

#endif
