// This file may be redistributed and modified only under the terms of
// the GNU Lesser General Public License (See COPYING for details).
// Copyright (C) 2001 Karsten Laux <klaux@rhrk.uni-kl.de>

#include "grid.h"

#include <assert.h>
#include <iostream>

Grid::Grid()
{
}

Grid::Grid(const Grid& grid)
{
  std::cerr << "assigning Grid instances is not implemented ... !" << std::endl;
  assert(false);
}

Grid::~Grid()
{
}

void
Grid::set(int i, int j, ValueType data)
{
  // search for row whose index is equal to j 
  Array::iterator j_itr = data_.find(j);
  if(j_itr == data_.end())
    {
      //if not found create and insert new row
      Row row;
      row[i] = data;
      data_[j] = row;
    }
  else
    {
      // search for column i in this found row
      Row::iterator i_itr = j_itr->second.find(i);
      if(i_itr == j_itr->second.end())
	{
	  //not found, insert new col
	  j_itr->second[i] = data;  
	}
      else
	{
	  i_itr->second = data;
	}
    }
}

Grid::ValueType&
Grid::at(int i, int j) throw (EmptyNode)
{
  // search for row whose index is equal to j 
  Array::iterator j_itr = data_.find(j);
  if(j_itr == data_.end())
    throw EmptyNode();
    
  // search for column i in this found row
  Row::iterator i_itr = j_itr->second.find(i);
  if(i_itr == j_itr->second.end())
    throw EmptyNode(); 

  return i_itr->second;
}

const Grid::ValueType&
Grid::at(int i, int j) const throw (EmptyNode)
{
  // search for row whose index is equal to j 
  Array::const_iterator j_itr = data_.find(j);
  if(j_itr == data_.end())
    throw EmptyNode();
    
  // search for column i in this found row
  Row::const_iterator i_itr = j_itr->second.find(i);
  if(i_itr == j_itr->second.end())
    throw EmptyNode(); 

  return i_itr->second;
}

void
Grid::clear(int i, int j) throw (EmptyNode)
{
  // search for row whose index is equal to j 
  Array::iterator j_itr = data_.find(j);
  if(j_itr == data_.end())
    throw EmptyNode();
    
  // search for column i in this found row
  Row::iterator i_itr = j_itr->second.find(i);
  if(i_itr == j_itr->second.end())
    throw EmptyNode(); 

  //remove col entry
  j_itr->second.erase(i_itr);
  //if row is empty remove row entry, too
  if(j_itr->second.empty())
    data_.erase(j_itr);

}

void
Grid::clear()
{
  data_.clear();
}

Grid::iterator::iterator()
{
  valid_i_ = valid_j_ = false;
}

Grid::iterator::iterator(const Grid::iterator& itr)
{
  i_itr_ = itr.i_itr_;
  j_itr_ = itr.j_itr_;
  i_ = itr.i_;
  j_ = itr.j_;
  data_ = itr.data_;
  valid_i_ = itr.valid_i_;
  valid_j_ = itr.valid_j_;
}

Grid::iterator::iterator(Grid* grid, int i, int j)
{
  data_ = &(grid->data_);
  i_ = i;
  j_ = j;

  // search for row whose index is equal to j 
  j_itr_ = data_->find(j);
  if(j_itr_ == data_->end())
    {
      valid_i_ = valid_j_ = false;
    }
  else
    {
      valid_j_ = true;
      // search for column i in this found row
      i_itr_ = j_itr_->second.find(i);
      valid_i_ = (i_itr_ != j_itr_->second.end());
    }
}

Grid::ValueType&
Grid::iterator::value() throw(Grid::OutOfRange)
{
  if(!valid())
    throw Grid::OutOfRange();

  return i_itr_->second;
}

bool
Grid::iterator::operator==(const Grid::iterator& a)
{
  return data_ == a.data_ && 
    i_itr_ == a.i_itr_ && j_itr_ == a.j_itr_ && 
    i_ == a.i_ && j_ == a.j_;
}

bool
Grid::iterator::operator!=(const Grid::iterator& a)
{
  return data_ != a.data_ || 
    i_itr_ != a.i_itr_ || j_itr_ != a.j_itr_ || 
    i_ != a.i_ || j_ != a.j_ ;
}

Grid::iterator&
Grid::iterator::next_i()
{
  if(valid_i_)
    {
      //ok, we own valid j_itr_ and i_itr_
      i_itr_++;
      if(i_itr_ == j_itr_->second.end())
	{
	  //upps we reached the end
	  i_++;
	  valid_i_ = false;
	}
      else
	{
	  //ok, found next element in this row
	  i_ = i_itr_->first;
	}
    }
  else
    {
      // not valid ... check where we are expected to be:
      if(valid_j_)
	{
	  /* ok, there is a row available for this j, 
	     so i_itr_ is invalid and we need to find next node, whose
	     i value is greater than current i_ */
	  i_itr_ = j_itr_->second.upper_bound(i_);
	  if(i_itr_ != j_itr_->second.end())
	    {
	      // got one !
	      valid_i_ = true;
	      i_ = i_itr_->first;
	    }
	  else
	    {
	      //there is no node where we expected one, so just inc i_
	      i_++;
	    }
	}
      else
	{ 
	  //neither valid j_itr_ nor valid i_itr_ so just increment i_
	  i_++;
	}
    }
  return *this;
}

Grid::iterator&
Grid::iterator::prev_i()
{
  if(valid_i_)
    {
      //ok, we own valid j_itr_ and i_itr_
      if(i_itr_ != j_itr_->second.begin())
	{
	  --i_itr_;
	  i_ = i_itr_->first;
	}
      else
	{
	  //reached beginning
	  //marked i_itr_ as invalid
	  i_itr_ = j_itr_->second.end();
	  --i_;
	  valid_i_ = false;
	}
    }
  else
    {
      // not valid ... check where we are expected to be:
      if(valid_j_)
	{
	  /* ok, there is a row available for this j, 
	     so i_itr_ is invalid and we need to find next node, whose
	     i value is smaller than current i_ */

	  //find first node whose key is not less than i_
	  i_itr_ = j_itr_->second.lower_bound(i_);
	  
	  //ok, now we own valid j_itr_ and i_itr_
	  if(i_itr_ != j_itr_->second.begin() && 
	     i_itr_ != j_itr_->second.end())
	    {
	      //one step down
	      --i_itr_;
	      i_ = i_itr_->first;
	      valid_i_ = true;
	    }
	  else
	    {
	      //reached beginning/end
	      //marked i_itr_ as invalid
	      i_itr_ = j_itr_->second.end();
	      --i_;
	    } 
	}
      else
	{
	  --i_;
	}
    }
  return *this;
}

Grid::iterator&
Grid::iterator::next_j()
{
  if(valid_j_)
    j_itr_++;
  else
    j_itr_ = data_->lower_bound(j_);

  if(j_itr_ != data_->end())
    {
      valid_j_ = true;
      j_ = j_itr_->first;
      i_itr_ = j_itr_->second.find(i_);
      valid_i_ = (i_itr_ != j_itr_->second.end());
    }
  else
    {
      valid_j_ = false;
      valid_i_ = false;
      j_++;
    }
    
  return *this;
}

Grid::iterator&
Grid::iterator::prev_j()
{
  if(valid_j_)
    {
      if(j_itr_ != data_->begin())
	{	 
	  j_itr_--;
	  j_ = j_itr_->first;
	  i_itr_ = j_itr_->second.find(i_);
	  valid_i_ = (i_itr_ != j_itr_->second.end());
	}
      else
	{
	  valid_j_ = false;
	  valid_i_ = false;
	  j_itr_ = data_->end();
	  j_--;
	}
    }
  else
    {
      j_itr_ = data_->lower_bound(j_);
      if(j_itr_ != data_->end() && j_itr_ != data_->begin())
	{
	  j_itr_--;
	  j_ = j_itr_->first;
	  valid_j_ = true;
	  i_itr_ = j_itr_->second.find(i_);
	  valid_i_ = (i_itr_ != j_itr_->second.end());

	}
      else
	{
	  valid_i_ = false;
	  valid_j_ = false;
	  j_itr_ = data_->end();
	  j_--;
	}
    }
  
  return *this;
}

      
