// This file may be redistributed and modified only under the terms of
// the GNU Lesser General Public License (See COPYING for details).
// Copyright (C) 2001 Karsten Laux <klaux@rhrk.uni-kl.de>

#ifndef _GRID_H_
#define _GRID_H_

#include <map>

class Grid
{  
 public:
  ///
  typedef void* ValueType;

  ///
  Grid();
  ///
  Grid(const Grid&);
  ///
  ~Grid();
  ///
  void clear();

  /** exceptions */
  //@{
  ///
  class EmptyNode {};
  ///
  class OutOfRange {};
  //@}

 protected:
  ///
  typedef std::map<int, ValueType> Row;
  ///
  typedef std::map<int, Row> Array;
  ///
  Array data_;

 public:

  /** access via iterators */
  //@{
  ///
  friend class iterator
    {
    protected:
      Grid::Row::iterator i_itr_;
      Grid::Array::iterator j_itr_;
      int i_,j_;
      Grid::Array* data_;
      bool valid_i_, valid_j_;

    public:
      iterator();
      iterator(const iterator&);
      iterator(Grid*, int i, int j);     

      Grid::ValueType& value() throw (Grid::OutOfRange);
      int i() const { return i_;};
      int j() const { return j_; };
      bool valid() const { return valid_i_ && valid_j_; };
      bool valid_i() const { return valid_i_; };
      bool valid_j() const { return valid_j_; };
      bool operator==(const iterator& );  
      bool operator!=(const iterator& );

      iterator& next_i();
      iterator& prev_i();
      iterator& next_j();
      iterator& prev_j();
 
    };
  //@}

  /** random access */
  //@{
  ///throws exception if there is nothing
  ValueType& at(int i, int j) throw (EmptyNode);
  ///throws exception if there is nothing
  const ValueType& at(int i, int j) const throw (EmptyNode);
  ///
  void set(int i, int j, ValueType data);
  ///
  void clear(int i, int j) throw (EmptyNode);
  //@}
  


};

#endif
  
