#include "lua_script.h"

#include <iostream> 

using namespace Lua;

// This function will be called from Lua
static int calcPos(Script::Handle handle)
{
  Script* script = Script::getRef(handle);
  assert(script);
  int argcount = script->getStacksize();
  assert(argcount == 3);

  Value t = script->popValue();
  Value pos = script->popValue();
  Value vel = script->popValue();

  assert(vel.isTable());

  Value result = Value::TableType();
  result[0] = pos[0].asNumber() + vel[0].asNumber()*t.asNumber();
  result[1] = pos[1].asNumber() + vel[1].asNumber()*t.asNumber();
  result[2] = pos[2].asNumber() + vel[2].asNumber()*t.asNumber(); 

  script->pushValue(result);
  return 1;
}
 
// This function will be called from Lua
static int change(Script::Handle handle)
{
  Script* script = Script::getRef(handle);
  assert(script);
  int argcount = script->getStacksize();
  Value result;

  try
    {
      if(argcount > 0)
	{
	  result = script->popValue();
	  if(result.isTable() && result.has("name") && result.has("version"))
	    {
	      cout << result["name"].asString()<<" V." << result["version"].asNumber() << endl;
 	      result["name"] = "LuaScript";
 	      result["version"] = 1.0;
	    }
	}
    }
  catch(...)
    {
      cout << "exception while processing args in helloLua()" << endl;
    }
  
  
  script->pushValue(result);

  // we return one result
  return 1;
}

int main (int, char**)
{
  Script script;

  //register function
  script.registerFunc("change",change);
  script.registerFunc("calcPos",calcPos);

  // Execute file
  script.doFile("test.lua");
 
  // Get Lua global variables 
  Value name = script.getGlobal("name");
  Value version = script.getGlobal("version");
  
  if(name.isString())
    cout << "name = " << name.asString() << endl;

  if(version.isNumber())
    cout << "version = " << version.asNumber() << endl;

  script.call("helloC");
 
  // Call Lua function
  Script::ListType args;
  args.push_back(name);
  args.push_back(version);
  script.call("display",args);
 

  // Call Lua function with C types
  args.clear();
  args.push_back("it works !");
  script.call("print",args);

  script.registerType("List");
  script.registerType("Map");

  Value lstObj = script.createObject("List",Value::TableType());
  Value mapObj = script.createObject("Map", Value::TableType());
  
  script.pushValue(mapObj);

  Value maptest = script.popValue();
  if(maptest.tag() == script.getTypeTag("Map"))
    cout << "got a Map object ! " << endl;
 

  script.call("vectorfun");

 return 0;
}
