-- this script provides the following commands (meant for testing mainly)
--
-- * Setting options at runtime: (type is either "Int", "Float" or "String")
--   /set	<module> <option> <val> [<type>]    	
--

cal_button = nil

-- init this module
function init()
   cal_button = DII_callMethod("Gui","bind_menubutton",0)
   cal_button.upPic = "mcal_button"
   cal_button.downPic = "mcal_button_pressed"
   DII_callMethod("Gui","change_menubutton",cal_button)

   DII_registerMsgHandler("Gui","menuCallback")

   return 1
end

-- shutdown the module
function deinit()
   DII_callMethod("Gui","release_menubutton",cal_button)
   return 1
end

--
--
function menuCallback(arg)
   if arg.args[1].id == cal_button.id then
     time()
   end
   return 0
end

function loadModul(name)
   local arg = { name }
   settag(arg, List_tag)
   DII_callService("mod_load", arg)
end

function loadScript(name)
   local arg = { name }
   settag(arg, List_tag)
   DII_callService("script_load", arg)
end

-- service "sreload" reloads the given script
--
function sreload(arg)
   if type(arg) ~= "table" then return 0 end
   
   settag(arg, List_tag)
   DII_callService("mod_unload", arg)
   DII_callService("script_load", arg)
      
   return 1
end

-- service "mreload" reloads the given module
--
function mreload(arg)
   if type(arg) ~= "table" then return 0 end
   
   settag(arg, List_tag)
   DII_callService("mod_unload", arg)
   DII_callService("mod_load", arg)
      
   return 1
end
   
-- service "set" which changes values in the global
-- varconf::Config instance
--
function set(arg)
   if type(arg) ~= "table" then return 0 end

   if arg[1] == "help" then
	msg = "no help available yet.\n"
        DII_callService("std_out",msg)
        return 0
   end
   
   setting = {}
   if arg[1] == "global" then
      setting.section = ""
   else
      setting.section = arg[1] 
   end

   setting.key = arg[2]
   
   if arg[4] then 
      if arg[4] == "Int" or arg[4] == "Float" then
         setting.value = tonumber(arg[3])
      end
   else
      setting.value = tostring(arg[3])
   end

   msg = " setting "..setting.section.."::"..setting.key.." to value "..tostring(setting.value).."\n"
   DII_callService("std_out",msg);
   -- tag "setting" to get treated as a Map-type
   settag(setting,Map_tag)
   DII_callMethod("Settings","setItem", setting);
   
   return 1
end
	
   
--
--
--
day_names = { "Lunot", "Manot", "Cianot", "Arnot", "Onot", "Sanot", "Nanot", "Tannot" }
month_names = { "Tynus", "Ronn", "Nyste", "Talos", "Etro", "Tuloc", "Jobe", "Enro", "Tuwen", "Umar", "Astu", "Usk" }
order_names = { "first", "second", "third", "fourth", "fifth" }

function time()
   local time = DII_callMethod("Player","getTime",0)
   if type(time) ~= "table" then return 0 end

   if not time.month then time.month = 1 end
   if not time.day then time.day = 1 end
   if not time.year then time.year = 0 end
   if not time.hour then time.hour = 0 end
   if not time.minute then time.minute = 0 end
   if not time.second then time.second = 0 end
   if not time.tick then time.tick = 0 end

   local week
   local weekday

   if time.day < 15 then
     week = 0
     weekday = time.day
   else
     week = 2
     weekday = time.day - 15
   end

   if weekday >= 7 then
     week = week + 1
     weekday = weekday - 7
   end

   time_string = "Today is the day "..day_names[weekday+1].." in the "..order_names[week+1].." week of the month "..month_names[time.month+1].."\n"

   -- The month and day start with zero, we want the displayed version to start with 1
   time_string = time_string ..format("date: %1$2d/%2$2d/%3$2d time: %4$2d:%5$02d:%6$02d  (%7$7d)",
                         time.month+1, time.day+1, time.year, 
                         time.hour, time.minute, time.second, time.tick)  


   DII_callService("std_out",time_string.."\n")
   return 1
end

-- all following lines are "global" code and executed as soon
-- as the script has been loaded and compiled
--

-- the semantics of the calls are the same as for binary modules,
-- just the name of the module is omitted
-- (see README.modules)
--
REGISTER_METHOD("init")
REGISTER_METHOD("deinit")

REGISTER_METHOD("time")
REGISTER_AS_SERVICE("time")
REGISTER_METHOD("set")
REGISTER_AS_SERVICE("set")
REGISTER_METHOD("sreload")
REGISTER_AS_SERVICE("sreload")
REGISTER_METHOD("mreload")
REGISTER_AS_SERVICE("mreload")

NEEDS_MODULE("Settings")
