-- fileaccess.lua
-- 
-- Module must be run in "trusted" mode as it requires fileaccess
--
-- This module offers a chrooted access to the local filesystem; all
-- file requests are re-rooted to the installation directory of
-- uclient
--


--
-- globals
--
rootdir = ""
handles = {}
MAX_HANDLE = 32

-- init this module
--
function init()
   mydebug("fileaccess.lua: Initializing chrooted fileaccess ...")
   rootdir = DII_callService("readRegistryEntry", "/install_dir")
   mydebug("                using "..rootdir.." as root directory.\n")

   -- initialize filehandle table (set all to nil)
   local index = 0;
   while index < MAX_HANDLE do
       handles[index] = nil
       index = index + 1
   end

   return 1
end

-- shutdown the module
--
function deinit()
   return 1
end

-- 
--
function requestIndex()
   local index = 1
   while index <= MAX_HANDLE do
      if handles[index] == nil then return index end
      index = index + 1
   end
   return nil
end

--
--
function file_user_open(arg)
    if type(arg) ~= "table" then return 0 end

    -- ensure that we are not fooled
    if strfind(arg[1],"..",1,1) then
       mydebug("fileaccess.lua: client passes relative filename !")
       mydebug("                Request ignored.")
       return nil
    end

    local filename = getenv("HOME").."/.uclient/"..arg[1]
    mydebug("fileaccess.lua: accessing file "..filename.."\n")
    local index = requestIndex()
    if index then
       handles[index] = openfile(filename, arg[2])
       return index
    end
    mydebug("fileaccess.lua: no more file handles left !")
    return nil
end

--
--
function file_open(arg)
    if type(arg) ~= "table" then return 0 end

    -- ensure that we are not fooled
    if strfind(arg[1],"..",1,1) then
       mydebug("fileaccess.lua: client passes relative filename !")
       mydebug("                Request ignored.")
       return nil
    end

    local filename = rootdir.."/"..arg[1]
    mydebug("fileaccess.lua: accessing file "..filename.."\n")
    local index = requestIndex()
    if index then
       handles[index] = openfile(filename, arg[2])
       return index
    end
    mydebug("fileaccess.lua: no more file handles left !")
    return nil
end

--
--
function file_close(index)
    if handles[index] then
        closefile(handles[index])
        handles[index] = nil
        return 1
    end
    return 0
end

--
--
function file_flush(index)
    if handles[index] then
        flush(handles[index])
        return 1
    end
    return 0
end

--
--
function file_read(arg)
    local content = ""
    if type(arg) ~= "table" then return 0 end
    if handles[arg[1]] then
       content = read(handles[arg[1]], arg[2])
    end
    if content == nil then
        mydebug("fileaccess.lua: error during reading file !")
    end
    return content
end

--
--
function file_write(arg)
    if type(arg) ~= "table" then return 0 end
    if handles[arg[1]] then
       return write(handles[arg[1]], arg[2])
    end
    return 0
end

--
--
function file_seek_set(arg)
    if type(arg) ~= "table" then return 0 end
    if handles[arg[1]] then
       return seek(handles[arg[1]], "set", arg[2])
    end
    return 0
end
--
--
function file_seek_current(arg)
    if type(arg) ~= "table" then return 0 end
    if handles[arg[1]] then
       return seek(handles[arg[1]], "cur", arg[2])
    end
    return 0
end

--
--
function file_seek_end(arg)
    if type(arg) ~= "table" then return 0 end
    if handles[arg[1]] then
       return seek(handles[arg[1]], "end", arg[2])
    end
    return 0
end

-- all following lines are "global" code and executed as soon
-- as the script has been loaded and compiled
--
-- the semantics of the calls are the same as for binary modules,
-- just the name of the module is omitted
-- (see README.modules)
--
REGISTER_METHOD("init")
REGISTER_METHOD("deinit")

REGISTER_METHOD("file_open")
REGISTER_METHOD("file_user_open")
REGISTER_METHOD("file_close")
REGISTER_METHOD("file_flush")
REGISTER_METHOD("file_read")
REGISTER_METHOD("file_write")
REGISTER_METHOD("file_seek_set")
REGISTER_METHOD("file_seek_current")
REGISTER_METHOD("file_seek_end")

REGISTER_AS_SERVICE("file_open")
REGISTER_AS_SERVICE("file_user_open")
REGISTER_AS_SERVICE("file_close")
REGISTER_AS_SERVICE("file_flush")
REGISTER_AS_SERVICE("file_read")
REGISTER_AS_SERVICE("file_write")
REGISTER_AS_SERVICE("file_seek_set")
REGISTER_AS_SERVICE("file_seek_current")
REGISTER_AS_SERVICE("file_seek_end")

