-- tools for creating Janus dialog or widget descriptions

function newJanusItem(class, attributes)

   item = DII_newMap()

   -- copy attributes entries (if any) to item
   if attributes and type(attributes)  == "table" then
      itr, val = next(attributes, nil)
      while itr ~= nil do
         if val then
           item[itr]=val
         end 
         itr, val = next(attributes, itr)
      end
   end

   item.type = class;
   return item;
end

function newJanusDescription()
   desc = newJanusItem("janus")
   desc.style = newJanusItem("style")
   desc.data = newJanusItem("data")
   desc.structure = newJanusItem("structure")
   desc.logic = newJanusItem("logic")
  
   return desc
end

function newBorderedDialog(x,y,width,height)
   
   local border = 16;
   local widget = newJanusItem("Dialog",{ x=x, y=y, width=width+2*border, height=height+2*border})

   widget.border_left = 
      newJanusItem("Widget",{x=0,y=border,width=border, height=height,background="dialogbox_frame_v"})
   widget.border_right = 
      newJanusItem("Widget",{x=width+border,y=border,width=border, height=height,background="dialogbox_frame_v"})
   widget.border_top = 
      newJanusItem("Widget",{x=border,y=0,width=width, height=border,background="dialogbox_frame_h"})
   widget.border_bottom = 
      newJanusItem("Widget",{x=border,y=height+border,width=width, height=border,background="dialogbox_frame_h"})

   widget.edge0 = 
      newJanusItem("Widget",{x=0,y=0,width=border, height=border,background="border_edge"})
   widget.edge1 = 
      newJanusItem("Widget",{x=0,y=height+border,width=border, height=border,background="border_edge"})
   widget.edge2 = 
      newJanusItem("Widget",{x=width+border,y=0,width=border, height=border,background="border_edge"})
   widget.edge3 = 
      newJanusItem("Widget",{x=width+border,y=height+border,width=border, height=border,background="border_edge"})

   widget.clientArea = newJanusItem("Widget",{x=border, y=border, width=width, height=height});

   return widget
end

function bindScreenSection(section)
   
--   request = { section = section }
--   settag(request, Map_tag)

   request = DII_newMap({section = section})

   offer = DII_callMethod("Gui","request_screenarea", request)

   if type(offer) ~= "table" then return nil end

   return DII_callMethod("Gui","bind_screenarea", offer)
end

function releaseScreenSection(section)
   DII_callMethod("Gui","release_screenarea",section)
end

loadExtension("dii.lua")