// The Worldforge Project
// Copyright (C) 1998, 1999,2000,2001 The Worldforge Project
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
// 
// For information about Worldforge and its authors, please contact
// the Worldforge Web Site at http://www.worldforge.org.
// 
// created by Karsten Laux 05/2001

#include "tilestretch.h"
#include <assert.h>

/* TODO: Change tile size to 72x36 ! */

/*
 * this all only works on 16 bits surfaces ! 
 */
#define BPP 2
#define DEST_BPP 2
#define SOURCE_BPP 2

#define TILE_WIDTH 70
#define TILE_HEIGHT 36

#define SHADE_SCALE 2

/*
 * --------------------------------------
 */

/*
 * different blit methods ... useful for debugging only 
 */

#define MEMCPY \
*((sint16*)dPtr) = *((sint16*)sPtr);

#define TRANS_PIXEL \
pixel = *((sint16*)sPtr); \
if(pixel != _source->transColor)   \
  *((sint16*)dPtr) = pixel;

#define ONLY_SHADED

#define TARGET_TEST *((sint16*)dPtr)=0x5523;

#define SOURCE_TEST pixel = *((sint16*)sPtr);

/* #define BLIT_PIXEL MEMCPY */
#define BLIT_PIXEL TRANS_PIXEL
/* #define BLIT_PIXEL ONLY_SHADED */

/*
 * ---------------------------------------
 */

/*
 * tables describing the borders of a standard tile (70x36) 
 */
/*
 * static const sint16 _ys_start[] = {
 * 18,18,16,16,14,14,13,13,12,12,11,11,10,10, 
 * 9,9,8,8,7,7,6,6,5,5,4,4,3,3, 
 * 2,2,1,1,0,0,1,1,2,2,3,3,4,4, 
 * 5,5,6,6,7,7,8,8,9,9,10,10,11,11, 
 * 12,12,13,13,14,14,15,15,16,16,17,17,18,18 }; 
 * static const sint16 _ys_end[] = { 19,19,20,20,21,21,22,22,23,23,24,24,25,25, 
 * 26,26,27,27,28,28,29,29,30,30,31,31,32,32, 
 * 33,33,34,34,35,35,36,36,35,35,34,34,33,33, 
 * 32,32,31,31,30,30,29,29,28,28,27,27,26,26, 
 * 25,25,24,24,23,23,22,22,21,21,20,20,19,19 }; 
 */

static const sint16 _ys_start[] =
    { 17, 17, 16, 16, 15, 15, 14, 14, 13, 13, 12, 12, 11, 11, 10, 10,
    9, 9, 8, 8, 7, 7, 6, 6, 5, 5,
    4, 4, 3, 3, 2, 2, 1, 1, 0, 0, 1, 1, 2, 2, 3, 3, 4, 4,
    5, 5, 6, 6, 7, 7, 8, 8, 9, 9,
    10, 10, 11, 11, 12, 12, 13, 13, 14, 14, 15, 15, 16, 16, 17, 17
};

static const sint16 _ys_end[] =
    { 18, 18, 19, 19, 20, 20, 21, 21, 22, 22, 23, 23, 24, 24,
    25, 25, 26, 26, 27, 27, 28, 28, 29, 29, 30, 30, 31, 31,
    32, 32, 33, 33, 34, 34, 35, 35, 34, 34, 33, 33, 32, 32,
    31, 31, 30, 30, 29, 29, 28, 28, 27, 27, 26, 26, 25, 25,
    24, 24, 23, 23, 22, 22, 21, 21, 20, 20, 19, 19, 18, 18
};

/*
 * tables for caching the borders of target area 
 */
//static sint16 _yt_start[70];
//static sint16 _yt_end[70];

/*
 * tables for the shading values 
 */
//static sint16 _slope_start[70];
//static sint16 _slope_end[70];

static Stretch_TargetSurface *_target;
static Stretch_SrcSurface *_source;
static sint8 *_shadow;

/********************************************
 * internal functions
 ********************************************/

void
calc_border(sint16 * scan_table,
            sint16 start_x, sint16 start_y, sint16 end_x, sint16 end_y)
{
    sint16 dx,
     dy,
     sdx,
     sdy,
     x,
     y,
     px,
     py;

    dx = end_x - start_x;
    dy = end_y - start_y;

   /*  The following is a quick implementation of Bresenham's algorithm,
    *  an efficient algorithm to render a line with pixels. The long dimension
    *  is incremented for each pixel, and the fractional slope is accumulated.
    *
    *  See http://www.nist.gov/dads/HTML/bresenham.html
    */

    sdx = (dx < 0) ? -1 : 1;
    sdy = (dy < 0) ? -1 : 1;

    dx = sdx * dx + 1;
    dy = sdy * dy + 1;

    // FIXME the code that uses this function is somewhat broken
    // assert(dx >= dy);

    x = y = 0;

    px = start_x;
    py = start_y;

    for (x = 0; x < dx; x++) {
        scan_table[px] = py;

        y += dy;
        if (y >= dx) {
            y -= dx;
            py += sdy;
        }
        px += sdx;
    }
}

/***********************************************
 * exported functions
 ***********************************************/

void
Stretch_setShadingTarget(sint8 * target)
{
    _shadow = target;
}

void
Stretch_setSource(Stretch_SrcSurface * source)
{
    _source = source;
}

void
Stretch_setTarget(Stretch_TargetSurface * target)
{
    _target = target;
}

void
Stretch_blit(Stretch_Args * args)
{
    sint16 x_min,
     x_max,
     y_min,
     y_max,
     s;
    sint16 yt_delta,
     ys_delta;
    sint16 sl,
     sl_end,
     sl_delta,
     sl_step,
     sl_h;
    uint8 *dPtr;
    uint8 *sPtr;
    sint8 *gPtr;
    sint16 x;
    sint16 y;
    uint16 pixel;

    /*
     * tables for caching the borders of target area 
     */
    sint16 _yt_start[70];
    sint16 _yt_end[70];

    /*
     * tables for the shading values 
     */
    sint16 _slope_start[70];
    sint16 _slope_end[70];

    /*
     * compute the border lines of the stretched tile 
     */
    calc_border(_yt_start, 0, args->h3, 34, args->h2);
    _yt_start[34] = args->h2;
    _yt_start[35] = args->h2;
    calc_border(_yt_start, 36, args->h2, 69, args->h1);

    calc_border(_yt_end, 0, args->h3 + 1, 34, args->h0);
    _yt_end[34] = args->h0;
    _yt_end[35] = args->h0;
    calc_border(_yt_end, 36, args->h0, 69, args->h1 + 1);


    /*
     * compute the slope values along the lines of the stretched tile 
     */
    calc_border(_slope_start, 0, args->sl3, 34, args->sl2);
    calc_border(_slope_start, 35, args->sl2, 69, args->sl1);
    calc_border(_slope_end, 0, args->sl3, 34, args->sl0);
    calc_border(_slope_end, 35, args->sl0, 69, args->sl1);


    /*
     * calcute clipping x-range 
     */
    x_min = (args->offset_x >= 0) ? 0 : -args->offset_x;
    x_max = (TILE_WIDTH + args->offset_x < _target->width)
        ? TILE_WIDTH : _target->width - args->offset_x;


    for (x = x_min; x < x_max; x++) {
        /*
         * calculate slope range 
         */
        sl = _slope_start[x];
        sl_end = _slope_end[x];

        sl_delta = sl_end - sl;
        if (sl_delta < 0) {
            sl_delta = -sl_delta;
            sl_step = -1;
        } else {
            sl_step = 1;
        }

        yt_delta = _yt_end[x] - _yt_start[x];
        ys_delta = _ys_end[x] - _ys_start[x];

        /* safety jumps ... */
        if (ys_delta <= 0)
            break;
        if (yt_delta <= 0)
            break;

        /*
         * clip y range to fit inside the target surface and calculate
         * starting values for the gPtr and dPtr 
         */
        if (_yt_start[x] + args->offset_y > 0) {
            y_min = _yt_start[x];

            gPtr =
                (sint8 *) _shadow + (x +
                                     args->offset_x) / DEST_BPP /
                SHADE_SCALE + (_yt_start[x] + args->offset_y) / 2 * 256;

            dPtr =
                (uint8 *) _target->pixels + (x +
                                             args->offset_x) * DEST_BPP +
                (_yt_start[x] + args->offset_y) * _target->pitch;
        } else {
            y_min = -args->offset_y;
            gPtr = _shadow + (x + args->offset_x) / DEST_BPP / SHADE_SCALE;
            dPtr = _target->pixels + (x + args->offset_x) * DEST_BPP;
        }

        y_max = (_yt_end[x] + args->offset_y < _target->height) ?
            _yt_end[x] + 1 : _target->height - args->offset_y;

        /*
         * Depending on the clipped y values calculate start value of
         * source pointer: sPtr 
         */
        sPtr = _source->pixels + (x + _source->offset_x) * SOURCE_BPP
            + (_source->offset_y + _ys_start[x] +
               (y_min -
                _yt_start[x]) * ys_delta / yt_delta) * _source->pitch;

        /*
         * The inner loop: Draw a vertical textured line to the target
         * surface using the source surface (tile) as texture 
         */
        s = 0;
        sl_h = 0;
        // for(y = y_min; y < y_max+1; y++)
        for (y = y_min; y < y_max; y++) {
            /*
             * bresenham's algorithm again...
             */
            s += ys_delta;
            if (s > yt_delta) {
                s -= yt_delta;
                sPtr += _source->pitch;
            }
            sl_h += sl_delta;
            if (sl_h > yt_delta) {
                sl_h -= yt_delta;
                sl += sl_step;
            }

						/* BLIT_PIXEL is a macro... see above */
            BLIT_PIXEL
                /*
                 * jump to next target row 
                 */
                dPtr += _target->pitch;

            /*
             * every target row but one, jump to next gamma map row 
             */
            if (y % SHADE_SCALE) {
                *gPtr = sl;
                gPtr += 256;
            }

        }

    }                           

}


void
LeftOverlap_blit(Stretch_Args * args, sint16 overlap)
{
    sint16 x_min,
     x_max,
     y_min,
     y_max;
    sint16 yt_delta,
     ys_delta;
    uint8 *dPtr,
    *sPtr;

    sint16 x,
     y;
    uint16 pixel;

    /*
     * tables for caching the borders of target area 
     */
    sint16 _yt_start[70];
    sint16 _yt_end[70];

    /*
     * compute the top and bottom border lines (only left half of tile) 
     */
    calc_border(_yt_start, 0, args->h3 - 18 - _source->offset_y, 34,
                args->h2 - _source->offset_y);
    _yt_start[34] = args->h2 - _source->offset_y;
    calc_border(_yt_end, 0, args->h3, 34, args->h2);
    _yt_end[34] = args->h2;

    /*
     * calcute clipping x-range 
     */
    x_min = (args->offset_x >= 0) ? 0 : -args->offset_x;
    x_max = (TILE_WIDTH / 2 + args->offset_x < _target->width)
        ? TILE_WIDTH / 2 : _target->width - args->offset_x;


    for (x = x_min; x < x_max; x++) {

        yt_delta = _yt_end[x] - _yt_start[x];
        ys_delta = _ys_end[x];

        // safety jumps ... 
        if (ys_delta <= 0)
            break;
        if (yt_delta <= 0)
            break;

        /*
         * clip y range to fit inside the target surface and calculate
         * starting values for the gPtr and dPtr 
         */
        if (_yt_start[x] + args->offset_y > 0) {
            y_min = _yt_start[x];

            dPtr =
                (uint8 *) _target->pixels + (x +
                                             args->offset_x) * DEST_BPP +
                (_yt_start[x] + args->offset_y) * _target->pitch;
        } else {
            y_min = -args->offset_y;
            dPtr = _target->pixels + (x + args->offset_x) * DEST_BPP;
        }

        y_max = (_yt_end[x] + args->offset_y < _target->height) ?
            _yt_end[x] : _target->height - args->offset_y;

        /*
         * Depending on the clipped y values calculate start value of
         * source pointer: sPtr 
         */
        sPtr = _source->pixels + (x + _source->offset_x) * SOURCE_BPP
            + (y_min - _yt_start[x]) * _source->pitch;

        /*
         * The inner loop: 
         */
        // for(y = y_min; y < y_max+1; y++)
        for (y = y_min; y < y_max; y++) {
            BLIT_PIXEL dPtr += _target->pitch;
            sPtr += _source->pitch;
        }

    }                           // for(x= ...
}

void
RightOverlap_blit(Stretch_Args * args, sint16 overlap)
{
    sint16 x_min,
     x_max,
     y_min,
     y_max;
    sint16 yt_delta,
     ys_delta;
    uint8 *dPtr,
    *sPtr;

    sint16 x,
     y;
    uint16 pixel;

    /*
     * tables for caching the borders of target area 
     */
    sint16 _yt_start[70];
    sint16 _yt_end[70];

    /*
     * compute the top and bottom border lines (only left half of tile) 
     */
    calc_border(_yt_start, 35, args->h2 - _source->offset_y, 69,
                args->h1 - _source->offset_y - 18);
    _yt_start[69] = args->h1 - _source->offset_y - 18;
    calc_border(_yt_end, 35, args->h2, 69, args->h1);
    _yt_end[69] = args->h1;

    /*
     * calcute clipping x-range 
     */
    x_min = (args->offset_x + TILE_WIDTH / 2 >= 0) ?
        TILE_WIDTH / 2 : TILE_WIDTH / 2 - (args->offset_x +
                                           TILE_WIDTH / 2);
    x_max = (TILE_WIDTH + args->offset_x < _target->width)
        ? TILE_WIDTH : _target->width - args->offset_x;


    for (x = x_min; x < x_max; x++) {

        yt_delta = _yt_end[x] - _yt_start[x];
        ys_delta = _ys_end[x] + overlap;

        // safety jumps ... 
        if (ys_delta <= 0)
            break;
        if (yt_delta <= 0)
            break;

        /*
         * clip y range to fit inside the target surface and calculate
         * starting values for the gPtr and dPtr 
         */
        if (_yt_start[x] + args->offset_y > 0) {
            y_min = _yt_start[x];

            dPtr =
                (uint8 *) _target->pixels + (x +
                                             args->offset_x) * DEST_BPP +
                (_yt_start[x] + args->offset_y) * _target->pitch;
        } else {
            y_min = -args->offset_y;
            dPtr = _target->pixels + (x + args->offset_x) * DEST_BPP;
        }

        y_max = (_yt_end[x] + args->offset_y < _target->height) ?
            _yt_end[x] : _target->height - args->offset_y;

        /*
         * Depending on the clipped y values calculate start value of
         * source pointer: sPtr 
         */
        sPtr = _source->pixels + (x + _source->offset_x) * SOURCE_BPP
            + (y_min - _yt_start[x]) * _source->pitch;

        /*
         * The inner loop: 
         */
        // for(y = y_min; y < y_max+1; y++)
        for (y = y_min; y < y_max; y++) {
            BLIT_PIXEL dPtr += _target->pitch;
            sPtr += _source->pitch;
        }

    }                           // for(x= ...
}
