/*
  SOLID - Software Library for Interference Detection
  Copyright (C) 1997-1998  Gino van den Bergen

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free
  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  Please send remarks, questions and bug reports to gino@win.tue.nl,
  or write to:
                  Gino van den Bergen
		  Department of Mathematics and Computing Science
		  Eindhoven University of Technology
		  P.O. Box 513, 5600 MB Eindhoven, The Netherlands
*/

#include "Box.h"
#include "BoxList.h"

static BoxList lists[3];

/*
// This is here to remind me what the variables mean
BBox Box::bbox() const {
  return BBox(pos - extent, pos + extent);
}
*/

Box::Box(DtObjectRef obj, const Vector& v) : ref(obj)
{
  for(int i = 0; i < 3; ++i)
    extent[i] = fabs(v[i]) / 2;

  setIdentity();

  for(int i = 0; i < 3; ++i)
    list_pos[i] = lists[i].add(*this, i);
}

Box::~Box()
{
  for(int i = 0; i < 3; ++i)
    lists[i].remove(list_pos[i]);
}

void Box::translate(const Vector& v)
{
  for(int i = 0; i < 3; ++i)
    lists[i].move(list_pos[i], i, v[i]);

  pos += v;
} 

bool intersect(const Box& a, const Box& b)
{
  Vector b2a = b.pos - a.pos;

  for(int i = 0; i < 3; ++i)
    if(a.extent[i] + b.extent[i] < fabs(b2a[i]))
      return false;

  return true;
}

