//  The Worldforge Project
//  Copyright (C) 2002 Ron Steinke and The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//
// created by Ron Steinke 2/17/2001

// This code is intended as a replacement for the work done by the
// Endpoint class in the SOLID library, previously used here

#include <list>
#include "Box.h"
#include "BoxList.h"

void BoxList::move(list<Box*>::iterator &i, int axis, Scalar shift)
{
  list<Box*>::iterator j = i;
  bool moved = false, at_begin = false;

  if(shift >= 0) {
    Scalar high = (*i)->pos[axis] + (*i)->extent[axis] + shift;
    ++j;

    while(j != end() && (*j)->pos[axis] - (*j)->extent[axis] <= high) {
      if(intersect(**i, **j))
        addPair(*i, *j);
      moved = true;
      ++j;
    }
  }
  else if(i != begin()) {
    Scalar low = (*i)->pos[axis] - (*i)->extent[axis] + shift;
    --j;

    while((*j)->pos[axis] + (*j)->extent[axis] >= low) {
      if(intersect(**i, **j))
        addPair(*i, *j);
      moved = true;
      if(j == begin()) {
        at_begin = true;
        break;
      }
      --j;
    }
  }

  if(!moved)
    return;

  Box *b = *i;

  erase(i);

  if(shift < 0 && !at_begin)
    ++j; // Get the element after where we want i to be

  i = insert(j, b);
}

list<Box*>::iterator BoxList::add(Box& b, int axis)
{
  Scalar val = b.pos[axis], ext = b.extent[axis], low = val - ext, high = val + ext;
  list<Box*>::iterator i, j = begin();

  while(j != end() && high < (*j)->pos[axis] - (*j)->extent[axis])
    ++j;

  i = insert(j, &b);


  if(i != begin()) {
    // Get the item before i
    --j;
    --j;

    while(j != begin() && low <= (*j)->pos[axis] + (*j)->extent[axis])
      if(intersect(**i, **j))
        addPair(*i, *j);
  }

  return i;
}
