//
//  The Worldforge Project
//  Copyright (c) 1999  The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the WorldForge Web Site at http://www.worldforge.org.
//
///

#ifndef COORD_H
#define COORD_H


#include <iostream>
#include <string>
#include <queue>
#include <stdio.h>
#include <math.h>

#include "../modules/tokenise.h"


class Coord
{
public:
// Members
	double			x;
	double			y;
	double			z;

// Constructors

	Coord();
	Coord(double init_x , double init_y , double init_z );
	Coord(const Coord &c);
	
// Assignment

	Coord &operator=(const Coord &c);	
	
// Operators

	Coord &operator+=(const Coord &c);
	Coord &operator-=(const Coord &c);
	Coord &operator*=(double d);
	Coord &operator/=(double d);
	
// Operations

	Coord &rotateZ(double theta);	// rotate theta degrees about the z axis	
	void toString(std::string &this_string) const;
	std::queue<std::string> &transmit(std::queue<std::string> &atbs);
	std::queue<const char*> &receive(std::queue<const char*> &atbs);
};
	


inline std::queue<std::string> &Coord::transmit(std::queue<std::string> &atbs)
{
	char temp[10];
	sprintf(temp,"%f",x);
	atbs.push(temp);
	sprintf(temp,"%f",y);
	atbs.push(temp);
	sprintf(temp,"%f",z);
	atbs.push(temp);
	return atbs;
}

inline std::queue<const char*> &Coord::receive(std::queue<const char*> &atbs)
{
	x = atof(atbs.front());
	atbs.pop();
	y = atof(atbs.front());
	atbs.pop();
	z = atof(atbs.front());
	atbs.pop();
	return atbs;
}


// Inlines
inline Coord::Coord()
{
	x = y = z = 0;
}

inline Coord::Coord (double init_x , double init_y , double init_z )
{
	x = init_x;
	y = init_y;
	z = init_z;
}

inline Coord::Coord (const Coord &c)
{
	x = c.x;
	y = c.y;
	z = c.z;
}


// Arithmetic operator members

inline Coord &Coord::operator=(const Coord &c)
{
	x = c.x;
	y = c.y;
	z = c.z;
	return *this;
}

inline Coord &Coord::operator+=(const Coord &c)
{
	x += c.x;
	y += c.y;
	z += c.z;	
	return *this;
}

inline Coord &Coord::operator-=(const Coord &c)
{
	x -= c.x;
	y -= c.y;
	z -= c.z;	
	return *this;
}

inline Coord &Coord::operator*=(double d)
{
	x *= d;
	y *= d;
	z *= d;	
	return *this;
}

inline Coord &Coord::operator/=(double d)
{
	return (*this)*=(1.0/d);
}

// RotateZ -- rotate about the Z axis by theta radians
inline Coord &Coord::rotateZ ( double theta )
{
	double tempx =  cos(theta)*x + sin(theta)*y;
	double tempy = -sin(theta)*x + cos(theta)*y;
	x = tempx;
	y = tempy;
	return *this;
//	z = z;	
}



inline void Coord::toString(std::string &this_string) const
{
	char temp[128];
	sprintf(temp,"\"%f %f %f\"",x,y,z);
	this_string += temp;
}

inline std::istream &operator>>(std::istream &in,Coord &c)
{
	return (in >> c.x >> c.y >> c.z);
}

inline std::ostream &operator<<(std::ostream &out,const Coord &c)
{
	return (out << c.x << " " << c.y << " " << c.z);
}



// Commutative arithmetic operators

inline Coord operator+(const Coord &c1,const  Coord &c2)
{
	return Coord(c1.x + c2.x, c1.y + c2.y, c1.z + c2.z);
}

inline Coord operator-(const Coord &c1,const  Coord &c2)
{
	return Coord(c1.x-c2.x, c1.y-c2.y, c1.z-c2.z);
}

inline Coord operator-(const Coord &c)
{
	return Coord(-c.x, -c.y, -c.z);
}


inline Coord operator*(const Coord &c,double d)
{
	return Coord(d*c.x, d*c.y, d*c.z);
}

inline Coord operator*(double d,const Coord &c)
{
	return Coord(d*c.x, d*c.y, d*c.z);
}

inline Coord operator/(const Coord &c,double d)
{
	return c*(1.0/d);
}

// Utilities

inline double norm(const Coord &c)
{
	return (c.x*c.x + c.y*c.y + c.z*c.z);
}

inline double abs(const Coord &c)
{
	return sqrt(norm(c));
}




#endif
