//
//  The Worldforge Project
//  Copyright (C) 1998,1999  The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//
// created by Karsten Laux 07/1999

#include "dialog.h"
#include "buttons.h"
#include "operation.h"
#include "viewport.h"
#include "gui.h"
#include <varconf/Config.h>

#include <uta/resources.h>
using namespace uta;

#include "dii/object.h"///

bool Screen::lock_ = false;

Option::Option(Widget* parent, int x, int y, std::string sopt, std::string desc, std::string path) :
  Widget(parent, x, y, 255, 25),
  optName_(sopt),
  path_(path)
{
  
  ToggleButton *opt =
    new ToggleButton(this, 0,0,25,25);
  opt->setUpPic(SURFACE_RES("button25"));
  opt->setDownPic(SURFACE_RES("button25_pressed"));
      
  Label *opttext = 
    new Label(this, 30,0,225,25,FONT_RES("button_font"));
  opttext->setAlign(Label::ALIGN_LEFT);
  opttext->setText(desc.c_str());

  if(varconf::Config::inst()->getItem(path_,optName_))
    opt->toggle();

  opt->pressed.connect(slot(this, &Option::pressed));
  opt->released.connect(slot(this, &Option::released));
    
};

Option::~Option()
{
}

void
Option::pressed()
{
  //  writeSettings(path_+"/"+optName_,1); 
  varconf::Config::inst()->setItem(path_,optName_,true);
  valueChanged.emit(1);
}

void
Option::released()
{
  //  writeSettings(path_+"/"+optName_,0); 
  varconf::Config::inst()->setItem(path_,optName_,false);
  valueChanged.emit(0);
}


Parameter::Parameter(Widget* parent, int x, int y, 
		     std::string opt, std::string desc, std::string def, std::string path) :
  Widget(parent, x, y, 250, 25),
  optName_(opt),
  path_(path)  
{  
  
  Label *opttext = 
    new Label(this, 0,0,125,25,FONT_RES("button_font"));
  opttext->setText(desc.c_str());
  opttext->setAlign(Label::ALIGN_LEFT);

  line_ = new LineEdit(this, 130,0,150,25,FONT_RES("text_font"));
  line_->setColor(lightgrey);
 
//   line_->setText(readSettings(path_+"/"+optName_).asString().c_str());
  std::string content = varconf::Config::inst()->getItem(path_,optName_);
  line_->setText(content.c_str());

  line_->textChanged.connect(slot(this, &Parameter::changed));

  if(std::string(line_->getText()).empty())
    {
      line_->setText(def.c_str());
    }
}

Parameter::~Parameter()
{
}

void
Parameter::changed(const char* data)
{
  if(!data)
    return;

  varconf::Config::inst()->setItem(path_,optName_, std::string(data));
  valueChanged.emit(data);
}


Screen::Screen():
  restore_(false),
  widget_(NULL),
  running_(false),
  result_(CANCEL)
{
}
  

Screen::~Screen()
{
  
}

int
Screen::exec()
{
 restore_ = Viewport::instance()->hideViewport();

  DII::Object out;
  DII::Object::MapType in, screenarea_;
  in["NAME"]=std::string("screenarea");
  in["section"]=0;

  if(lock_)
    return CANCEL;

  lock_ = true;
 
  out = Gui::instance()->request_screenarea(in);
  if(!doesContain(out,"section"))
    return CANCEL;

  //discard old input and use old output as new input
  in = out.AsMap();
   
  //actually bind the screenarea 
  out = Gui::instance()->bind_screenarea(in);
 
  if(!doesContain(out,"parent_widget"))
    return CANCEL;
 
  int x=out.AsMap()["x"].AsInt();
  int y=out.AsMap()["y"].AsInt();
  int w=out.AsMap()["width"].AsInt();
  int h=out.AsMap()["height"].AsInt();
   
  Widget* rootwin = objToPtr<Widget*>(out.AsMap()["parent_widget"]);
 
  assert(rootwin);
   
  screenarea_ = out.AsMap();
  
  screenarea_["NAME"]=std::string("screenarea");
 
  widget_ = create(rootwin,x,y,w,h);

  result_ = CANCEL;
 
  if(widget_)
    {
      running_ = true;
      widget_->setBackground(SURFACE_RES("background"));
    }
 
  while(running_ && uta::Application::Instance->keepAlive())
    {
      idleFunction();
    }
 
  cleanup();

  if(widget_)
    delete widget_;

  //release screenarea again:
  Gui::instance()->release_screenarea(screenarea_);

  lock_ = false;

  if(restore_)
    Viewport::instance()->showViewport();

  return result_;
}

void
Screen::accept()
{
  
  running_ = false;
  result_ = OK;
}

void
Screen::reject()
{
  
  running_ = false;
  result_ = CANCEL;
}
