//
//  The Worldforge Project
//  Copyright (C) 1998,1999  The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//
// created by Karsten Laux 07/1999

#ifndef _UCLIENT_DIALOG_H
#define _UCLIENT_DIALOG_H

#include <uta/dialog.h>
#include <uta/pushbutton.h>
#include <uta/togglebutton.h>
#include <uta/label.h>
#include <uta/lineedit.h>
#include <uta/application.h>
#include <uta/listbox.h>

///
class Option : public uta::Widget
{
public:
  ///
  void pressed();
  ///
  void released();
  ///
  std::string optName_;
  ///
  std::string path_;
  ///
  Option(uta::Widget* parent, int x, int y, std::string opt, std::string desc, std::string path="/options");
  ///
  virtual ~Option();
  ///
  SigC::Signal1<void, int> valueChanged;
  
};



///
class Parameter : public uta::Widget
{

public:
  ///
  std::string optName_;
  ///
  std::string path_;
  ///
  uta::LineEdit *line_;
  ///
  void changed (const char*);
  ///
  Parameter(uta::Widget* parent, int x, int y, 
	    std::string opt, std::string desc, std::string def ="", std::string path = "/options");
  ///
  virtual ~Parameter();

  ///
  SigC::Signal1<void, const std::string&> valueChanged;
 
};

///
class Screen : virtual public SigC::Object
{
 public:
  ///return values of exec()
  enum { OK, CANCEL };
  ///
  Screen();
  ///
  virtual ~Screen();
  ///
  int exec();

  ///
  void accept();
  ///
  void reject();

 protected:

  /**create actual widget.
     derived classes need to implement this method !
     parameters are the parent widget and allocated childspace
  */
  virtual uta::Widget* create(uta::Widget*, int,int,int,int) = 0;
  /**cleanup.
     This is called after the screen is hidding, but before
     the widgets get deleted; 
     use this to readout the widget contents !
     You must not do this in the derived class's destructor.
  */
  virtual void cleanup() {};
  /**idle function.
     descendent may override this in order to do sth. while 
     waiting for user input (e.g. update dialog content)
     ... actually this is called instead of application main loop.
  */
  virtual void idleFunction() {};
  ///
  bool restore_;
/*   /// */
/*   DII::Object::MapType screenarea_; */
  ///
  uta::Widget* widget_;
  ///
  bool running_;
  ///
  int result_;

  static bool lock_; // Can't run more than one of these at once
};

#endif
