#ifndef _TOOLBOX_MISC_H_
#define _TOOLBOX_MISC_H_

#include <uta/surface.h>
#include <uta/painter.h>

//blit src to dst centering the src surface inside dst if
//crop dst if needed
inline bool centerBlit(const uta::Surface* src, uta::Surface* dst)
{
  if(!src || !dst)
    return false;
  if(src->empty() || dst->empty())
    return false;
  
  int dw = (dst->width() - src->width())/2;
  int dh = (dst->height() - src->height())/2;
  
  int w = src->width();
  int h = src->height();
    
	uta::Rect dstR;
  float scale;

  if(dw >= 0 && dh >= 0)
    {
      //src fits into dst
      dstR = uta::Rect(dw,dh,w,h);
      src->blit(dst, dstR);
    }
  else
    {
      float scale_w = (float)dst->width()/(float)src->width();
      float scale_h = (float)dst->height()/(float)src->height();
      
      scale = (scale_w < scale_h) ? scale_w : scale_h;

      //cerr << "w="<<w<<"  h="<<h<<endl;
      w = (int)(w*scale);
      h = (int)(h*scale);
      
      //cerr << "new dimensions: w="<<w<<"  h="<<h<<endl;
      dw = (dst->width()-w)/2;
      dh = (dst->height()-h)/2;

/*       int cw = (dst->width() > (dw+w)) ? 0 : dw+w-dst->width()-1; */
/*       int ch = (dst->height() > (dh+h)) ? 0 : dh+h-dst->height()-1; */
      int cw = 0;
      int ch = 0;

      dstR=uta::Rect(dw,dh,w+cw,h+ch);
      //   cerr << "centerBlit: blitting to dest "<< dstR << endl;

/*       using tmp as a intermediate surface and omitting the last scanline */
/* 	 hides "bugs" in scaledBlit  */
      
      uta::Surface* tmp = new uta::Surface(dst->width(), dst->height(), uta::Pixelformat::BGR888);
/*       tmp->setTransColor(black); */
/*       tmp->setTransparency(true); */
/*       tmp->clear(); */

      src->scaledBlit(tmp,dstR);

      uta::Painter *painter = new uta::Painter((uta::Surface*)src);
      tmp->setTransColor(painter->getPixel(uta::Point(0,0)));
      delete painter;
      tmp->setTransparency(true);

      dstR = uta::Rect(dstR.upperLeft(),dstR.lowerRight()+uta::Point(0,-1));
      tmp->blit(dst,dstR,dstR);

      delete tmp;

      // src->scaledBlit(dst,dstR);
    }
  return true;
}

#endif
