//
//  The WorldForge Project
//  Copyright (c) 1999  The WorldForge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the WorldForge Web Site at http://www.worldforge.org.
//
//
#include "my_time.h"

//define this to use SDL_GetTicks(), this requires to link and initialize
//SDL, but works on many many systems (sun, beos, freebsd, win, linux, macos)
//
//if you are compiling for win32 or linux, leave it undef'ed, so that this
//modules will run standalone
#undef USE_SDL

#if !defined(USE_SDL)
 #if !defined(__WIN32__) && !defined(_WIN32)
  #include <sys/time.h>
  #include <unistd.h>
 #else
  #include <windows.h>
  #include <mmsystem.h>
 #endif
#else
 #include <SDL/SDL.h>
#endif



my_time::my_time()
{
  set_now();
}


my_time::my_time(int t)
{
  mseconds = (long int)t;
}

void my_time::set_now()
{
#if defined(USE_SDL)
  mseconds = SDL_GetTicks();
#else
 #if !defined(__WIN32__) && !defined(_WIN32)
  struct timeval now;
  gettimeofday(&now, NULL);
  mseconds=now.tv_sec*1000+now.tv_usec/1000;
 #else
  //this is easy, but warps around every 49,7
  mseconds=GetTickCount();
 #endif
#endif

}

bool my_time::operator>(const my_time &t)
{
  return mseconds > t.mseconds;
}

bool my_time::operator<(const my_time &t)
{
  return mseconds < t.mseconds;
}

my_time &my_time::operator=(const my_time &t)
{
  mseconds = t.mseconds;
  return *this; 
}

std::ostream &operator<<(std::ostream &out, const my_time &t)
{
  return (out << t.mseconds/1000.0); 
}

//some friends
my_time operator-(const my_time &t1,const my_time &t2)
{
  my_time result;
  result.mseconds = t1.mseconds - t2.mseconds;
  return result;
}

my_time operator+(const my_time &t1,const my_time &t2)
{
  my_time result;
  result.mseconds = t1.mseconds + t2.mseconds;
  return result;
}
