//  The Worldforge Project
//  Copyright (C) 1998, 1999, 2000 The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//
// created by Karsten Laux 03/2000

#include "operation.h"

std::string typeOfOperation(const Atlas::Message::Object& obj) 
     throw (NoOperationException)
{
  Atlas::Message::Object::MapType::const_iterator itr;
  itr = obj.AsMap().find("objtype");
  if("op" != itr->second.AsString())
      throw NoOperationException();
  itr = obj.AsMap().find("parents");
  return itr->second.AsList().begin()->AsString();
}

uta::Point objToPoint(const Atlas::Message::Object& obj)
  throw(NoPointException)
{
  if(!obj.IsList())
    throw NoPointException();
  if(obj.AsList().size() != 2)
    throw NoPointException();

  uta::Point p;
  Atlas::Message::Object::ListType::const_iterator itr = obj.AsList().begin();
  p.x = itr->AsInt();
  itr++;
  p.y = itr->AsInt();
  
  return p;
}

Atlas::Message::Object pointToObj(const uta::Point& p)
{
  Atlas::Message::Object::ListType l;
  l.push_back(p.x);
  l.push_back(p.y);
    
  return l;
}

uta::Rect objToRect(const Atlas::Message::Object& obj)
  throw(NoRectException)
{
  if(!obj.IsList())
    throw NoRectException();
  if(obj.AsList().size() != 2)
    throw NoRectException();

  uta::Point ul,lr;
  Atlas::Message::Object::ListType::const_iterator itr = obj.AsList().begin();
  ul.x = itr->AsInt();
  itr++;
  ul.y = itr->AsInt();
  itr++;
  lr.x = itr->AsInt();
  itr++;
  lr.y = itr->AsInt(); 

  return uta::Rect(ul,lr);
}

Atlas::Message::Object rectToObj(const uta::Rect& r)
{
  Atlas::Message::Object::ListType l;
  l.push_back(r.upperLeft().x);
  l.push_back(r.upperLeft().y);
  l.push_back(r.lowerRight().x);
  l.push_back(r.lowerRight().y);

  return l;
}

Coord objToCoord(const Atlas::Message::Object& obj)
     throw(NoCoordException)
{
  if(!obj.IsList())
    throw NoCoordException();
  if(obj.AsList().size() != 3)
    throw NoCoordException();

  Coord c;
  Atlas::Message::Object::ListType::const_iterator itr = obj.AsList().begin();

  if(itr->IsFloat())
    c.x = itr->AsFloat();
  else if(itr->IsInt())
    c.x = itr->AsInt();
  else 
    throw NoCoordException();

  itr++;

  if(itr->IsFloat())
    c.y = itr->AsFloat();
  else if(itr->IsInt())
    c.y = itr->AsInt();
  else 
    throw NoCoordException();

  itr++;
  
  if(itr->IsFloat())
    c.z = itr->AsFloat();
  else if(itr->IsInt())
    c.z = itr->AsInt();
  else 
    throw NoCoordException();

  return c;
}

Atlas::Message::Object coordToObj(const Coord& c)
{
  Atlas::Message::Object::ListType l;
  l.push_back(c.x);
  l.push_back(c.y);
  l.push_back(c.z);
    
  return l;
}

Quaternion objToQuaternion(const Atlas::Message::Object& obj)
     throw(NoQuaternionException)
{
  if(!obj.IsList())
    throw NoQuaternionException();
  if(obj.AsList().size() != 4)
    throw NoQuaternionException();

  Quaternion q;
  Atlas::Message::Object::ListType::const_iterator itr = obj.AsList().begin();

  if(itr->IsFloat())
    q[0] = itr->AsFloat();
  else if(itr->IsInt())
    q[0] = itr->AsInt();
  else 
    throw NoQuaternionException();

  itr++;

  if(itr->IsFloat())
    q[1] = itr->AsFloat();
  else if(itr->IsInt())
    q[1] = itr->AsInt();
  else 
    throw NoQuaternionException();

  itr++;
  
  if(itr->IsFloat())
    q[2] = itr->AsFloat();
  else if(itr->IsInt())
    q[2] = itr->AsInt();
  else 
    throw NoQuaternionException();

  itr++;
  
  if(itr->IsFloat())
    q[3] = itr->AsFloat();
  else if(itr->IsInt())
    q[3] = itr->AsInt();
  else 
    throw NoQuaternionException();

  return q;
}

Atlas::Message::Object quaternionToObj(const Quaternion& q)
{
  Atlas::Message::Object::ListType quat;
  quat.push_back(q[0]);
  quat.push_back(q[1]);
  quat.push_back(q[2]);
  quat.push_back(q[3]);

  return Atlas::Message::Object(quat);
}


bool doesContain(const Atlas::Message::Object& obj, const std::string& id)
{
  return obj.AsMap().find(id) != obj.AsMap().end();
}


