//  The Worldforge Project
//  Copyright (C) 1998, 1999, 2000 The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//
// created by Karsten Laux 08/2000

#ifndef _UCLIENT_OPERATION_H_
#define _UCLIENT_OPERATION_H_

#include <Atlas/Objects/Operation/RootOperation.h>
#include "ftos.h"
#include <uta/rect.h>
#include "coord.h"
#include "Quaternion.h"
#include <typeinfo>
#include <cstdlib>
#include <stdexcept>

typedef Atlas::Objects::Operation::RootOperation Operation;

class NoOperationException : public std::domain_error {
public:
  NoOperationException() : 
    domain_error("cannot cast to type Operation") {};
};

class NoCoordException : public std::domain_error {
public:
  NoCoordException() : 
    domain_error("cannot cast to type Coord") {};
};

class NoCastException  : public std::domain_error {
public:
  NoCastException() :
    domain_error("cannot pointer cast to requested type") {};
};

class NoPointException  : public std::domain_error {
public:
  NoPointException() :
    domain_error("cannot cast to tyoe uta::Point") {};
};

class NoRectException  : public std::domain_error {
public:
  NoRectException() :
    domain_error("cannot cast to type uta::Rect") {};
};

class NoQuaternionException  : public std::domain_error {
public:
  NoQuaternionException() :
    domain_error("cannot cast to type Quaternion") {};
};

///
std::string typeOfOperation(const Atlas::Message::Object& obj) throw (NoOperationException);

///
uta::Point objToPoint(const Atlas::Message::Object& obj) throw(NoPointException);
///
Atlas::Message::Object pointToObj(const uta::Point&);

///
uta::Rect objToRect(const Atlas::Message::Object& obj) throw(NoRectException);
///
Atlas::Message::Object rectToObj(const uta::Rect&);

///
Coord objToCoord(const Atlas::Message::Object& obj) throw(NoCoordException);
///
Atlas::Message::Object coordToObj(const Coord& c);

///
Quaternion objToQuaternion(const Atlas::Message::Object& obj) throw(NoQuaternionException);
///
Atlas::Message::Object quaternionToObj(const Quaternion& q);

///
bool doesContain(const Atlas::Message::Object& obj, const std::string& id);

///
template<class T> 
T objToPtr(const Atlas::Message::Object& obj) throw(NoCastException)
{
#ifdef DEBUG
  Atlas::Message::Object::MapType::const_iterator itr1, itr2;
  itr1 = obj.AsMap().find("type");
  itr2 = obj.AsMap().find("pointer");
  if(itr1 != obj.AsMap().end() && itr2 != obj.AsMap().end())
    {
      if(itr1->second.IsString() && itr2->second.IsString())
	{
	  if(strcmp(typeid(T).name(), itr1->second.AsString().c_str()) == 0)
	    return reinterpret_cast<T>((void*)strtoul(itr2->second.AsString().c_str(),NULL,10) );
	  cerr << "objToPtr<>: cannot cast " << typeid(T).name() 
	       << " to " << itr1->second.AsString() << endl;
	}
      else
	cerr << "objToPtr<>: this is no Pointer object ! " << endl;
    }

  throw NoCastException();
  return NULL;
#else
  return reinterpret_cast<T>((void*)strtoul(obj.AsString().c_str(),NULL,10) );
#endif
}

///
template<class T>
Atlas::Message::Object ptrToObj(T ptr)
{
#ifdef DEBUG
  Atlas::Message::Object::MapType obj;
  obj["type"] = std::string(typeid(T).name());
  obj["pointer"] = ultos((unsigned long)ptr);
  return obj;
#else
  return ultos((unsigned long)ptr);
#endif
}

#endif

