//  The Worldforge Project
//  Copyright (C) 1998, 1999 The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//
// created by Karsten Laux 06/2000

#include "tree.h"
#include <assert.h>

Tree::LeafMap Tree::emptyMap_;

Tree::Tree() : root_(NULL), leaves_(NULL)
{
}

Tree::Tree(Tree* root) : root_(root), leaves_(NULL)
{
}

Tree::~Tree() 
{ 
  if(leaves_) 
    {
      LeafMap::iterator itr = leaves_->begin();
      while(itr != leaves_->end())
	{
	  delete itr->second;
	  itr++;
	}
      delete leaves_; 
    }
};

Tree* 
Tree::iterator::operator*()
{
  return current_->second;
}
      
Tree::iterator& 
Tree::iterator::operator++()
{
  assert(tree_);

  if(current_ != tree_->leaves_->end())
    {
      if(current_->second->size() > 0)
	{
	  visited_.push(current_);
	  trees_.push(tree_);
	  tree_ = current_->second;
	  current_ = tree_->leaves_->begin();
	}
      else
	{
	  current_++;
	}
    }

  if(current_ == tree_->leaves_->end())
    {  
      //reached end of tree
      //something left on the stack ?
      do
	{
	  if(!visited_.empty())
	    {
	      // pop last iterator from stack
	      current_ = visited_.top();
	      visited_.pop();
	      tree_ = trees_.top();
	      trees_.pop();
	      current_++;
	    }
	} while( (current_ == tree_->leaves_->end()) && (!visited_.empty()));
      
    }
  
  return *this;
}

Tree::iterator& 
Tree::iterator::operator++(int)
{
  return ++(*this);
}

bool 
Tree::iterator::operator==(const Tree::iterator& a)
{
  return (tree_ == a.tree_)&&(current_ == a.current_);
}  

bool 
Tree::iterator::operator!=(const Tree::iterator& a)
{
  return (tree_ != a.tree_)||(current_ != a.current_);
}  


Tree*
Tree::find(const std::string& id)
{
  if(!leaves_)
    return NULL;

  LeafMap::iterator itr = leaves_->find(id);
  if(itr != leaves_->end())
    return itr->second;
  
  itr = leaves_->begin();
  Tree* result = NULL;
  while(itr != leaves_->end())
    {
      if((result = itr->second->find(id)) != NULL)
	break;
      itr++;
    }
  return result;
}


void 
Tree::insert(const std::string& id, Tree* leaf) 
{ 
  if(!leaves_)
    leaves_ = new LeafMap();

  LeafMap::iterator itr = leaves_->find(id);
  if(itr != leaves_->end())
    {
      //      if(itr->second)
      //	delete itr->second;
      leaves_->erase(itr);
    }
  
  (*leaves_)[id]=leaf; 
}
  
void 
Tree::remove(const std::string& id)
{ 
  if(!leaves_)
    return;

  LeafMap::iterator itr = leaves_->find(id);
  if(itr != leaves_->end())
    leaves_->erase(itr);
}


Tree* 
Tree::local_find(const std::string& id)
{
  if(!leaves_)
    return NULL;

  LeafMap::iterator itr = leaves_->find(id);
  if(itr != leaves_->end())
    return itr->second;

  return NULL;
}
