//
//  The Worldforge Project
//  Copyright (C) 1998-2002  The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//
//  Created by Malcolm Walker 03/15/2002

#include <list>
#include <uta/resources.h>
#include "widgets.h"

using namespace uta;

FixLineEdit::FixLineEdit(Widget* parent, int x, int y) :
  LineEdit(parent, x,y, 250, 25)
{
  setColor(Color(0,0,0,32));
  setBackground(SURFACE_RES("lineedit_background"));
}

FixComboEdit::FixComboEdit(Widget* parent, int x, int y) :
  LineEdit(parent, x, y, 225, 25)
{
	_parent = parent;
	setColor(Color(0,0,0,32));
	setBackground(SURFACE_RES("lineedit_background"));
	
	_listVisible = false;
	
	_dropButton = new ToggleButton(parent, x + 225, y, 25, 25);
	_dropButton->setColor(Color(0,0,0,32));
	_dropButton->setUpPic(SURFACE_RES("arrow_down"));
	_dropButton->setDownPic(SURFACE_RES("arrow_down_pressed"));

	_dropButton->pressed.connect(slot(this, &FixComboEdit::dropDown));
	_dropButton->released.connect(slot(this, &FixComboEdit::rollUp));

//  _list = new ListBox(parent, x, y + 25, 250, 100);
  _list = new ListBox(this, 0, 25, 250, 100);
//	_list->setColor(Color(0,0,0,48));
	_list->setBackground(SURFACE_RES("lineedit_background"));
	_list->selectionChanged.connect(slot(this, &FixComboEdit::updateText));
//	_list->disable();
//	_list->hide();
	// TODO: find some way of rolling up listbox when same item is 
	// clicked
//  _list->clicked.connect(slot(this, &FixComboEdit::rollUp));	

	forceUpdate(1);
	_parent->forceUpdate(1);
	rollUp();  // Calling hide() directly on the list widget causes
	           // some VERY strange behaviour in libuta
}

void FixComboEdit::dropDown() 
{
  _listVisible = true;
	_list->enable();
	_list->show();
}

void FixComboEdit::rollUp()
{
	// TODO: examine these calls and remove any
	// redundant ones
	_listVisible = false;
	_list->disable();
	_list->hide();
	_parent->forceUpdate(1);
}

void FixComboEdit::setList(std::vector<std::string>& newlist) 
{
	_list->clear();
	_list->addElements(newlist);
}

void FixComboEdit::updateText(const char* t) 
{
	setText(t);
//	rollUp();
	_dropButton->release();
}
