//
//  The Worldforge Project
//  Copyright (C) 1998,1999  The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//
// created by Karsten Laux 12/2000


#include <uta/debug.h>
#include <uta/resources.h>
#include <uta/version.h>

using namespace uta;
#include "app.h"

#include "../modules/console.h"
#include "../modules/appmod.h"

static int shotCount = 0;

UClientApp::UClientApp(int argc, char** argv):
  Application(argc, argv)
{
}

UClientApp::~UClientApp()
{
}

bool 
UClientApp::handleEvent(const Event* event)
{

  bool handled = Application::handleEvent(event);

  if(handled)
    return true;

  if(event->type() == Event::KEY)
    {
      if(((KeyEvent *)event)->state() == KeyEvent::PRESSED)
	{
	  
	  unsigned int key = ((KeyEvent *)event)->key() ;
	  unsigned int mod = ((KeyEvent *)event)->modifier();
	  
	  if(key == SDLK_PRINT)
	    {
	      int xsize = rootWindow()->width();
	      int ysize = rootWindow()->height();

	      Surface* screenshot = new Surface(xsize,
						ysize, 
						Pixelformat::RGB888);
	      assert(screenshot);

	      char filename[128];
	      sprintf(filename,"screenshot_%0d.png", shotCount);
	      shotCount++;

	      rootWindow()->screen()->blit(screenshot, 
					   Rect(0,0,xsize,ysize),
					   Rect(0,0,xsize,ysize));

	      std::string info = 
		"screenshot gets written to :"+std::string(filename)+" ...";
	      if(Console::instance())
		Console::instance()->std_out(info);

	      screenshot->convert(Pixelformat::RGBA8888);
	      info = "...";
	      if(Console::instance())
		Console::instance()->std_out(info);

	      //takes some time ...
	      screenshot->writeToFile(filename);

	      delete screenshot;
	      info = "...OK.";
	      if(Console::instance())
		Console::instance()->std_out(info);
	    } 	  
	  else if(key == SDLK_RETURN && (mod & KEY_MOD_ALT) )
	    {
	      //ALT-RETURN .. toggle fullscreen mode on / off
	      SDL_WM_ToggleFullScreen(SDL_GetVideoSurface());
	    }
	}
      }      
  else if(event->type() == Event::QUIT)
    {
      debugN(3,std::cerr << "App: received a Quit Event." << std::endl;);
      App::instance()->quit();
#ifdef __WIN32__
      mouse()->setPointer(NULL);
#endif
    }
  else if (event->type() == Event::ICONIFIED)
    {
      debugN(3,std::cerr << "UClientApp has been iconified." << std::endl;);
    }
  else if (event->type() == Event::RESTORED)
    {
      debugN(3,std::cerr << "UClientApp has been restored." << std::endl;);
#ifdef __WIN32__
      // restore the mouse pointer : setting to
      // null forces the use of the default pointer
      mouse()->setPointer(NULL);
#endif
    }
   else if(event->type() == Event::LOSTFOCUS)
    {
      debugN(3,std::cerr << "UClientApp has lost focus." << std::endl;);
    }
  else if(event->type() == Event::GOTFOCUS)
    {
      debugN(3,std::cerr << "UClientApp has Regained focus." << std::endl;);
#ifdef __WIN32__
      mouse()->setPointer(NULL);
#endif
    }    

  return handled;
}

int 
UClientApp::startupProc() 
{  
  return 0;
}

int 
UClientApp::eventloopProc()
{
  //this is called very often (up to 50times per second)
  //so only really fast things should happen here
  
  
  return 0;

}

int 
UClientApp::shutdownProc()
{
  return 0;
}
