//
//  The Worldforge Project
//  Copyright (C) 1998,1999  The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//
// created by Karsten Laux 01/2000

#define NEED_DII_SERVICES

#include <strstream>

#include "dispatcher.h"
#include "config.h"
#include "loader/loader.h"

using namespace DII;

namespace UClient {

Dispatcher dispatcher;

  Dispatcher::Dispatcher() :
    Module("Dispatcher")
{
#if !defined(__WIN32__)
  Loader::setSearchPath(std::string(INSTALLDIR)+std::string("/lib"));
  Loader::setScriptPath(std::string(INSTALLDIR)+std::string("/share/uclient/scripts"));
#else
  Loader::setSearchPath("");
  Loader::setScriptPath(std::string("./share/uclient/scripts"));
#endif


  REGISTER_METHOD(Dispatcher, mod_load);
  REGISTER_METHOD(Dispatcher, script_load);
  REGISTER_METHOD(Dispatcher, mod_unload);
  REGISTER_METHOD(Dispatcher, mod_reload);
  REGISTER_METHOD(Dispatcher, help);
 REGISTER_METHOD(Dispatcher, commands);

  REGISTER_METHOD(Dispatcher, queryModules);
  REGISTER_METHOD(Dispatcher, queryMethods);
  
  REGISTER_AS_SERVICE(mod_load);
  REGISTER_AS_SERVICE(script_load);
  REGISTER_AS_SERVICE(mod_unload);
  REGISTER_AS_SERVICE(mod_reload);

  REGISTER_AS_SERVICE(help);
  REGISTER_AS_SERVICE(commands);
  REGISTER_AS_SERVICE(queryModules);
  REGISTER_AS_SERVICE(queryMethods);

}


Dispatcher::~Dispatcher()
{
  unloadAll();
}

void
Dispatcher::unloadAll()
{

  DII::Loader::releaseAll();

}

DII::Object
Dispatcher::help(const DII::Object&)
{
    DII::callService("browse",DII::Object::ListType(1,"uclient/help/help.html"));
    return DII::SUCCESS;
}

DII::Object
Dispatcher::commands(const DII::Object&)
{
  std::string result;
  StringSList services = DII::queryServices();
  result = 
    "Available commands :\n"
    "-----------------------\n";

  StringSList::iterator itr = services.begin();
  while(itr != services.end())
    result += *itr++ + std::string("  ");
  
  result+=std::string("\n\nPress Enter to activate the command line.\n"
		 "Every input is send as talk to all other players.\n"
		 "Commands are lead in with a slash, e.g. /help.\n"
		 "Clicking on the triangles above and below the textwindow will resize it.\n"
		 "The text can be scrolled up and down by the small button right of\n"
		 "the window.\n");
  
  return result;
}

DII::Object
Dispatcher::mod_load(const DII::Object& param)
{
  DII::Object result = DII::FAILURE;
  
  if(!param.IsList() || param.AsList().size()<1)
    return DII::FAILURE;

  std::string modname = param.AsList().begin()->AsString();
  
  if(!modname.empty())
    {
      if(Loader::loadModule(modname))
	result = std::string("module ")+modname+std::string(" successfully loaded.\n");
      else
	result = std::string("failed to load module")+modname;
    }
  
  return result;
}

DII::Object
Dispatcher::script_load(const DII::Object& param)
{
  DII::Object result = DII::FAILURE;
  
  if(!param.IsList() || param.AsList().size()<1)
    return DII::FAILURE;

  std::string scriptname = param.AsList().begin()->AsString();
  
  if(!scriptname.empty())
    {
      if(Loader::loadScript(scriptname))
	result = std::string("script ")+scriptname+std::string(" successfully loaded.\n");
      else
	result = std::string("failed to load script")+scriptname;
    }
  
  return result;
}
 
DII::Object
Dispatcher::mod_unload(const DII::Object& param)
{
   DII::Object result = DII::FAILURE;
  
  if(!param.IsList() || param.AsList().size()<1)
    return DII::FAILURE;

  std::string modname = param.AsList().begin()->AsString();
    
  if(!modname.empty())
    {
      if(Loader::releaseModule(modname)) 
	result = std::string("module ")+modname+" successfully removed.\n";
      else
	result = std::string("failed to remove module ")+modname;
    }

  return result;
}


DII::Object
Dispatcher::mod_reload(const DII::Object& param)
{
  mod_unload(param);
  mod_load(param);
  return 0;
}

DII::Object
Dispatcher::queryModules(const DII::Object&)
{
  DII::Object::ListType List;
  std::strstream str;
  
  Loader::dump(str);
  
  std::list<std::string> names;
  Loader::getModuleList(names);

  std::list<std::string>::iterator itr = names.begin();
  while(itr != names.end())
    {
      List.push_back(*itr);
      //      liststr += "  " + *itr + "\n";
      itr++;
    }  
  
  std::string liststr = std::string(str.str());
 
  DII::Object::MapType result;
  result["message"]=liststr;
  result["list"]=List;

	delete [] str.str();

  return result;
}

DII::Object 
Dispatcher::queryMethods(const DII::Object& param)
{
 
  DII::Object::MapType result;
  DII::Object::ListType List;
  std::string liststr;
  std::string modname;

  if(param.IsString())
    {
      modname = param.AsString();
    }
  else
    {
      if(!param.IsList() || param.AsList().size()<1)
	return DII::FAILURE;
      modname = param.AsList().begin()->AsString();
    }
  
  if(modname.empty())
    return DII::FAILURE;

  DII::Module* mod = DII::lookupModule(modname.c_str());
  if(mod)
    {
      DII::Module::Methods::const_iterator itr = mod->methods().begin();
      while(itr != mod->methods().end())
	{
	  liststr += std::string(itr->first)+std::string("\n");
	  List.push_back(std::string(itr->first));
	  itr++;
	}
    }
  else
    liststr = "unknown module.";
  
  result["message"] = liststr;
  result["list"] = List;
	
  return result;
}



}
