//
//  The Worldforge Project
//  Copyright (C) 1998,1999  The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//
// created by Karsten Laux 06/1999

#include <uta/debug.h>

#include "res_loader.h"
#include <uta/widget.h>
#include <uta/resources.h>
#include <uta/application.h>
using namespace uta;

#include "dii/object.h"
#include "dii/serialize.h"

#include "toolbox/operation.h"

#include "registry/registry.h"
#include "media/content_request.h"

#include <fstream>

// stat is used to check if a file really exists - uclient will
// spit out lots of errors now if there is no file
#include <sys/stat.h>
#include <errno.h>

int resourceLoader(const char* resfilename)
{
  // We expect the resources to live in the forge subdirectory of
  // the install dir -- this is not $PREFIX but $PREFIX/share/ (by default)
    
  std::string prefix; 
  try
    {
      std::string install_dir = Registry::instance()->readInstallDir();
      prefix = install_dir + std::string("/forge");
    }
  catch(...)
    {
      std::cerr << "resourceLoader: failed to read registry." << std::endl;
      return 1;
    }

  int exitcode = 0;

  DII::Object resourceList;
  
  if(!DII::Serialize::read(resfilename, resourceList))
    {
      std::cerr << "Resource loader could not parse " << resfilename << "!" << std::endl;
    } 

  int y = (int)(uapp->rootWindow()->height()*0.875);
  int x = (int)(uapp->rootWindow()->width()*0.1);

  int barmax = uapp->rootWindow()->width()-2*x;
  int barwidth = 0;

  const Surface *progressbar = 0;

  // Preload the progressbar image
  try
    {
      const std::string res_name = "progressbar_resload";
      DII::Object::MapType& images = resourceList.AsMap()["images"].AsMap();
      const std::string file = prefix + images[res_name].AsString();
      REG_SURFACE_RES(res_name, file);
      progressbar = SURFACE_RES(res_name);
    }
  catch(...)
    {
      std::cerr << "Progressbar media not found!" << std::endl;
      assert(false);
    }

  Widget* bar = new Widget(uapp->rootWindow(), x, y, 5,25);
  bar->setColor(Color(16,16,16));
  bar->setBackground(progressbar);
  
  debugN(4,std::cerr << "loading resources from "<<prefix<<std::endl;);
  
  std::string s;

  std::string resourceFilename;
  struct stat resourceFilestats;

  unsigned n=0;
  unsigned nEntries=0;
  DII::Object::MapType::iterator itr;
  if(doesContain(resourceList,"images")) 
	{
    try {  
      debugN(4,std::cerr << endl << "loading images: " << std::endl;);
      
      //global eventloop not yet running... so we have to update
      //manually
      uapp->rootWindow()->sync();
  
      DII::Object::MapType& images = resourceList.AsMap()["images"].AsMap();
      n=0;
      nEntries = images.size();
      for (itr = images.begin(); itr != images.end(); itr++ ) 
        {
          debugN(4,std::cerr << "loading : " << itr->first << std::endl;);

          // Check if file exists...
          resourceFilename = prefix + itr->second.AsString();
          if (stat(resourceFilename.c_str(), &resourceFilestats) != 0)
            {
	      ContentRequest request(ContentRequest::SYNC);
	      request.create(itr->second.AsString());
	      request.process();
	      resourceFilename = ContentRequest::getCacheDir() + itr->second.AsString();
            }

          REG_SURFACE_RES(itr->first, resourceFilename);
          
          barwidth = (int)(barmax/4 * (0.0 + (n+1) / (nEntries)));
          bar->resize(barwidth,25);
          
          //global eventloop not yet running... so we have to update
          //manually
          uapp->rootWindow()->sync();
          n++;
        }
    }
    catch(...)
    {
      debugN(4, std::cerr << "- failed to load any surfaces." << std::endl;);
    }
  }

  if(doesContain(resourceList,"fonts"))
  {
    try
    {
      debugN(4,std::cerr << std::endl << "loading fonts: " << std::endl;);
            
      DII::Object::MapType& fonts = resourceList.AsMap()["fonts"].AsMap();
      n=0;
      nEntries = fonts.size();  
      for (itr = fonts.begin(); itr != fonts.end(); itr++ ) 
        {
          debugN(4,std::cerr << "loading : " << itr->first << std::endl;);
          resourceFilename = prefix + itr->second.AsString();
					std::string fontName = resourceFilename.substr(0,	resourceFilename.find(","));
          if (stat(fontName.c_str(), &resourceFilestats) != 0)
            {
	      ContentRequest request(ContentRequest::SYNC);
	      request.create(itr->second.AsString());
	      request.process();
	      resourceFilename = ContentRequest::getCacheDir() + itr->second.AsString();
            }

          REG_FONT_RES(itr->first, resourceFilename);
          
          barwidth = (int)(barmax/4 * (1.0 + (n+1) / (nEntries)));
          //          barwidth = barmax * (n+1) / (nEntries) - 1;
          bar->resize(barwidth,25);
          
          //global eventloop not yet running... so we have to update
          //manually
          uapp->rootWindow()->sync();
          n++;
        }
    }
    catch(...)
    {
      debugN(4, std::cerr << "- failed to load any fonts." << std::endl;);
    }
  }

  if(doesContain(resourceList,"samples"))
  {
    try
    {
      debugN(4,std::cerr << endl << "loading samples: " << std::endl;);
     
      DII::Object::MapType& samples = resourceList.AsMap()["samples"].AsMap();
      n=0;
      nEntries = samples.size();
      for (itr = samples.begin(); itr != samples.end(); itr++ ) 
        {
          debugN(4,std::cerr << "loading : " << itr->first << std::endl;);
          resourceFilename = prefix + itr->second.AsString();
          if (stat(resourceFilename.c_str(), &resourceFilestats) != 0)
            {
	      ContentRequest request(ContentRequest::SYNC);
	      request.create(itr->second.AsString());
	      request.process();
	      resourceFilename = ContentRequest::getCacheDir() + itr->second.AsString();
            }

          REG_SAMPLE_RES(itr->first, resourceFilename);
          
          barwidth = (int)(barmax/4 * (2.0 + (n+1) / (nEntries)));
          //          barwidth = barmax * (n+1) / (nEntries) - 1;
          bar->resize(barwidth,25);
          
          //global eventloop not yet running... so we have to update
          //manually
          uapp->rootWindow()->sync();
          n++;
        }
    }
    catch(...)
    {
      debugN(4, std::cerr << "- failed to load any samples." << std::endl;);
    }
  }

  if(doesContain(resourceList,"songs"))
  {
    try
    {
      debugN(4,std::cerr << endl << "loading songs: " << std::endl;);
                    
      DII::Object::MapType& songs = resourceList.AsMap()["songs"].AsMap();
      n=0;
      nEntries = songs.size();  
      for (itr = songs.begin(); itr != songs.end(); itr++ ) 
      {
        debugN(4,std::cerr << "loading : " << itr->first << std::endl;);
        resourceFilename = prefix + itr->second.AsString();
        if (stat(resourceFilename.c_str(), &resourceFilestats) != 0)
        {
	  ContentRequest request(ContentRequest::SYNC);
	  request.create(itr->second.AsString());
	  request.process();
	  resourceFilename = ContentRequest::getCacheDir() + itr->second.AsString();
        }

        REG_MUSIC_RES(itr->first, resourceFilename);
            

        barwidth = (int)(barmax/4 * (3.0 + (n+1) / (nEntries)));  
        //barwidth = barmax * (n+1) / (nEntries) - 1;
        bar->resize(barwidth,25);
              
        //global eventloop not yet running... so we have to update
        //manually
        uapp->rootWindow()->sync();
        n++;
      }
    }
    catch(...)
    {
      debugN(4, std::cerr << "- failed to load any songs." << std::endl;);
    }
  }

  // no progress bar since this takes hardly any time:
  if(doesContain(resourceList,"colors"))
  {
    try
    {
      debugN(4,std::cerr << endl << "loading colors: " << std::endl;);
        
      DII::Object::MapType& colors = resourceList.AsMap()["colors"].AsMap();
      n=0;
      nEntries = colors.size();
      for (itr = colors.begin(); itr != colors.end(); itr++ ) 
      {
        debugN(4,std::cerr << "loading : " << itr->first << std::endl;);
        resourceFilename = prefix + itr->second.AsString();
        if (stat(resourceFilename.c_str(), &resourceFilestats) != 0)
        {
	  ContentRequest request(ContentRequest::SYNC);
	  request.create(itr->second.AsString());
	  request.process();
	  resourceFilename = ContentRequest::getCacheDir() + itr->second.AsString();
        }


        REG_COLOR_RES(itr->first, resourceFilename);
             
        n++;
      }
    }
    catch(...)
    {
      debugN(4, std::cerr << "- failed to load any color definitions." << std::endl;);
    }
  }

  delete bar;

  return exitcode;
}
