#include "media/sprite_cache.h"
#include "dii/object.h"
#include "dii/serialize.h"
#include "operation.h"

#include "registry/registry.h"

#include <uta/debug.h>

 
#include <Atlas/Message/Object.h>
#include <Atlas/Message/Encoder.h>
#include <Atlas/Net/Stream.h>
#include <Atlas/Objects/Decoder.h>
#include <Atlas/Codecs/XML.h>
 
#include <string>
#include <fstream>
#include <map>

using Atlas::Message::Object;

class FileDecoder : public Atlas::Message::DecoderBase 
{
  std::ifstream m_file;
  Atlas::Codecs::XML m_codec;

  struct hasher
  {
    size_t operator () (const std::string& s) const
    {
      unsigned long __h = 0;
      std::string::const_iterator itr = s.begin();
      while(itr != s.end())
        __h = 5*__h + *itr++;
      
      return size_t(__h);
    }
  };

  typedef STRING_HASH_MAP(int) StringIntHashMap;

  StringIntHashMap known;
  StringIntHashMap::iterator kItr;

  int m_count;
  
  virtual void ObjectArrived(const Object & obj) 
  {
    if (!obj.IsMap()) 
      {
	cerr << "ERROR: Non map object in file" << endl << flush;
	return;
      }

    const Object::MapType & omap = obj.AsMap();
    Object::MapType::const_iterator I;    
    if (((I = omap.find("parents")) != omap.end()) &&
	(I->second.IsList())) 
      {
	const Object::ListType & constlist = I->second.AsList();
	if(constlist.size() > 0)
	  {
	    SpriteDesc desc;

	    //set defaults
	    desc.mode ="standing";
	    desc.sex = "";
	    desc.size = "";
	    desc.status = "";

	    desc.type = constlist.front().AsString();
	    kItr = known.find(desc.type);
	    if(kItr == known.end())
	      {
		known[desc.type] = 0;
		cout << "compiling " << desc.asString() << endl;
		SpriteID id = SpriteCache::inst()->request(desc);
		SpriteCache::inst()->lock(id);
		SpriteCache::inst()->release(id);
		SpriteCache::inst()->check();     // this will flush the loaded sprite cache
		m_count++;
	      }
	    else
	      {
		kItr->second ++;
	      }
	  }
      }  
    else 
      {            
	std::cerr << "WARNING: Object in file has no parent attribute. Ignored."
		  << endl << flush;
      }
  }
  
public:
  FileDecoder(const std::string & filename) :
    m_file(filename.c_str()), 
    m_codec((iostream&)m_file, this), m_count(0)
  {
    
    
  }
  void read() {
    while (!m_file.eof()) {
      m_codec.Poll();
    }
  }
  
  void report() {
    std::cout << m_count << " sprites compiled."
	      << endl << flush;
  }
};

int readAtlasXML(const std::string& filename)
{
  FileDecoder decoder(filename);
  decoder.read();
  decoder.report();
  return 0;
}


int readSpritesXML()
{
  std::string install_dir = Registry::instance()->readInstallDir();
  std::string filename = install_dir+"/uclient/sprites.xml";

  DII::Object obj;
    
  if(!DII::Serialize::read(filename.c_str(), obj))
    {
      debugN(8,cerr<<"SpriteFactory: Error parsing sprites.xml"<<endl;);
      return false;
    };
    
  DII::Object::MapType& header = obj.AsMap()["header"].AsMap();
  DII::Object::MapType& sprites = obj.AsMap()["sprites"].AsMap();

  std::string version = header["version"].AsString();
  if(version != std::string(VERSION))
    {
      cerr << "Warning: sprite definitions are for UClient "<<version
	   << ", but you are running Version " << VERSION << endl;
    }

  debugN(8,cerr << sprites.size() << " sprite definitions found !" << endl;);
  
  DII::Object::MapType::iterator itr;
  for(itr = sprites.begin(); itr != sprites.end(); itr++)
    {
      SpriteDesc desc;
      desc.type = itr->first;
      DII::Object::ListType modes = itr->second.AsMap()["modes"].AsList();
      DII::Object::ListType::iterator mItr;
      
      for(mItr = modes.begin(); mItr != modes.end(); mItr++)
	{
	  DII::Object::MapType mode = mItr->AsMap();
	  
	  //set defaults
	  desc.mode ="standing";
	  desc.sex = "";
	  desc.size = "";
	  desc.status = "";
	  
	  if(doesContain(mode,"mode"))
	    desc.mode = mode["mode"].AsString();
	  if(doesContain(mode,"sex"))
	    desc.sex = mode["sex"].AsString();	      
	  if(doesContain(mode,"size"))
	    desc.size = mode["size"].AsString();
	  if(doesContain(mode,"status"))
	    desc.status = mode["status"].AsString();
	  
	  cout << "compiling " << desc.asString() << endl;
	  SpriteID id = SpriteCache::inst()->request(desc);
	  SpriteCache::inst()->lock(id);
	  SpriteCache::inst()->release(id);
	  SpriteCache::inst()->check();     // this will flush the loaded sprite cache
	}
      
    }
  return 0;
}
