// The Worldforge Project
// Copyright (C) 1998, 1999 The Worldforge Project
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
// 
// For information about Worldforge and its authors, please contact
// the Worldforge Web Site at http://www.worldforge.org.
// 
// created by Karsten Laux 05/2000

#include "entity.h"
#include "dii/object.h"
#include "operation.h"

#include "../environment/environment.h"

#include <uta/debug.h>
#include <uta/resources.h>

using namespace uta;

#undef DEBUG

// how exact do we handle time intervals ? 
#define EPSILON_T 10

#define MAX_SPEED 5

using namespace World;
using namespace std;

namespace World {

    Entity::Entity(const Parameters & param):Tree(param.root),
     stamp_(0),
     gid_(-1),
     orientation_(0),
     needNewSprite_(true),
     visible_(true),
     unreal_(false),
     modeRestoreTime_(0),
     spriteRef_(NULL) {
        // read id
        try {
            // FIXME !! dangerouns !!
            id_ = param.createArg.AsMap().find("id")->second.AsString();
        } catch(...) {
            std::cerr << "Entity::Entity() cannot get my id ! " << std::endl;
        }

        bbox_ = Coord(0.05, 0.05, 0.05);
        bboxMedian_ = Coord(0.0, 0.0, 0.0);

//        face_ = Coord(1.0, 0.0, 0.0);
        velocity_ = Coord(0.0, 0.0, 0.0);

    }

    Entity::~Entity() {
    }

    void Entity::sound(const DII::Object & soundArg) {

    }

    bool Entity::sight(const DII::Object & _sightArg) {
        bool changed = false;
        // extract properties
        string name,
         desc,
         parent;

        if (!_sightArg.IsMap()) {
#ifdef DEBUG
            std::cerr << "Entity::sight(...) argument is not Map !" << std::endl;
#endif
            return false;
        }

        try {

#ifdef DEBUG
            std::cerr << "Entity: parsing sight argument : " << std::endl;
            DII::Object::MapType::const_iterator mItr =
                _sightArg.AsMap().begin();
            while (mItr != _sightArg.AsMap().end()) {
                cerr << "     " << mItr->first << ": ";
                if (mItr->second.IsString()) {
                    cerr << mItr->second.AsString() << endl;
                } else if (mItr->second.IsList()) {
                    try {
                        Coord coord = objToCoord(mItr->second);
                        cerr << "<Coord>:" << coord << endl;
                    }

                    catch(...) {
                        cerr << "<List>" << endl;
                    }
                } else {
                    if (mItr->second.IsMap())
                        cerr << "<MAP> " << endl;
                    else if (mItr->second.IsFloat())
                        cerr << mItr->second.AsFloat() << endl;
                    else if (mItr->second.IsInt())
                        cerr << mItr->second.AsInt() << endl;
                    else
                        cerr << "-- missing value --" << endl;

                    mItr++;
                }
            }
#endif
            const DII::Object::MapType & sightArg = _sightArg.AsMap();
            DII::Object::MapType::const_iterator itr;

            // store all attributes ! (a bit of a hack here ... need to
            // clean this up ... FIXME !!!)
            itr = sightArg.begin();
            while (itr != sightArg.end()) {
                if (itr->first != "name" &&
                    itr->first != "contains" &&
                    itr->first != "description" &&
                    itr->first != "pos" &&
                    itr->first != "loc" &&
                    itr->first != "id" &&
                    itr->first != "parents" &&
                    itr->first != "stamp" &&
                    itr->first != "bbox" && itr->first != "bmedian") {
                    attributes_[itr->first] = itr->second;
                }
                itr++;
            }

            itr = sightArg.find("stamp");
            if (itr != sightArg.end()) {
                stamp_ = itr->second.AsFloat();
            }

            itr = sightArg.find("name");
            if (itr != sightArg.end()) {
                name = itr->second.AsString();
                if (!name.empty() && name_ != name) {
                    name_ = name;
                    changed = true;
                }
            }

            itr = sightArg.find("description");
            if (itr != sightArg.end()) {
                desc = itr->second.AsString();
                if (!desc.empty() && description_ != desc) {
                    description_ = desc;
                    changed = true;
                }
            }

	
            itr = sightArg.find("parents");
            if (itr != sightArg.end()) {
                DII::Object::ListType parentList = itr->second.AsList();
                if (!parentList.empty()) {
                    parent = parentList.front().AsString();
                    if (parent_ != parent) {
                        parent_ = parent;
                        changed = true;

                        // FIXME !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! (we want
                        // to release tomorrow...)
                        // hi hi we are releasing again and it is still
                        // not fixed !!!
                        if (parent_ == "weather" || parent_ == "creator"
                            || parent_ == "wall")
                            visible_ = false;
                        else
                            visible_ = true;

                        if (parent_ == "lych" || parent_ == "skeleton")
                            unreal_ = true;
                        else
                            unreal_ = false;

                        // I guess that this is not needed anymore :
                        // if(parent_ == "sty")
                        // bbox_ = Coord(10.0,10.0,0);
                        // else if(parent_ == "tent")
                        // bbox_ = Coord(8.0,8.0,0.0);
                    }
                }
            }

							
            /*
             * old way of defining bounding box 
             */
            itr = sightArg.find("bounding_box");
            if (itr != sightArg.end()) {
#ifdef DEBUG
                cerr <<
                    "Warning: World::Entity got old attribute: \"bounding_box\" !"
                    << endl;
#endif
                bbox_ = objToCoord(itr->second) * 0.5;
                bboxMedian_ = bbox_;
                changed = true;
            }

            /*
             * changed definition of bounding box since Acorn 0.4: bbox is 
             * half the actual bounding box bmedian is the offset of the
             * bounding box from the entity's position 
             */
            itr = sightArg.find("bbox");
            if (itr != sightArg.end()) {
						    // Consider changing uclient internals to match cyphesis
						    if (itr->second.AsList().size() == 6) {
								    // This is actually two Coords.
								    // They match up to diagonally opposite corners of
								    // the bounding box.
                    DII::Object::ListType bblist = itr->second.AsList();
                    Coord lowerLeft  = Coord(bblist[0].AsNum(), bblist[1].AsNum(), bblist[2].AsNum());
                    Coord upperRight = Coord(bblist[3].AsNum(), bblist[4].AsNum(), bblist[5].AsNum());
										bbox_ = upperRight - lowerLeft;
										bboxMedian_ = lowerLeft + (0.5f * bbox_);
										changed = true;										
                } else {
									  // Default case: bbox is a single coord, reference is
									  // origin
                    bbox_ = objToCoord(itr->second);
										changed = true;
								}
            }
						itr = sightArg.find("bmedian");
            if (itr != sightArg.end()) {
                bboxMedian_ = objToCoord(itr->second);
                changed = true;
            }

            if (parent_ == "weather") {
                itr = sightArg.find("rain");
                if (itr != sightArg.end()) {
                    float rain_ = itr->second.AsFloat();

		    if(Environment::instance()) {
                      if (rain_ > 2.0)
                          Environment::instance()->weather(string("thunder"));
                      else if (rain_ > 0.0)
                          Environment::instance()->weather(string("rain"));
                      else
                          Environment::instance()->weather(string("sunny"));
		    }
                }

                itr = sightArg.find("snow");
                if (itr != sightArg.end()) {
                    float snow_ = itr->second.AsFloat();

		    if(Environment::instance()) {
                      if (snow_ > 0.0)
                          Environment::instance()->weather(string("snow"));
                      else
                          Environment::instance()->weather(string("sunny"));
		    }
                }
            }
            // try to extract position and velocity information
            // and keep track of changes
            changed |= move(_sightArg);

            // has already been called in move
            // requestSprite();
        }
        catch(...) {
#ifdef DEBUG
            cerr << "Entity: error in sight argument ! " << endl;

            DII::Object::MapType::const_iterator mItr =
                _sightArg.AsMap().begin();
            while (mItr != _sightArg.AsMap().end()) {
                cerr << mItr->first << ": ";

                if (mItr->second.IsString())
                    cerr << mItr->second.AsString() << endl;
                else if (mItr->second.IsList()) {
                    try {
                        Coord coord = objToCoord(mItr->second);
                        cerr << "<Coord>:" << coord << endl;
                    }
                    catch(...) {
                        cerr << "<List>" << endl;
                    }
                } else if (mItr->second.IsMap())
                    cerr << "<MAP> " << endl;
                else if (mItr->second.IsFloat())
                    cerr << mItr->second.AsFloat() << endl;
                else if (mItr->second.IsInt())
                    cerr << mItr->second.AsInt() << endl;
                else
                    cerr << "-- missing value --" << endl;

                mItr++;
            }
#endif
            return false;
        }

        return changed;
    } 
		
		////////////////////////////////////////////////////////////

    bool Entity::move(const DII::Object & _moveArg) {
        // has appereance been changed ?
        bool changed = false;
        // have we actually moved ?
        bool moved = false;

        if (!_moveArg.IsMap()) {
#ifdef DEBUG
            cerr << "Entity::move(...) argument is not Map !" << endl;
#endif
            return false;
        }

        try {
            const DII::Object::MapType & moveArg = _moveArg.AsMap();
            DII::Object::MapType::const_iterator itr;

            DII::Object loc, value;

#ifdef DEBUG
            cerr << "Entity: parsing move argument : " << endl;
            DII::Object::MapType::const_iterator mItr =
                _moveArg.AsMap().begin();
            while (mItr != _moveArg.AsMap().end()) {
                cerr << "     " << mItr->first << ": ";
                if (mItr->second.IsString())
                    cerr << mItr->second.AsString() << endl;
                else if (mItr->second.IsList()) {
                    try {
                        Coord coord = objToCoord(mItr->second);
                        cerr << "<Coord>:" << coord << endl;
                    }
                    catch(...) {
                        cerr << "<List>" << endl;
                    }
                } else if (mItr->second.IsMap())
                    cerr << "<MAP> " << endl;
                else if (mItr->second.IsFloat())
                    cerr << mItr->second.AsFloat() << endl;
                else if (mItr->second.IsInt())
                    cerr << mItr->second.AsInt() << endl;
                else
                    cerr << "-- missing value --" << endl;

                mItr++;
            }
#endif

            itr = moveArg.find("stamp");
            if (itr != moveArg.end()) {
                stamp_ = itr->second.AsFloat();
            }

            itr = moveArg.find("loc");
            if (itr != moveArg.end()) {
                string ref = itr->second.AsString();
                if (ref != refID_) {
                    moved = true;
                    // a re-root is seldom, so we stay on the save side
                    // and mark this Entity to have completely changed
                    changed = true;
                    refID_ = ref;
                }
            }

            if ((itr = moveArg.find("velocity")) != moveArg.end()) {
                Coord vel = objToCoord(itr->second);
                // if(fabs(abs(vel-velocity_)) > 0.001)
                // { 
                velocity_ = vel;
                changed = true;
                moved = true;

                /*
                 * a change in velocity does not necessarily involve a
                 * sprite change ... this is checked for when checking for 
                 * a "mode" attribute 
                 */
                // }
            }
            if ((itr = moveArg.find("face")) != moveArg.end()) {
                Coord face = objToCoord(itr->second);

                if (abs(face) > 0.5) {
//                    face_ = face;
                    changed = true;
                    needNewSprite_ = true;

                    int orientation = getIsoOrientation(face);
                    if (orientation != orientation_) {
                        orientation_ = orientation;
                        changed = true;
                        needNewSprite_ = true;
                    }
                }
            }

	    // "orientation" is really the same attribute as "face",
	    // so we need to store it in the same place

            if ((itr = moveArg.find("orientation")) != moveArg.end()) {
                // get the quaterion as a list
                Quaternion orientQuat = objToQuaternion(itr->second);

                // TODO: come up with a condition for rotation: e.g. if
                // incoming quat is signifigantly different than existing
                if (true) {
		    Coord face;
  
		    // When cyphesis sends us a quaternion to represent 
		    // orientation, an unrotated vector is pointing due 
		    // east.  We consider this to be the vector (1, 0, 0)

		    // The following is a simplified quaternion rotation of
		    // the vector (1, 0, 0)
		    face.x = 2 * (orientQuat[3] * orientQuat[3]
			+ orientQuat[0] * orientQuat[0]) - 1;
		    face.y = 2 * (orientQuat[1] * orientQuat[0]
			+ orientQuat[3] * orientQuat[2]);
		    face.z = 0;

//		    face_ = face;

                    changed = true;
                    needNewSprite_ = true;

                    int orientation = getIsoOrientation(face);
                    if (orientation != orientation_) {
                        orientation_ = orientation;
                        changed = true;
                        needNewSprite_ = true;
                    }
                }
            }

            if ((itr = moveArg.find("pos")) != moveArg.end()) {
                Coord pos = objToCoord(itr->second);
                delta_ = pos - position_;
                // if(fabs(abs(delta_)) > 0.001)
                // {
                lastUpdate_.set_now();
                position_ = pos;
                moved = true;
                // }
            }
            // ------- non atlas attributes follow -----------------

            itr = moveArg.find("mode");
            if (itr != moveArg.end()) {
                string mode = itr->second.AsString();
                if (mode != mode_) {
                    mode_ = mode;
                    changed = true;
                    needNewSprite_ = true;
                }
            } else {
                // not mode attribute found .. figure out ourselves
                string mode;
                float speed_ = fabs(abs(velocity_));
                if (speed_ < 0.1)
                    mode = "standing";
                else if (speed_ > 2)
                    mode = "running";
                else
                    mode = "walking";
                if (mode_ != mode) {
                    mode_ = mode;
                    changed = true;
                    needNewSprite_ = true;
                }
            }



            if (moved)
                lastUpdate_.set_now();

            // requestSprite();
        }
        catch(...) {
#ifdef DEBUG
            cerr << "Entity: error in move argument: " << endl;

            DII::Object::MapType::const_iterator mItr =
                _moveArg.AsMap().begin();
            while (mItr != _moveArg.AsMap().end()) {
                cerr << mItr->first << ": ";
                if (mItr->second.IsString())
                    cerr << mItr->second.AsString() << endl;
                else if (mItr->second.IsList()) {
                    try {
                        Coord coord = objToCoord(mItr->second);
                        cerr << "<Coord>:" << coord << endl;
                    }
                    catch(...) {
                        cerr << "<List>" << endl;
                    }
                } else if (mItr->second.IsMap())
                    cerr << "<MAP> " << endl;
                else if (mItr->second.IsFloat())
                    cerr << mItr->second.AsFloat() << endl;
                else if (mItr->second.IsInt())
                    cerr << mItr->second.AsInt() << endl;
                else
                    cerr << "-- missing value --" << endl;

                mItr++;
            }
#endif
            return false;
        }

#ifdef DEBUG
        cerr << "Entity: done." << endl;
#endif

        return changed || moved;
    }

    bool Entity::set(const DII::Object & setArg) {
        // for entity baseclass this is the same
        return sight(setArg);
    }

    bool Entity::stepback() {
        position_ -= delta_;
        delta_ = Coord(0.0, 0.0, 0.0);

        lastUpdate_.set_now();
        return true;
    }

    bool Entity::move(const my_time & theTime) {
        // time since last update in miliseconds
        long int deltaT = theTime() - lastUpdate_();
        bool changed = false;
        //float speed = fabs(abs(velocity_));
        // False speed: this is not the magnitude of the velocity but
				// the norm.  
				float speed = norm(velocity_);

        //if (speed > 0.001) {
				if (speed > 0.0316f) {
            delta_ = velocity_ * (deltaT / 1000.0);
            position_ += delta_;
            changed = true;
        } else {
            delta_ = Coord(0.0, 0.0, 0.0);
        }

        lastUpdate_.set_now();

        return changed;
    }

    Quaternion Entity::orientQuat() const {
        double half_angle = orientation_ * 3.1415927 / 8;
        return Quaternion(0, 0, sin(half_angle), cos(half_angle));
/*
	Vector axis(0, -face_.z, face_.y); // cross((1, 0, 0), face_)

	// stupid interface, abs() for mag and norm() for sqr_mag
	double face_mag = abs(face_);
	double axis_mag = axis.length();

	if(!eqz(axis_mag))
	   return Quaternion(axis, asin(axis_mag/face_mag));
	else if(face_.x > 0) // no rotation
	   return Quaternion(0, 0, 0, 1);
	else // 180 degree rotation, pick z axis at random
	   return Quaternion(0, 0, 1, 0);
*/
    }

    bool Entity::action(const string & action, const my_time & restoreTime) {
        bool changed = false;
        if (action != savedMode_) {
#ifdef DEBUG
            std::cerr << "Entity: " << id_ << " starting action " << action <<
                std::endl;
#endif
            changed = true;
            if (savedMode_.empty())
                savedMode_ = mode_;
            mode_ = action;
        }

        modeRestoreTime_ = restoreTime();

        return changed;
    }

    bool Entity::tick(const my_time & theTime) {
        bool changed = false;
        if (!savedMode_.empty() && theTime() > modeRestoreTime_) {
#ifdef DEBUG
            cerr << "Entity: " << id_ << " ending action " << mode_ <<
                endl;
#endif

            mode_ = savedMode_;
            savedMode_ = "";
            changed = true;
        }
        return changed;
    }
		
    bool Entity::accelerate(float val) {
        float speed_ = fabs(abs(velocity_));
        float speed = speed_;

        speed += val;
        if (speed < 0.0)
            speed = 0.0;
        if (speed > MAX_SPEED)
            speed = MAX_SPEED;

        if (speed_ != speed) {
	    if (speed == 0.0)
		velocity_ = Coord(0.0, 0.0, 0.0);
            else if (speed_ > 0.0) {
                int orientation = getIsoOrientation(velocity_);
                velocity_ = velocity_ * speed / speed_;
                if (orientation != orientation_) {
                    orientation_ = orientation;
                    needNewSprite_ = true;
                }
            }
            else {
                double angle = orientation_ * 3.1415927 / 4;
                velocity_ = Coord(speed * cos(angle), speed * sin(angle), 0);
            }

            speed_ = speed;

            string mode;
            if (speed_ < 0.1)
                mode = "standing";
            else if (speed_ > 2)
                mode = "running";
            else
                mode = "walking";
            if (mode_ != mode) {
                mode_ = mode;
                needNewSprite_ = true;
                // requestSprite();
            }

            lastUpdate_.set_now();
            return true;
        }

        return false;
    }

    bool Entity::halt() {
        if (abs(velocity_) != 0.0) {
            velocity_ = Coord(0, 0, 0);
            if (mode_ != "standing") {
                mode_ = "standing";
                needNewSprite_ = true;
                // requestSprite();
            }

            lastUpdate_.set_now();
            return true;
        }

        return false;
    }

    bool Entity::turn(float val) {
	
        // Rotate the velocity vector anyway
        velocity_.rotateZ(val);

        int new_orientation;

        double speed = fabs(abs(velocity_));
				double offset = 0.5f;

				if (val < 0.0f) {
            offset = -0.5f;
        }

        if(speed > 0.0f) {
            new_orientation = getIsoOrientation(velocity_);
        }
        else {
            // need to round to the nearest integer.  Adding the offset and
            // casting (truncating) to int does this.
            new_orientation = (orientation_ - 
                              (int)((4 * val / 3.1415927) + offset) ) % 8;
            if(new_orientation < 0)
                new_orientation += 8;
        }

        // Only update the sprite if the entity has turned enough to change 
        // the orientation index.
        if (new_orientation != orientation_) {
            orientation_ = new_orientation;
            lastUpdate_.set_now();
            return true;
        }

        return false;
    }

DII::Object Entity::asObject() const {

        DII::Object::MapType obj = attributes_;

        obj["id"] = id_;
        obj["name"] = name_;
        obj["mode"] = mode_;
        obj["description"] = description_;
        obj["stamp"] = stamp_;

        string posStr;
         position_.toString(posStr);
         obj["position"] = posStr;

         DII::Object::ListType parent;
         parent.push_back(parent_);
         obj["parents"] = parent;
         obj["pos"] = coordToObj(position_);
         obj["velocity"] = coordToObj(velocity_);
         obj["orientation"] = quaternionToObj(orientQuat());
//         obj["face"] = coordToObj(face_);
         obj["loc"] = refID_;

         DII::Object::ListType contains;
        Entity *entity;

         LeafMap::const_iterator itr = Tree::leaves().begin();
        while (itr != Tree::leaves().end()) {
            entity = dynamic_cast < Entity * >(itr->second);
            itr++;
            assert(entity);
            contains.push_back(entity->id());
        }
        if (contains.size() > 0)
             obj["contains"] = contains;

        return obj;
    }

std::list<std::string> Entity::contains() const
{
  std::list<std::string> items;

  LeafMap::const_iterator itr = Tree::leaves().begin();
  while (itr != Tree::leaves().end()) {
    Entity *entity = dynamic_cast < Entity * >(itr->second);
    itr++;
    assert(entity);
    items.push_back(entity->id());
  }

  return items;
}

#define PLAY_SAMPLE(sample) { if(SAMPLE_RES(sample)) SAMPLE_RES(sample)->play(); }

    bool Entity::touch(const DII::Object &) {
        if (parent_ == "pig") {
            unsigned index = (unsigned)(drand48() * (double)5);
            switch (index) {
            case 0:
                PLAY_SAMPLE("pig_snort1");
                break;
            case 1:
                PLAY_SAMPLE("pig_snort2");
                break;
            case 2:
                PLAY_SAMPLE("pig_snort3");
                break;
            case 3:
                PLAY_SAMPLE("pig_snort4");
                break;
            case 4:
                PLAY_SAMPLE("pig_snort5");
                break;
            }
        } else if (parent_ == "farmer" || parent_ == "merchant"
                   || parent_ == "guard" || parent_ == "butcher") {
            // a human does not like to be touched:
            unsigned index = (unsigned)(drand48() * (double)3);

            switch (index) {
            case 0:
                PLAY_SAMPLE("human_grumpy1");
                break;
            case 1:
                PLAY_SAMPLE("human_grumpy2");
                break;
            case 2:
                PLAY_SAMPLE("human_grumpy3");
                break;
            }
        } else if (parent_ == "skeleton") {
            // skeletons have a hard life
            PLAY_SAMPLE("skeleton_sigh");
        } else if (parent_ == "wolf") {
            unsigned index = (unsigned)(drand48() * (double)3);
            switch (index) {
            case 0:
                PLAY_SAMPLE("wolf_howling");
                break;
            case 1:
            case 2:
                PLAY_SAMPLE("wolf_snarling");
                break;
            }
        } else {
            // play default: "knockknock"
            PLAY_SAMPLE("knocking");
        }
        // nothing has happened to me
        return false;
    }

}
