#ifndef _WORLD_HEIGHTMAP_H_
#define _WORLD_HEIGHTMAP_H_

namespace World {

/** HeightMap class.
 *  Warning: none of the methods does any parameter checking !
 */
class HeightMap
{
 public:
  ///
  enum { DEFAULT_LEVEL = 0 };
  ///Internally stored height data
  typedef signed short level_t;
  ///
  typedef signed int height_t;
  ///
  HeightMap(int sizex, int sizey, level_t level = DEFAULT_LEVEL) :
    sizeX_(sizex),
    sizeY_(sizey)
    {
      assert(sizeX_ > 0);
      assert(sizeY_ > 0);
      heightMap_ = new level_t[sizeX_ * sizeY_];
      level_t *tmp = heightMap_;

      int n = sizeX_ * sizeY_;
      while(n--)
	*tmp++ = level;

      assert(heightMap_);
    }
  ///
  ~HeightMap()
    {
      delete [] heightMap_;
    }

  ///
  level_t getGroundLevel(int x, int y) 
    { 
      assert(x >= 0 && x < sizeX_);
      assert(y >= 0 && y < sizeY_);
      return heightMap_[x + sizeX_ * y]; 
    };
  ///
  void setGroundLevel(int x, int y, level_t level) 
    {       
      assert(x >= 0 && x < sizeX_);
      assert(y >= 0 && y < sizeY_);
      heightMap_[x + sizeX_ * y] = level; 
    };      
  ///
  const level_t* getGroundLevelMap() const 
    { return heightMap_; };

  ///
  int getSizeX() const { return sizeX_;};
  ///
  int getSizeY() const { return sizeY_;};

 protected:
  ///
  level_t* heightMap_;
  ///
  int sizeX_;
  ///
  int sizeY_;

};

} // namespace

#endif
